--
--  Class NewFrameBorder

local addonData, triggers = ...

local NewFrameBorder = {}
NewFrameBorder.__index = NewFrameBorder

function NewFrameBorder.Create()
	local obj = {}
	setmetatable(obj,NewFrameBorder)
	obj:Init()
	return obj
end

function NewFrameBorder:Init()
	self.items = {}
	self.description = "ConfigNewFrames"
	self.parent = nil
	self.dimAlpha = .5
	self.brightAlpha = 1
end

function NewFrameBorder:CreateNewNotification(access, imageSelect)
	local top = 0
	local left = 0
	local size = self.snapTo:GetWidth()
	if(access == "TOP")then
		top = self.snapTo:GetTop() - size
		left = self.snapTo:GetLeft()
	elseif(access == "BOTTOM")then
		top = self.snapTo:GetTop() + size
		left = self.snapTo:GetLeft()
	elseif(access == "LEFT")then
		top = self.snapTo:GetTop()
		left = self.snapTo:GetLeft() - size
	elseif(access == "RIGHT")then
		top = self.snapTo:GetTop()
		left = self.snapTo:GetLeft() + size
	end
	triggers.Notifications:CreateNewNotification(imageSelect,top, left, size, true)
end

function NewFrameBorder:CreateBorderButton(access,toolTip)
	local b = UI.CreateFrame("Frame", self.description .. access, self.parent)
	b:SetBackgroundColor(1,1,0)
	b:SetAlpha(self.dimAlpha)
	b:SetVisible(false)

	triggers.Tooltip(b,toolTip,40, 300,function(p) p:SetAlpha(triggers.Configuration.Notifications.NewFrameBorder.brightAlpha) end, function(p) p:SetAlpha(triggers.Configuration.Notifications.NewFrameBorder.dimAlpha) end)

	function b.Event:LeftClick()
		triggers.Configuration.Notifications.NewFrameBorder:CreateNewNotification(access, false)
	end

	function b.Event:RightClick()
		triggers.Configuration.Notifications.NewFrameBorder:CreateNewNotification(access, true)
	end

	return b
end

function NewFrameBorder.UISetup(handle,parent)
	local self = triggers.Configuration.Notifications.NewFrameBorder
	self.parent = parent

	self.top = self:CreateBorderButton("TOP","Left-click to add a new ability above.\n Right-click to add a new notification")
	self.bottom = self:CreateBorderButton("BOTTOM","Left-click to add a new ability below.\n Right-click to add a new notification")
	self.left = self:CreateBorderButton("LEFT","Left-click to add a new ability to the left.\n Right-click to add a new notification")
	self.right = self:CreateBorderButton("RIGHT","Left-click to add a new ability to the right.\n Right-click to add a new notification")
end

function NewFrameBorder:Show(visible, snapTo)
	self.snapTo = snapTo
	if(snapTo)then

		self.top:SetPoint("BOTTOMLEFT", snapTo, "TOPLEFT")
		self.top:SetPoint("BOTTOMRIGHT", snapTo, "TOPRIGHT")
		self.top:SetHeight(25)

		self.bottom:SetPoint("TOPLEFT", snapTo, "BOTTOMLEFT")
		self.bottom:SetPoint("TOPRIGHT", snapTo, "BOTTOMRIGHT")
		self.bottom:SetHeight(25)

		self.left:SetPoint("TOPRIGHT", snapTo, "TOPLEFT")
		self.left:SetPoint("BOTTOMRIGHT", snapTo, "BOTTOMLEFT")
		self.left:SetWidth(25)

		self.right:SetPoint("TOPLEFT", snapTo, "TOPRIGHT")
		self.right:SetPoint("BOTTOMLEFT", snapTo, "BOTTOMRIGHT")
		self.right:SetWidth(25)
	end
	self.top:SetVisible(visible)
	self.bottom:SetVisible(visible)
	self.left:SetVisible(visible)
	self.right:SetVisible(visible)
end


triggers.Configuration.Notifications.NewFrameBorder = NewFrameBorder.Create()

Command.Event.Attach(Triggers.Event.ContextCreated , NewFrameBorder.UISetup, "NewFrameBorder.UISetup")