--
-- Class Notification
--
--   Manages a list of Notification

local addonData, triggers = ...
triggers.Configuration = triggers.Configuration or {}
triggers.Configuration.Notifications = triggers.Configuration.Notifications or {}

local Notification = {}
Notification.__index = Notification

function Notification.Create(parent)
	local obj = {}
	setmetatable(obj,Notification)
	obj:Init(parent, notification)
	return obj
end

function Notification:CreateTriggerMatch(myPoint,attachTo,theirPoint,x,y)
	self.frame.triggerMatchWithNoTriggers =  UI.CreateFrame("Frame",self.description .. "TriggerMatchWithNoTriggers", self.parent)
	self.frame.triggerMatchWithNoTriggers:SetLayer(self.parent:GetLayer() + 1)
	self.frame.triggerMatchWithNoTriggers.any = libTUI.ToggleFrame.Create(self.frame.triggerMatchWithNoTriggers
										,"Any Triggers", "Any Triggers"
										,nil
										,182
										,"TOPLEFT", self.frame.triggerMatchWithNoTriggers, "TOPLEFT" ,0 ,0
										,false
										,false
										,libTUI.ToggleFrame.Borders.Left)
	table.insert(self.buttons,self.frame.triggerMatchWithNoTriggers.any)
	triggers.Tooltip(self.frame.triggerMatchWithNoTriggers.any.label,"Displays this notification when the first trigger matches.",30,350)
	self.frame.triggerMatchWithNoTriggers.all = libTUI.ToggleFrame.Create(self.frame.triggerMatchWithNoTriggers
										,"All Triggers", "All Triggers"
										,nil
										,182
										,"TOPLEFT" ,self.frame.triggerMatchWithNoTriggers.any.label ,"TOPRIGHT" ,5 , 0
										,false
										,false
										,libTUI.ToggleFrame.Borders.None)
	table.insert(self.buttons,self.frame.triggerMatchWithNoTriggers.all)
	triggers.Tooltip(self.frame.triggerMatchWithNoTriggers.all.label,"All triggers must match to display this notification.",30,330)
	self.frame.triggerMatchWithNoTriggers.none = libTUI.ToggleFrame.Create(self.frame.triggerMatchWithNoTriggers
										,"No Triggers", "No Triggers"
										,nil
										,182
										,"TOPLEFT" ,self.frame.triggerMatchWithNoTriggers.all.label ,"TOPRIGHT" ,5 , 0
										,false
										,false
										,libTUI.ToggleFrame.Borders.Right)
	table.insert(self.buttons,self.frame.triggerMatchWithNoTriggers.none)
	triggers.Tooltip(self.frame.triggerMatchWithNoTriggers.none.label,"Hides the notification on the first matched trigger.",30,320)
	self.frame.triggerMatchWithNoTriggers:SetHeight(30)
	self.frame.triggerMatchWithNoTriggers:SetWidth(400)
	self.frame.triggerMatchWithNoTriggers:SetPoint(myPoint,attachTo,theirPoint,x,y)


	self.frame.triggerMatchWithNoTriggers.any.onChanged = function()
		self.frame.triggerMatchWithNoTriggers.none:Enable(false)
		self.frame.triggerMatchWithNoTriggers.all:Enable(false)
	end
	self.frame.triggerMatchWithNoTriggers.all.onChanged = function()
		self.frame.triggerMatchWithNoTriggers.none:Enable(false)
		self.frame.triggerMatchWithNoTriggers.any:Enable(false)
	end
	self.frame.triggerMatchWithNoTriggers.none.onChanged = function()
		self.frame.triggerMatchWithNoTriggers.any:Enable(false)
		self.frame.triggerMatchWithNoTriggers.all:Enable(false)
	end

	self.frame.triggerMatchWithoutNoTriggers =  UI.CreateFrame("Frame",self.description .. "triggerMatchWithoutNoTriggers", self.parent)
	self.frame.triggerMatchWithoutNoTriggers:SetLayer(self.parent:GetLayer() + 1)
	self.frame.triggerMatchWithoutNoTriggers.any = libTUI.ToggleFrame.Create(self.frame.triggerMatchWithoutNoTriggers
										,"Any Triggers", "Any Triggers"
										,nil
										,273
										,"TOPLEFT", self.frame.triggerMatchWithoutNoTriggers, "TOPLEFT" ,0 ,0
										,false
										,false
										,libTUI.ToggleFrame.Borders.Left)
	table.insert(self.buttons,self.frame.triggerMatchWithoutNoTriggers.any)
	triggers.Tooltip(self.frame.triggerMatchWithoutNoTriggers.any.label,"Displays this notification when the first trigger matches.",30,350)
	self.frame.triggerMatchWithoutNoTriggers.all = libTUI.ToggleFrame.Create(self.frame.triggerMatchWithoutNoTriggers
										,"All Triggers", "All Triggers"
										,nil
										,273
										,"TOPLEFT" ,self.frame.triggerMatchWithoutNoTriggers.any.label ,"TOPRIGHT" ,5 , 0
										,false
										,false
										,libTUI.ToggleFrame.Borders.Right)
	table.insert(self.buttons,self.frame.triggerMatchWithoutNoTriggers.all)
	triggers.Tooltip(self.frame.triggerMatchWithoutNoTriggers.all.label,"All triggers must match to display this notification.",30,330)
	self.frame.triggerMatchWithoutNoTriggers:SetHeight(30)
	self.frame.triggerMatchWithoutNoTriggers:SetWidth(400)
	self.frame.triggerMatchWithoutNoTriggers:SetPoint(myPoint,attachTo,theirPoint,x,y)

	self.frame.triggerMatchWithoutNoTriggers.any.onChanged = function()
		self.frame.triggerMatchWithoutNoTriggers.all:Enable(false)
	end
	self.frame.triggerMatchWithoutNoTriggers.all.onChanged = function()
		self.frame.triggerMatchWithoutNoTriggers.any:Enable(false)
	end

	self.frame.rolesListScrollView = UI.CreateFrame("SimpleScrollView", self.description .. "_RoleList",self.frame)
	self.frame.rolesListScrollView:SetPoint("TOPRIGHT", self.frame, "TOPRIGHT",-10,0)
	self.frame.rolesListScrollView:SetWidth(170)
	self.frame.rolesListScrollView:SetHeight(100)

	self.frame.roleList = UI.CreateFrame("SimpleList", self.description .. "_AbilityList_SimpleList", self.frame.rolesListScrollView)
	self.frame.roleList:SetFontSize(14)
	self.frame.rolesListScrollView:SetContent(self.frame.roleList)
end

function Notification:Init(parent)
	self.parent = parent
	self.size = { width = self.parent:GetWidth(), height = 140 }
	self.description = "Notification"
	self.frame = UI.CreateFrame("Frame",self.description, self.parent)
	self.frame:SetVisible(true)
	self.frame:SetLayer(self.parent:GetLayer() + 1)
	self.frame:SetHeight(self.size.height)
	self.frame:SetWidth(self.size.width)
	self.frame:SetBackgroundColor(0,0,0,.3)
	self.frame.inputs = {}
	self.buttons = {}

	self.frame.anchor = UI.CreateFrame("Texture",self.description .. "Anchor", self.parent)
	self.frame.anchor:SetHeight(13)
	self.frame.anchor:SetWidth(25)
	self.frame.anchor:SetPoint("TOPLEFT",self.frame,"TOPLEFT",5,15)
	self.frame.anchor:SetTexture(triggers.addonId, "\\images\\anchorUp.png")

	self.frame.combatOnly = libTUI.ToggleFrame.Create(self.frame
										,"Combat Only", "Always"
										,nil
										,110
										,"BOTTOMLEFT" ,self.frame.anchor ,"BOTTOMRIGHT" ,20 ,0
										,false
										,false
										,libTUI.ToggleFrame.Borders.All
										,{1,1,1,1})
	table.insert(self.buttons,self.frame.combatOnly)
	triggers.Tooltip(self.frame.combatOnly.label,"Display this notification in combat only.",30,300)

	self.frame.inputs.description = libTUI.Input(self.frame
												,self.description .. "_staticText"
												,"Description" ,90
												,"TOPLEFT" ,self.frame.combatOnly.label ,"TOPRIGHT" ,20 ,0
												,440
												,"")
	triggers.Tooltip(self.frame.inputs.description.label,"Describe this notification to be displayed in the Sharing Center",30,450)

	self.frame.description = UI.CreateFrame("Text", self.description .. "Text", self.frame)
	self.frame.description:SetLayer(self.frame:GetLayer() + 1)
	self.frame.description:SetPoint("TOPLEFT", self.frame.combatOnly.label, "TOPRIGHT",20,0)
	self.frame.description:SetFontColor(0,1,1,1)
	self.frame.description:SetFontSize(16)
	self.frame.description:SetWidth(520)

	self.frame.middleSection = UI.CreateFrame("Frame",self.description .. "middleSection", self.frame)
	self.frame.middleSection:SetLayer(self.frame:GetLayer() + 1)
	self.frame.middleSection:SetHeight(60)
	self.frame.middleSection:SetWidth(690)
	self.frame.middleSection:SetVisible(true)
	self.frame.middleSection:SetPoint("TOPLEFT", self.frame.combatOnly.label, "BOTTOMLEFT",-30,10)
	self.frame.middleSection.inputs = {}
	self.frame.middleSection.inputs.size = libTUI.Input(self.frame.middleSection
												,self.description .. "_staticText"
												,"Size" ,50
												,"TOPLEFT" ,self.frame.middleSection ,"TOPLEFT" ,0 ,0
												,70
												,"")
	triggers.Tooltip(self.frame.middleSection.inputs.size.label,"Size of this notifications image.",30)

	self.frame.middleSection.inputs.layer = libTUI.Input(self.frame.middleSection
												,self.description .. "_staticText"
												,"Layer" ,50
												,"TOPLEFT" ,self.frame.middleSection.inputs.size ,"TOPRIGHT" ,20 ,0
												,70
												,"")
	triggers.Tooltip(self.frame.middleSection.inputs.layer.label,"Sets the layer of this notification. Bottom = 1",30)


	self.frame.middleSection.inputs.staticText = libTUI.Input(self.frame.middleSection
												,self.description .. "_staticText"
												,"Static Text" ,80
												,"TOPLEFT" ,self.frame.middleSection.inputs.layer ,"TOPRIGHT" ,20 , 0
												,200
												,"")
	triggers.Tooltip(self.frame.middleSection.inputs.staticText.label,"This notification will always display this text\nunless a triggers text overrides it.",40)

	self.frame.middleSection.staticTextColor = UI.CreateFrame("Frame",self.description .. "_staticTextColor", self.frame.middleSection)
	self.frame.middleSection.staticTextColor:SetWidth(20)
	self.frame.middleSection.staticTextColor:SetHeight(20)
	self.frame.middleSection.staticTextColor:SetLayer(101)
	self.frame.middleSection.staticTextColor:SetVisible(true)
	self.frame.middleSection.staticTextColor:SetPoint("TOPLEFT", self.frame.middleSection.inputs.staticText.background , "TOPRIGHT" ,8 ,3)
	self.frame.middleSection.staticTextColor:SetBackgroundColor(1,1,1, 1)
	self.frame.middleSection.staticTextColor.color = {1, 1, 1}

	self.frame.middleSection.inputs.top = libTUI.Input(self.frame.middleSection
												,self.description .. "_staticText"
												,"Top" ,50
												,"TOPLEFT" ,self.frame.middleSection.inputs.size.label ,"BOTTOMLEFT" ,0 ,5
												,70
												,"")
	triggers.Tooltip(self.frame.middleSection.inputs.top.label,"Location of this notifications top edge.",30)

	self.frame.middleSection.inputs.left = libTUI.Input(self.frame.middleSection
												,self.description .. "_staticText"
												,"Left" ,50
												,"TOPLEFT" ,self.frame.middleSection.inputs.top ,"TOPRIGHT" ,20 ,0
												,70
												,"")
	triggers.Tooltip(self.frame.middleSection.inputs.left.label,"Location of this notifications left edge.",30)

	self.frame.middleSection.opacity = libTUI.Slider(self.frame.middleSection
												, self.description .. "_opacity"
												,"Opacity"
												,60
												,"TOPLEFT"
												,self.frame.middleSection.inputs.left
												,"TOPRIGHT"
												,40
												,0
												,150
												,100)
	triggers.Tooltip(self.frame.middleSection.opacity.input.label,"The opacity of ths notifications image.",30)

	self.frame.middleSection.changeImage = libTUI.Button(self.frame.middleSection
											,triggers.addonId
											,self.description .. "_ChangeImage"
											,{width = 80, height=25}
											,"TOPLEFT"
											,self.frame.middleSection.staticTextColor
											,"TOPRIGHT"
											,30
											,0
											,"\\images\\image.png"
											,"\\images\\imageOver.png"
											,false)

	self:CreateTriggerMatch("BOTTOMLEFT",self.frame,"BOTTOMLEFT",10,0)
	self.frame.triggerMatchWithNoTriggers:SetLayer(self.frame:GetLayer() + 1)
	self.frame.triggerMatchWithoutNoTriggers:SetLayer(self.frame:GetLayer() + 1)

	self.frame.inputs.roles = libTUI.Input(self.frame
												,self.description .. "_staticText"
												,"Specs" ,50
												,"BOTTOMRIGHT" ,self.frame ,"BOTTOMRIGHT" ,-250 ,-5
												,240
												,"")
	triggers.Tooltip(self.frame.inputs.roles.label,"Filters this notification based on your spec.\nThe value is a comma delimited list\nPartial matches are used.",70)

	self.frame.anchor.Event.LeftClick = function ()
		self.down = not self.down
		self:SetGrid()
	end

	self.frame.middleSection.opacity.input.Event.TextfieldChange = function ()
		local pos = self.frame.middleSection.opacity.input:GetText()
		pos = tonumber(pos) or 1
		if(pos > 100)then
			pos = 100
		end
		if(pos <=0)then
			pos = 0
		end
		self.frame.middleSection.opacity:SetPosition(pos)
		if(self.callback)then
			self.callback()
		end
	end

	self.frame.middleSection.staticTextColor.Event.LeftClick = function()
		local callback = function(r, g, b)
			self.frame.middleSection.staticTextColor:SetBackgroundColor(r,g,b,1)
			if(self.callback)then
				self.callback()
			end
		end
		triggers.ColorPicker.Display(callback,"TOPRIGHT",self.frame.middleSection.staticTextColor,"BOTTOMRIGHT",0,0)
	end
end

function Notification:SetGrid()
	if(self.down)then
		self.frame.anchor:SetTexture(triggers.addonId,"\\images\\anchorUp.png")
		self.frame.middleSection:SetVisible(true)
		self.frame:SetHeight(self.size.height)
		self.frame.inputs.description.background:SetWidth(440)
		self.frame.inputs.description:SetWidth(430)
		self.frame.rolesListScrollView:SetVisible(true)
	else
		self.frame.anchor:SetTexture(triggers.addonId,"\\images\\anchorDown.png")
		self.frame.middleSection:SetVisible(false)
		self.frame:SetHeight(70)
		self.frame.inputs.description.background:SetWidth(620)
		self.frame.inputs.description:SetWidth(610)
		self.frame.rolesListScrollView:SetVisible(false)
	end
	if(self.toggles.notification)then
		self.frame.triggerMatch = self.frame.triggerMatchWithNoTriggers
		self.frame.triggerMatchWithoutNoTriggers:SetVisible(false)
		self.frame.triggerMatchWithNoTriggers:SetVisible(true)
	else
		self.frame.triggerMatch = self.frame.triggerMatchWithoutNoTriggers
		self.frame.triggerMatchWithoutNoTriggers:SetVisible(true)
		self.frame.triggerMatchWithNoTriggers:SetVisible(false)
	end

	self.frame.triggerMatch.any:Enable(self.toggles.triggers==triggers.DisplayIf.ANY)
	self.frame.triggerMatch.all:Enable(self.toggles.triggers==triggers.DisplayIf.ALL)
	if(self.frame.triggerMatch.none)then
		self.frame.triggerMatch.none:Enable(self.toggles.triggers==triggers.DisplayIf.NONE)
	end
	self.frame.combatOnly:Enable(self.toggles.combatOnly)
end

function Notification:Set(n,callback)
	self.callback = callback
	self.toggles = {}
	self.toggles.triggers = n.displayIf
	self.toggles.notification = not n.abilityIcon
	self.toggles.combatOnly = n.combatOnly

	self.frame.middleSection.inputs.size:SetText(tostring(n.size))
	self.frame.middleSection.inputs.layer:SetText(tostring(n.layer))
	self.frame.middleSection.inputs.top:SetText(tostring(n.position.top))
	self.frame.middleSection.inputs.left:SetText(tostring(n.position.left))
	self.frame.middleSection.inputs.staticText:SetText(tostring(n.text or ""))
	self.frame.inputs.description:SetText(tostring(n.description or ""))
	self.frame.description:SetText(tostring(n.description or ""))
	self.frame.middleSection.opacity.input:SetText(tostring(n.opacity or ""))
	self.frame.middleSection.opacity:SetPosition(n.opacity or 100)
	self.frame.middleSection.staticTextColor:SetBackgroundColor(n.color[1],n.color[2],n.color[3],1)
	self.frame.inputs.roles:SetText(n.roles or "")

	self.frame.middleSection.changeImage:SetVisible(self.toggles.notification)

	self.frame.inputs.description:SetVisible(n.deletable)
	self.frame.inputs.description.label:SetVisible(n.deletable)
	self.frame.inputs.description.background:SetVisible(n.deletable)
	self.frame.description:SetVisible(not n.deletable)
	self:SetGrid()

	for name, item in pairs(self.frame.middleSection.inputs)do
		item.Event.TextfieldChange = function()
			callback()
		end
	end

	for name, item in pairs(self.frame.inputs)do
		item.Event.TextfieldChange = function()
			callback()
		end
	end

	for _, item in ipairs (self.buttons)do
		item.Changed = function()
			callback()
		end
	end

	self.frame.middleSection.opacity.input.Event.TextfieldChange = function()
		callback()
	end
	self.frame.middleSection.opacity.Event.SliderChange = function ()
		local pos = self.frame.middleSection.opacity:GetPosition()
		self.frame.middleSection.opacity.input:SetText(tostring(pos))
		callback()
	end

	self.frame.middleSection.changeImage.Event.LeftClick = function()
		triggers.ImageScroller:Show(true)
		triggers.ImageScroller.ImageSelected = function (_, resource,image)
			if(resource ~= nil and image ~= nil)then
				n.image = resource .. "|" .. image
				n:UpdateImage()
			else
				n.image = nil
				n:UpdateImage()
			end
		end
	end

	local items = {}
	local roles = Inspect.Role.List()
	local sortedRoles = {}
	for roleId, name in pairs(roles) do
		table.insert(sortedRoles,{roleId=roleId, name=name})
	end

	local sort_func = function( a,b ) return a.name < b.name end
	table.sort( sortedRoles, sort_func )

	for i, v in ipairs(sortedRoles) do
		table.insert(items, v.name)
	end
	self.frame.roleList:SetItems(items)
	self.frame.roleList.Event.ItemClick = function(item,_,_)
		local s = self.frame.inputs.roles:GetText()
		s = s .. "," .. item
		self.frame.inputs.roles:SetText(s)
		self.frame.roleList:SetSelectedItem(nil,true)
	end
end

function Notification:UpdateSizeAndPosition(size, position)
	self.frame.middleSection.inputs.size:SetText(tostring(size))
	self.frame.middleSection.inputs.top:SetText(tostring(position.top))
	self.frame.middleSection.inputs.left:SetText(tostring(position.left))
end

function Notification:Update(n)
	n.size = tonumber(self.frame.middleSection.inputs.size:GetText())
	if(n.size < 40)then
		n.size = 40
	end
	if(n.size > 500)then
		n.size = 500
	end
	n.layer = tonumber(self.frame.middleSection.inputs.layer:GetText())
	n.position.top = tonumber(self.frame.middleSection.inputs.top:GetText())
	n.position.left = tonumber(self.frame.middleSection.inputs.left:GetText())
	local t = self.frame.middleSection.inputs.staticText:GetText()
	if(string.len(t) > 0)then
		n.text = t
	else
		n.text = nil
	end
	t = self.frame.inputs.description:GetText()
	if(string.len(t) > 0)then
		n.description = t
	else
		n.description = nil
	end
	n.opacity = tonumber(self.frame.middleSection.opacity.input:GetText())
	n.frame.texture:SetAlpha(n.opacity *.01)

	if(self.frame.triggerMatch.any.enabled)then
		n.displayIf = triggers.DisplayIf.ANY
	elseif(self.frame.triggerMatch.all.enabled)then
		n.displayIf = triggers.DisplayIf.ALL
	else
		n.displayIf = triggers.DisplayIf.NONE
	end
	local r,g,b,a = self.frame.middleSection.staticTextColor:GetBackgroundColor()
	n.color = { r, g, b }

	n.roles = self.frame.inputs.roles:GetText()
end

function Notification:Close()
	for name, item in pairs(self.frame.middleSection.inputs)do
		item:SetKeyFocus(false)
	end

	for name, item in pairs(self.frame.inputs)do
		item:SetKeyFocus(false)
	end

	self.frame.middleSection.opacity.input:SetKeyFocus(false)

end

triggers.Configuration.Notifications.Notification = Notification
