--
-- Class Trigger
--
--   Manages a list of Trigger

local addonData, triggers = ...
triggers.Configuration = triggers.Configuration or {}
triggers.Configuration.Notifications = triggers.Configuration.Notifications or {}

local Trigger = {}
Trigger.__index = Trigger

function Trigger.Create(parent, listBox)
	local obj = {}
	setmetatable(obj,Trigger)
	obj:Init(parent,listBox)
	return obj
end

function Trigger:Init(parent,listBox)
	self.parent = parent
	self.size = { width = parent:GetWidth(), height = 190 }
	self.description = "Trigger"
	self.frame = UI.CreateFrame("Texture",self.description, self.parent)
	self.frame:SetVisible(true)
	self.frame:SetLayer(parent:GetLayer() + 1)
	self.frame:SetHeight(self.size.height)
	self.frame:SetWidth(self.size.width)

	self.frame:SetTexture(triggers.addonId,"\\images\\panelTexture.png")

	self.frame.lTrigger = libTUI.Label( self.frame
										,"label"
										,"Trigger Type"
										,300
										,"TOPLEFT"
										,self.frame
										,"TOPLEFT"
										,10
										,0)

	self.frame.casting = libTUI.ToggleFrame.Create(self.frame
										,"Casting", "Casting"
										,nil
										,90
										,"TOPLEFT", self.frame.lTrigger, "BOTTOMLEFT" ,0 ,-5
										,false
										,false
										,libTUI.ToggleFrame.Borders.Left)
	triggers.Tooltip(self.frame.casting.label,"Create a trigger that monitors castbars.",30,280)
	self.frame.casting:Enable(true)

	self.frame.buff = libTUI.ToggleFrame.Create(self.frame
										,"Buff", "Buff"
										,nil
										,90
										,"TOPLEFT", self.frame.casting.label , "TOPRIGHT" ,5 ,0
										,false
										,false
										,libTUI.ToggleFrame.Borders.None)
	triggers.Tooltip(self.frame.buff.label,"Create a trigger to monitor buffs.",30,280)

	self.frame.ability = libTUI.ToggleFrame.Create(self.frame
										,"Ability", "Ability"
										,nil
										,90
										,"TOPLEFT", self.frame.buff.label , "TOPRIGHT" ,5 ,0
										,false
										,false
										,libTUI.ToggleFrame.Borders.Right)
	triggers.Tooltip(self.frame.ability.label,"Create a trigger for cooldowns of abilities.",30,280)

	self.frame.inputs = {}
	self.frame.inputs.skillName = libTUI.Input(self.frame
												,self.description .. "_staticText"
												,"Skill / Ability" , 200
												,"TOPLEFT" ,self.frame.lTrigger ,"TOPRIGHT" ,5 , 0
												,280
												,"",true)
	triggers.Tooltip(self.frame.inputs.skillName.label,"The name of the ability/buff/cast.",30,280)

	self.frame.lApplies = libTUI.Label( self.frame
										,"label"
										,"Applies to"
										,250
										,"TOPRIGHT"
										,self.frame
										,"TOPRIGHT"
										,-20
										,0)

	self.frame.self = libTUI.ToggleFrame.Create(self.frame
										,"Yourself", "Your"
										,nil
										,80
										,"TOPLEFT", self.frame.lApplies , "BOTTOMLEFT" , 0 , -5
										,false
										,false
										,libTUI.ToggleFrame.Borders.All
										,{1,1,1,1})
	triggers.Tooltip(self.frame.self.label,"Toggles between yourself and not.",30,280)

	self.frame.targetHostility = UI.CreateFrame("Frame",self.description .. "middleSection", self.frame)
	self.frame.targetHostility:SetLayer(self.frame:GetLayer() + 1)
	self.frame.targetHostility:SetHeight(26)
	self.frame.targetHostility:SetWidth(170)
	self.frame.targetHostility:SetVisible(true)

	self.frame.targetHostility:SetPoint("TOPLEFT", self.frame.self.label, "TOPRIGHT", 15,0)

	self.frame.targetHostility.hostile = libTUI.ToggleFrame.Create(self.frame.targetHostility
										,"Enemy", "Friendly"
										,nil
										,80
										,"TOPLEFT", self.frame.targetHostility, "TOPLEFT" ,0 , 0
										,false
										,false
										,libTUI.ToggleFrame.Borders.Left
										,{1,1,1,1})
	triggers.Tooltip(self.frame.targetHostility.hostile.label,"When your target or focus is a friendly or hostile.",30,350)

	self.frame.targetHostility.target = libTUI.ToggleFrame.Create(self.frame.targetHostility
										,"Target", "Focus"
										,nil
										,80
										,"TOPLEFT", self.frame.targetHostility.hostile.label, "TOPRIGHT" ,5 , 0
										,false
										,false
										,libTUI.ToggleFrame.Borders.Right
										,{1,1,1,1})
	triggers.Tooltip(self.frame.targetHostility.target.label,"Specifies this trigger applies to your target or focus",30,350)


	self.frame.self.onChanged = function()
		self.frame.targetHostility:SetVisible(not self.frame.self.enabled)
	end

	self.frame.inputs.encounter = libTUI.Input(self.frame
												,self.description .. "_staticText"
												,"Encounter" ,75
												,"TOPLEFT" ,self.frame.self.label ,"BOTTOMLEFT" ,-105 , 10
												,290
												,"")
	triggers.Tooltip(self.frame.inputs.encounter.label,"Select an encounter to filter this trigger for specific encounter related mechanics.",30,550)



	self.frame.buffGroup =  UI.CreateFrame("Frame",self.description .. "buffGroup", self.parent)
	self.frame.buffGroup:SetLayer(self.frame:GetLayer() + 1)
	self.frame.buffGroup:SetPoint("TOPLEFT", self.frame.lTrigger,"BOTTOMLEFT", 0, 30)

	self.frame.buffGroup.selfCast = libTUI.ToggleFrame.Create(self.frame.buffGroup
										,"Self Cast", "Casted by Anyone"
										,nil
										,170
										,"TOPLEFT", self.frame.buffGroup , "TOPLEFT" ,30 , 0
										,false
										,false
										,libTUI.ToggleFrame.Borders.All
										,{1,1,1,1})
	triggers.Tooltip(self.frame.buffGroup.selfCast.label,"Toggles between selfcasted or any others casted.",30,320)

	self.frame.buffGroup.active = libTUI.ToggleFrame.Create(self.frame.buffGroup
										,"Active", "Missing"
										,nil
										,120
										,"TOPLEFT", self.frame.buffGroup.selfCast.label , "TOPRIGHT" ,40 ,0
										,false
										,false
										,libTUI.ToggleFrame.Borders.All
										,{1,1,1,1})
	triggers.Tooltip(self.frame.buffGroup.active.label,"Toggles between active and missing buffs.",30,320)

	self.frame.abilityGroup =  UI.CreateFrame("Frame",self.description .. "AbilityGroup", self.parent)
	self.frame.abilityGroup:SetLayer(self.frame:GetLayer() + 1)
	self.frame.abilityGroup:SetPoint("TOPLEFT", self.frame.lTrigger,"BOTTOMLEFT", 0, 30)
	self.frame.abilityGroup:SetHeight(30)
	self.frame.abilityGroup:SetWidth(550)

	self.frame.abilityGroup.filtered = libTUI.ToggleFrame.Create(self.frame.abilityGroup
										,"Filtered", "Not Filtered"
										,nil
										,170
										,"TOPLEFT", self.frame.abilityGroup , "TOPLEFT" ,30 ,0
										,false
										,false
										,libTUI.ToggleFrame.Borders.All
										,{1,1,1,1})
	triggers.Tooltip(self.frame.abilityGroup.filtered.label,"This trigger will be filtered if this ability\nis not found when you change your spec.",40,280)

	self.frame.abilityGroup.ready = libTUI.ToggleFrame.Create(self.frame.abilityGroup
										,"Ready", "On Cooldown"
										,nil
										,120
										,"TOPLEFT", self.frame.abilityGroup.filtered.label , "TOPRIGHT" ,40 ,0
										,false
										,false
										,libTUI.ToggleFrame.Borders.All
										,{1,1,1,1})
	triggers.Tooltip(self.frame.abilityGroup.ready.label,"Toggles between On Cooldown and Ready.",30,280)

	self.frame.buffGroup:SetVisible(false)
	self.frame.abilityGroup:SetVisible(false)
	self.frame.casting.onChanged = function()
		self.frame.buff:Enable(false)
		self.frame.ability:Enable(false)
		self.frame.buffGroup:SetVisible(false)
		self.frame.abilityGroup:SetVisible(false)
		self.frame.timer:SetVisible(false)
		self.frame.timerSection:SetVisible(false)
		self.frame.stacksSection:SetVisible(false)
		self.frame.stacks:SetVisible(false)
	end
	self.frame.buff.onChanged = function()
		self.frame.casting:Enable(false)
		self.frame.ability:Enable(false)
		self.frame.buffGroup:SetVisible(true)
		self.frame.abilityGroup:SetVisible(false)
		self.frame.timer:SetVisible(true)
		self.frame.timerSection:SetVisible(false)
		self.frame.stacksSection:SetVisible(false)
		self.frame.stacks:SetVisible(true)
	end
	self.frame.ability.onChanged = function()
		self.frame.casting:Enable(false)
		self.frame.buff:Enable(false)
		self.frame.buffGroup:SetVisible(false)
		self.frame.abilityGroup:SetVisible(true)
		self.frame.timer:SetVisible(true)
		self.frame.timerSection:SetVisible(false)
		self.frame.stacksSection:SetVisible(false)
		self.frame.stacks:SetVisible(false)
	end

	self.frame.anchor = UI.CreateFrame("Texture",self.description .. "Anchor", self.parent)
	self.frame.anchor:SetHeight(13)
	self.frame.anchor:SetWidth(25)
	self.frame.anchor:SetPoint("TOPLEFT",self.frame.abilityGroup,"BOTTOMLEFT",0,5)
	self.frame.anchor:SetTexture(triggers.addonId, "\\images\\anchorUp.png")

	self.frame.timerSection = UI.CreateFrame("Frame",self.description .. "middleSection", self.parent)
	self.frame.timer = libTUI.ToggleFrame.Create(self.frame
										,"Timer", "Timer"
										,self.frame.timerSection
										,60
										,"TOPLEFT", self.frame.abilityGroup , "BOTTOMLEFT" ,40 , 5
										,false
										,false
										,libTUI.ToggleFrame.Borders.Left)
	triggers.Tooltip(self.frame.timer.label,"Enables checking the timer.",30,280)

	self.frame.timerSection:SetLayer(self.frame:GetLayer() + 1)
	self.frame.timerSection:SetHeight(25)
	self.frame.timerSection:SetWidth(200)
	self.frame.timerSection.inputs = {}

	self.frame.timerSection.inputs.timerValue = libTUI.Input(self.frame.timerSection
												,self.description .. "_staticText"
												,nil ,0
												,"TOPLEFT" ,self.frame.timerSection ,"TOPLEFT" , 5 ,2
												,40
												,"")

	self.frame.timerSection.timerLessThan = libTUI.ToggleFrame.Create(self.frame.timerSection
										,"Less-than equal", "Greater-than equal"
										,nil
										,150
										,"TOPRIGHT", self.frame.timerSection , "TOPRIGHT" ,0 , 0
										,false
										,false
										,libTUI.ToggleFrame.Borders.Right
										,{1,1,1,1})
	triggers.Tooltip(self.frame.timerSection.timerLessThan.label,"Evalues the value greater-than or less-than.",30,280)

	self.frame.stacksSection = UI.CreateFrame("Frame",self.description .. "middleSection", self.parent)
	self.frame.stacks = libTUI.ToggleFrame.Create(self.frame
										,"Stacks", "Stacks"
										,self.frame.stacksSection
										,60
										,"TOPLEFT", self.frame.timerSection , "TOPRIGHT" ,20 , 0
										,false
										,false
										,libTUI.ToggleFrame.Borders.Left)
	triggers.Tooltip(self.frame.stacks.label,"Enables checking the stacks.",30,280)

	self.frame.stacksSection:SetLayer(self.frame:GetLayer() + 1)
	self.frame.stacksSection:SetHeight(25)
	self.frame.stacksSection:SetWidth(200)
	self.frame.stacksSection.inputs = {}

	self.frame.stacksSection.inputs.stacksValue = libTUI.Input(self.frame.stacksSection
												,self.description .. "_staticText"
												,nil ,0
												,"TOPLEFT" ,self.frame.stacksSection ,"TOPLEFT" ,5 ,2
												,40
												,"")

	self.frame.stacksSection.stacksLessThan = libTUI.ToggleFrame.Create(self.frame.stacksSection
										,"Less-than equal", "Greater-than equal"
										,nil
										,150
										,"TOPRIGHT", self.frame.stacksSection , "TOPRIGHT" ,0,0
										,false
										,false
										,libTUI.ToggleFrame.Borders.Right
										,{1,1,1,1})
	triggers.Tooltip(self.frame.stacksSection.stacksLessThan.label,"Evalues the value greater-than or less-than.",30,280)

	self.frame.middleSection = UI.CreateFrame("Frame",self.description .. "middleSection", self.parent)
	self.frame.middleSection:SetLayer(self.frame:GetLayer() + 1)
	self.frame.middleSection:SetHeight(75)
	self.frame.middleSection:SetWidth(self.frame:GetWidth())
	self.frame.middleSection:SetVisible(true)
	self.frame.middleSection:SetPoint("TOPLEFT", self.frame, "TOPLEFT", 0,90)
	self.frame.middleSection.inputs = {}

	self.frame.middleSection.lJob = libTUI.Label( self.frame.middleSection
										,"label"
										,"Raid Job"
										,350
										,"BOTTOMLEFT"
										,self.frame.middleSection
										,"BOTTOMLEFT"
										,20
										,-15)

	self.frame.middleSection.tank = libTUI.ToggleFrame.Create(self.frame.middleSection
										,"Tank", "Tank"
										,nil
										,60
										,"TOPLEFT", self.frame.middleSection.lJob, "BOTTOMLEFT" ,0 , -5
										,false
										,false
										,libTUI.ToggleFrame.Borders.Left)
	triggers.Tooltip(self.frame.middleSection.tank.label,"Filters this trigger the raid icon Tank.",30,280)
	self.frame.middleSection.tank:Enable(true)

	self.frame.middleSection.heals = libTUI.ToggleFrame.Create(self.frame.middleSection
										,"Healer", "Healer"
										,nil
										,60
										,"TOPLEFT", self.frame.middleSection.tank.label, "TOPRIGHT" , 5 , 0
										,false
										,false
										,libTUI.ToggleFrame.Borders.None)
	triggers.Tooltip(self.frame.middleSection.heals.label,"Filters this trigger the raid icon Heals.",30,280)
	self.frame.middleSection.heals:Enable(true)

	self.frame.middleSection.dps = libTUI.ToggleFrame.Create(self.frame.middleSection
										,"DPS", "DPS"
										,nil
										,60
										,"TOPLEFT", self.frame.middleSection.heals.label, "TOPRIGHT" , 5 , 0
										,false
										,false
										,libTUI.ToggleFrame.Borders.None)
	triggers.Tooltip(self.frame.middleSection.dps.label,"Filters this trigger the raid icon DPS.",30,280)
	self.frame.middleSection.dps:Enable(true)

	self.frame.middleSection.support = libTUI.ToggleFrame.Create(self.frame.middleSection
										,"Support", "Support"
										,nil
										,70
										,"TOPLEFT", self.frame.middleSection.dps.label, "TOPRIGHT" , 5 , 0
										,false
										,false
										,libTUI.ToggleFrame.Borders.Right)
	triggers.Tooltip(self.frame.middleSection.support.label,"Filters this trigger the raid icon Support.",30,280)
	self.frame.middleSection.support:Enable(true)

	self.frame.middleSection.inputs.staticText = libTUI.Input(self.frame.middleSection
												,self.description .. "_staticText"
												,"Text" ,40
												,"TOPRIGHT" ,self.frame.middleSection ,"TOPRIGHT" ,-300 , 0
												,200
												,"")
	triggers.Tooltip(self.frame.middleSection.inputs.staticText.label,"This text will be displayed when this trigger matches.\n It will override the notifications text.",40,350)

	self.frame.middleSection.staticTextColor = UI.CreateFrame("Frame",self.description .. "_staticTextColor", self.frame.middleSection)
	self.frame.middleSection.staticTextColor:SetWidth(20)
	self.frame.middleSection.staticTextColor:SetHeight(20)
	self.frame.middleSection.staticTextColor:SetLayer(101)
	self.frame.middleSection.staticTextColor:SetVisible(true)
	self.frame.middleSection.staticTextColor:SetPoint("TOPLEFT", self.frame.middleSection.inputs.staticText.background , "TOPRIGHT" ,8 ,3)
	self.frame.middleSection.staticTextColor:SetBackgroundColor(1,1,1, 1)
	self.frame.middleSection.staticTextColor.color = {1, 1, 1}


	self.frame.middleSection.opacity = libTUI.Slider(	self.frame.middleSection                                    -- parent
												, self.description .. "_opacity"              --,id
												,"Opacity"                                    --,labelText
												,60                                           --,labelWidth
												,"TOPLEFT"                                   --,selfPoint
												,self.frame.middleSection.inputs.staticText.label                 --,anchorFrame
												,"BOTTOMLEFT"                                 --,anchorPoint
												,-18                                            --,x
												,6                                            --,y
												,150                                          --,sliderWidth
												,100)                                         --,defaultValue)

	self.frame.middleSection.hideSpark = libTUI.ToggleFrame.Create(self.frame.middleSection
										,"Hide Spark", "Show Spark"
										,nil
										,100
										,"TOPRIGHT", self.frame.middleSection.opacity.input.label, "TOPLEFT" , -20 , 15
										,false
										,false
										,libTUI.ToggleFrame.Borders.All
										,{1,1,1,1})
	triggers.Tooltip(self.frame.middleSection.hideSpark.label,"Display the fade at < 1 seconds.",30,280)

	self.frame.disabled = libTUI.ToggleFrame.Create(self.frame
										,"Disabled", "Enabled"
										,nil
										,95
										,"BOTTOMRIGHT" ,self.frame ,"BOTTOMRIGHT" ,-10 , -5
										,false
										,true
										,libTUI.ToggleFrame.Borders.All
										,{1,1,1,1})
	triggers.Tooltip(self.frame.disabled.label,"Set the disbaled state of the selected trigger.",30,300)

	self.frame.update = libTUI.Button(self.frame
											,triggers.addonId
											,self.description .. "_update"
											,{width = 85, height=25}
											,"TOPRIGHT",self.frame.disabled.label,"TOPLEFT",-20,0
											,"\\images\\update.png"
											,"\\images\\updateOver.png"
											,false)

	self.frame.add = libTUI.Button(self.frame
									,triggers.addonId
									,self.description .. "_Add"
									,{width = 65, height=25}
									,"TOPRIGHT",self.frame.update,"TOPLEFT",-20,0
									,"\\images\\add.png"
									,"\\images\\addOver.png"
									,false)
	self.frame.add.Event.LeftClick = function ()
		self.onAdd()
	end

	self.toggles = {}

	self.frame.anchor.Event.LeftClick = function ()
		self.toggles.down = not self.toggles.down
		self:SetGrid()
	end

	self.frame.middleSection.opacity.input.Event.TextfieldChange = function ()
		local pos = self.frame.middleSection.opacity.input:GetText()
		pos = tonumber(pos) or 1
		if(pos > 100)then
			pos = 100
		end
		if(pos <=0)then
			pos = 0
		end
		self.frame.middleSection.opacity:SetPosition(pos)
	end

	self.frame.middleSection.opacity.Event.SliderChange = function ()
		local pos = self.frame.middleSection.opacity:GetPosition()
		self.frame.middleSection.opacity.input:SetText(tostring(pos))
	end

	self.frame.listScrollView = UI.CreateFrame("SimpleScrollView", self.description .. "_abilityList", self.frame)
	self.frame.listScrollView:SetPoint("TOPLEFT", listBox, "TOPLEFT", 0, 0)
	self.frame.listScrollView:SetPoint("BOTTOMRIGHT", listBox, "BOTTOMRIGHT", 0, 0)
	self.frame.listScrollView:SetBackgroundColor(0,0,0,1)
	self.frame.list = UI.CreateFrame("SimpleList", self.description .. "_AbilityList_SimpleList", self.frame.listScrollView)
	self.frame.list:SetFontSize(14)
	self.frame.list:SetBackgroundColor(0,0,0,1)
	self.frame.listScrollView:SetVisible(false)
	self.frame.listScrollView:SetContent(self.frame.list)

	self.frame.middleSection.opacity:SetPosition(100)
	self:SetGrid()


	self.frame.inputs.skillName.Event.KeyFocusGain = function()
		self.frame.inputs.skillName.Event.TextfieldChange = function()
			local t = libTUtility.Trim(self.frame.inputs.skillName:GetText())
			self.frame.listScrollView:SetVisible(false)
			if(string.len(t) > 0)then
				local ttype
				if (self.frame.buff.enabled)then
					ttype = libTConstants.CatalogItemType.BUFF
				elseif(self.frame.ability.enabled)then
					ttype = libTConstants.CatalogItemType.ABILITY
				elseif(self.frame.casting.enabled)then
					ttype = libTConstants.CatalogItemType.CASTBAR
				end
				local texts, ids = libAbilityCatalog.Catalog:GetCatalogItemStartingWith(t,ttype, false)
				self.frame.listScrollView:SetVisible(true)
				self.frame.list:SetLevelFontColor(1, 1, 1, 1)
				self.frame.list:SetLevelSelectable(1, true)
				self.frame.list:SetItems(texts, ids)
				self.frame.list.Event.ItemSelect = function(_, item, value, index)
					self.abilityId = value
					local ability = libAbilityCatalog.Catalog:GetCatalogItem(self.abilityId)
					self.frame.inputs.skillName:SetText(ability.name)
					self.frame.listScrollView:SetVisible(false)
				end
			end
		end
	end

	self.frame.inputs.skillName.Event.KeyFocusLoss = function()
		self.frame.list.Event.ItemSelect = function() end
		self.frame.listScrollView:SetVisible(false)
	end

	self.frame.inputs.encounter.Event.KeyFocusGain = function()
		self.frame.listScrollView:SetVisible(true)
		local texts, encounterIds, indent = libEncounters.EncounterManager:GetEncountersByZone()
		self.frame.list:SetLevelFontColor(1, 1, 1, 0)
		self.frame.list:SetLevelFontColor(2, 1, .5, 0)
		self.frame.list:SetLevelSelectable(1, false)
		self.frame.list:SetLevelSelectable(2, true)
		self.frame.list:SetItems(texts, encounterIds, indent)
		self.frame.list.Event.ItemSelect = function(_, item, value, index)
			self.encounterId = value
			self.frame.inputs.encounter:SetText(item)
			self.frame.listScrollView:SetVisible(false)
		end
	end

	self.frame.inputs.encounter.Event.KeyFocusLoss = function()
		self.frame.list.Event.ItemSelect = function() end
		self.frame.listScrollView:SetVisible(false)
	end

	self.frame.middleSection.staticTextColor.Event.LeftClick = function()
		local callback = function(r, g, b)
			self.frame.middleSection.staticTextColor:SetBackgroundColor(r,g,b,1)
		end
		triggers.ColorPicker.Display(callback,"TOPRIGHT",self.frame.middleSection.staticTextColor,"BOTTOMRIGHT",0,0)
	end
end

function Trigger:SetGrid()
	if(self.toggles.down)then
		self.frame.anchor:SetTexture(triggers.addonId,"\\images\\anchorUp.png")
		self.frame.middleSection:SetVisible(true)
		self.frame:SetHeight(self.size.height)
	else
		self.frame.anchor:SetTexture(triggers.addonId,"\\images\\anchorDown.png")
		self.frame.middleSection:SetVisible(false)
		self.frame:SetHeight(120)
	end
end

function Trigger:Set(t,callback)
	if(t.type == libTConstants.CatalogItemType.BUFF)then
		self.frame.buff:Enable(true)
		self.frame.ability:Enable(false)
		self.frame.casting:Enable(false)
		self.frame.buffGroup:SetVisible(true)
		self.frame.abilityGroup:SetVisible(false)
		self.frame.timer:SetVisible(true)
		self.frame.timerSection:SetVisible(false)
		self.frame.stacks:SetVisible(true)
		self.frame.stacksSection:SetVisible(false)
	elseif(t.type == libTConstants.CatalogItemType.ABILITY)then
		self.frame.ability:Enable(true)
		self.frame.casting:Enable(false)
		self.frame.buff:Enable(false)
		self.frame.buffGroup:SetVisible(false)
		self.frame.abilityGroup:SetVisible(true)
		self.frame.timer:SetVisible(true)
		self.frame.timerSection:SetVisible(false)
		self.frame.stacks:SetVisible(false)
		self.frame.stacksSection:SetVisible(false)
	elseif(t.type == libTConstants.CatalogItemType.CASTBAR)then
		self.frame.casting:Enable(true)
		self.frame.ability:Enable(false)
		self.frame.buff:Enable(false)
		self.frame.buffGroup:SetVisible(false)
		self.frame.abilityGroup:SetVisible(false)
		self.frame.timer:SetVisible(false)
		self.frame.timerSection:SetVisible(false)
		self.frame.stacksSection:SetVisible(false)
		self.frame.stacks:SetVisible(false)
	end

	self.frame.inputs.skillName:SetText(t.name)
	local encounter = libEncounters.EncounterManager:GetEncounter(t.encounterId)
	if(encounter and encounter.name)then
		self.frame.inputs.encounter:SetText(encounter.name)
	end
	self.frame.timer:Enable(t.timer)
	if(t.timer)then
		self.frame.timerSection.inputs.timerValue:SetText(tostring(t.time))
		self.frame.timerSection:SetVisible(true)
	end
	self.frame.stacks:Enable(t.stacker)
	if(t.stacker)then
		self.frame.stacksSection.inputs.stacksValue:SetText(tostring(t.stacks))
		self.frame.stacksSection:SetVisible(true)
	end
	self.frame.disabled:Enable(t.disabled)
	self.frame.middleSection.heals:Enable(bit.band(libTConstants.Roles.HEALER,t.roles) == libTConstants.Roles.HEALER)
	self.frame.middleSection.dps:Enable(bit.band(libTConstants.Roles.DPS,t.roles) == libTConstants.Roles.DPS)
	self.frame.middleSection.tank:Enable(bit.band(libTConstants.Roles.TANK,t.roles) == libTConstants.Roles.TANK)
	self.frame.middleSection.support:Enable(bit.band(libTConstants.Roles.SUPPORT,t.roles) == libTConstants.Roles.SUPPORT)
	self.frame.buffGroup.selfCast:Enable(t.selfCast)
	self.frame.buffGroup.active:Enable(bit.band(libTConstants.BuffState.ACTIVE, t.buffState) == libTConstants.BuffState.ACTIVE)
	self.frame.self:Enable(bit.band(libTConstants.Unit.PLAYER,t.unit) == libTConstants.Unit.PLAYER)
	self.frame.targetHostility:SetVisible(not self.frame.self.enabled)
	self.frame.targetHostility.hostile:Enable(bit.band(libTConstants.Relation.HOSTILE,t.relation) == libTConstants.Relation.HOSTILE)
	self.frame.abilityGroup.ready:Enable(bit.band(libTConstants.AbilityState.READY,t.abilityState) == libTConstants.AbilityState.READY)
	self.frame.stacksSection.stacksLessThan:Enable(t.stackCompare < 0)
	self.frame.timerSection.timerLessThan:Enable(t.timerCompare < 0)
	self.frame.middleSection.hideSpark:Enable(not t.showSpark)
	self.frame.abilityGroup.filtered:Enable(t.filtered)
	self.frame.middleSection.inputs.staticText:SetText(t.text or "")
	self.frame.middleSection.opacity:SetPosition(t.opacity or 100)
	self.frame.targetHostility.target:Enable(bit.band(libTConstants.Unit.TARGET,t.unit) == libTConstants.Unit.TARGET)
	self.frame.middleSection.staticTextColor:SetBackgroundColor( t.color[1], t.color[2], t.color[3] , 1)
	self.abilityId = t.abilityId
	self.encounterId = t.encounterId
	self:SetGrid()

	self.frame.update.Event.LeftClick = function ()
		callback()
	end
end

function Trigger:Clear()
	self.frame.casting:Enable(true)
	self.frame.timer:SetVisible(false)
	self.frame.timerSection:SetVisible(false)
	self.frame.stacksSection:SetVisible(false)
	self.frame.stacks:SetVisible(false)
	self.frame.ability:Enable(false)
	self.frame.buff:Enable(false)
	self.frame.buffGroup:SetVisible(false)
	self.frame.abilityGroup:SetVisible(false)
	self.frame.inputs.skillName:SetText("")
	self.frame.inputs.encounter:SetText("")
	self.frame.timer:Enable(false)
	self.frame.timerSection.inputs.timerValue:SetText("")
	self.frame.stacks:Enable(false)
	self.frame.stacksSection.inputs.stacksValue:SetText("")
	self.frame.disabled:Enable(false)
	self.frame.middleSection.heals:Enable(true)
	self.frame.middleSection.dps:Enable(true)
	self.frame.middleSection.tank:Enable(true)
	self.frame.middleSection.support:Enable(true)
	self.frame.buffGroup.selfCast:Enable(false)
	self.frame.buffGroup.active:Enable(false)
	self.frame.self:Enable(false)
	self.frame.targetHostility:SetVisible(true)
	self.frame.targetHostility.hostile:Enable(false)
	self.frame.abilityGroup.ready:Enable(false)
	self.frame.stacksSection.stacksLessThan:Enable(false)
	self.frame.timerSection.timerLessThan:Enable(false)
	self.frame.middleSection.hideSpark:Enable(false)
	self.frame.abilityGroup.filtered:Enable(false)
	self.frame.middleSection.inputs.staticText:SetText("")
	self.frame.middleSection.opacity:SetPosition( 100)
	self.frame.targetHostility.target:Enable(false)
	self.frame.middleSection.staticTextColor:SetBackgroundColor( 1, 1, 1 , 1)
	self.abilityId = nil
	self.encounterId = nil
	self:SetGrid()

	self.frame.update.Event.LeftClick = function () end
end

function Trigger:Update(t)
	if (self.frame.buff.enabled)then
		t.type = libTConstants.CatalogItemType.BUFF
	elseif(self.frame.ability.enabled)then
		t.type = libTConstants.CatalogItemType.ABILITY
	elseif(self.frame.casting.enabled)then
		t.type = libTConstants.CatalogItemType.CASTBAR
	end
	t.name = self.frame.inputs.skillName:GetText()
	t.abilityId = self.abilityId
	if(string.len(libTUtility.Trim(self.frame.inputs.encounter:GetText())) > 0)then
		t.encounterId = self.encounterId
	else
		t.encounterId = nil
	end
	t.disabled = self.frame.disabled.enabled
	t.roles = 0
	if(self.frame.middleSection.heals.enabled)then t.roles = t.roles + libTConstants.Roles.HEALER end
	if(self.frame.middleSection.dps.enabled)then t.roles = t.roles + libTConstants.Roles.DPS end
	if(self.frame.middleSection.tank.enabled)then t.roles = t.roles + libTConstants.Roles.TANK end
	if(self.frame.middleSection.support.enabled)then t.roles = t.roles + libTConstants.Roles.SUPPORT end
	t.selfCast = self.frame.buffGroup.selfCast.enabled
	t.buffState = libTConstants.BuffState.MISSING
	if(self.frame.buffGroup.active.enabled)then t.buffState = libTConstants.BuffState.ACTIVE end
	if(self.frame.self.enabled)then
		t.unit = libTConstants.Unit.PLAYER
	elseif(self.frame.targetHostility.target.enabled)then
		t.unit = libTConstants.Unit.TARGET
	else
		t.unit = libTConstants.Unit.FOCUS
	end
	if(self.frame.targetHostility.hostile.enabled)then
		t.relation = libTConstants.Relation.HOSTILE
	else
		t.relation = libTConstants.Relation.FRIENDLY
	end
	if(self.frame.timer.enabled)then
		t.timer = true
		t.time = tonumber(self.frame.timerSection.inputs.timerValue:GetText())
		t.timerCompare = 1
		if(self.frame.timerSection.timerLessThan.enabled)then
			t.timerCompare = -1
		end
	else
		t.timer = false
		t.time = nil
		t.timerCompare = -1
	end
	if(self.frame.stacks.enabled)then
		t.stacker = true
		t.stacks = tonumber(self.frame.stacksSection.inputs.stacksValue:GetText())
		t.stackCompare = 1
		if(self.frame.stacksSection.stacksLessThan.enabled)then
			t.stackCompare = -1
		end
	else
		t.stacker = false
		t.stacks = nil
		t.stackCompare = -1
	end

	local st = self.frame.middleSection.inputs.staticText:GetText()
	if(string.len(st) > 0)then
		t.text = st
	else
		t.text = nil
	end

	if(self.frame.abilityGroup.ready.enabled)then
		t.abilityState = libTConstants.AbilityState.READY
	else
		t.abilityState = libTConstants.AbilityState.COOLDOWN
	end
	t.filtered = self.frame.abilityGroup.filtered.enabled
	t.showSpark = not self.frame.middleSection.hideSpark.enabled
	t.opacity = self.frame.middleSection.opacity:GetPosition()
	if(t.opacity == 100)then
		t.opacity = nil
	end
	local r,g,b,a = self.frame.middleSection.staticTextColor:GetBackgroundColor()
	t.color = { r, g, b }
end

function Trigger:Close()
	self.frame.inputs.skillName:SetKeyFocus(false)
	self.frame.inputs.encounter:SetKeyFocus(false)
	self.frame.timerSection.inputs.timerValue:SetKeyFocus(false)
	self.frame.stacksSection.inputs.stacksValue:SetKeyFocus(false)
	for name, item in pairs(self.frame.middleSection.inputs)do
		item:SetKeyFocus(false)
	end
end

triggers.Configuration.Notifications.Trigger = Trigger
