--
-- Class Config
--
--   Manages a list of Config

local addonData, triggers = ...
triggers.Configuration = triggers.Configuration or {}
triggers.Configuration.Notifications = triggers.Configuration.Notifications or {}

local Config = {}
Config.__index = Config


function Config.Create()
	local obj = {}
	setmetatable(obj,Config)
	obj:Init()
	return obj
end

function Config:Init()
	self.size = { width = 900, height = 500 }
	self.description = "Config"
	self.padding = 10
end

function Config.UISetup(handle,parent)
	local self = triggers.Configuration.Notifications.Config

	self.parent = parent
	self.frame = libTUI.TUIClosableWindow(parent, self.description, self.size, "Triggers Configures", 120)

	self.frame.abilityBoundingBox = UI.CreateFrame("Frame",self.description .. "boundingbox", self.frame)

	self.frame.trigger = triggers.Configuration.Notifications.Trigger.Create(self.frame, self.frame.abilityBoundingBox)
	self.frame.notification = triggers.Configuration.Notifications.Notification.Create(self.frame)

	self.frame:SetHeight(self.size.height)

	self.frame.notification.frame:SetPoint("TOPLEFT", self.frame.header, "BOTTOMLEFT", 0, self.padding)
	self.frame.trigger.frame:SetPoint("TOPLEFT", self.frame.notification.frame, "BOTTOMLEFT", 0, self.padding)

	self.frame.statusBar = UI.CreateFrame("Frame",self.description .. "statusbar", self.frame)
	self.frame.statusBar:SetLayer(self.frame:GetLayer() + 1)
	self.frame.statusBar:SetHeight(20)
	self.frame.statusBar:SetWidth(self.frame:GetWidth())
	self.frame.statusBar:SetVisible(true)
	self.frame.statusBar:SetPoint("BOTTOMLEFT", self.frame, "BOTTOMLEFT", 0,0)
	self.frame.statusBar:SetBackgroundColor(0,0,0,1)

	self.frame.startsWith = libTUI.CheckBox(self.frame.statusBar
											,self.description .. "_TriggerDisabled"
											,"Search Starts with"
											,130
											,"TOPLEFT"
											,self.frame.statusBar
											,"TOPLEFT"
											,10
											,0)

	self.frame.abilityBoundingBox:SetPoint("TOPLEFT", self.frame.trigger.frame, "BOTTOMLEFT", 0, self.padding)
	self.frame.abilityBoundingBox:SetPoint("BOTTOMRIGHT", self.frame.statusBar, "TOPRIGHT", -45, 0)

	self.frame.triggerListScrollView = UI.CreateFrame("SimpleScrollView", self.description .. "_TriggerList", self.frame)
	self.frame.triggerListScrollView:SetPoint("TOPLEFT", self.frame.trigger.frame, "BOTTOMLEFT", 0, self.padding)
	self.frame.triggerListScrollView:SetPoint("BOTTOMRIGHT", self.frame.statusBar, "TOPRIGHT", -45, 0)
	self.frame.triggerListScrollView:SetBackgroundColor(0,0,0,1)
	self.frame.triggerList = UI.CreateFrame("SimpleList", self.description .. "_AbilityList_SimpleList", self.frame.triggerListScrollView)
	self.frame.triggerList:SetFontSize(14)
	self.frame.triggerListScrollView:SetContent(self.frame.triggerList)

	self.frame.delete = libTUI.Button(self.frame.header
										,triggers.addonId
										,self.description .. "_Delete"
										,{width = 100, height=30}
										,"TOPRIGHT",self.frame.header,"TOPRIGHT",-30,4
										,"\\images\\delete.png"
										,"\\images\\deleteOver.png"
										,true)
	triggers.Tooltip(self.frame.delete,"Deletes this notification\n Use the checkbox to confirm.",40,300,self.frame.delete.Event.MouseIn,self.frame.delete.Event.MouseOut)

	self.frame.disabled = libTUI.ToggleFrame.Create(self.frame.header
										,"Disabled", "Enabled"
										,nil
										,95
										,"TOPLEFT" ,self.frame ,"TOPLEFT" ,20 ,6
										,false
										,true
										,libTUI.ToggleFrame.Borders.All
										,{1,1,1,1})
	triggers.Tooltip(self.frame.disabled.label,"Set the enabled state of this notification.",30,300)

	self.frame.export = libTUI.Button(	self.frame.header
										,triggers.addonId
										,self.description .. "_Share"
										,{width = 55, height=32}
										,"TOPLEFT" ,self.frame.disabled.label ,"TOPRIGHT" ,10 ,-2
										,"\\images\\share.png"
										,"\\images\\shareOver.png"
										,false)
	triggers.Tooltip(self.frame.export,"Exports this notification as text",30)

	self.frame.up = libTUI.Button(	self.frame
									,triggers.addonId
									,self.description .. "_Up"
									,{width = 35, height=25}
									,"TOPRIGHT",self.frame.trigger.frame,"BOTTOMRIGHT",-4,10
									,"\\images\\up.png"
									,"\\images\\upOver.png"
									,false)
	triggers.Tooltip(self.frame.up,"Moves the selected trigger up in sequence.",30)

	self.frame.down = libTUI.Button(self.frame
									,triggers.addonId
									,self.description .. "_Down"
									,{width = 35, height=25}
									,"TOPRIGHT",self.frame.up,"BOTTOMRIGHT",0,10
									,"\\images\\down.png"
									,"\\images\\downOver.png"
									,false)
	triggers.Tooltip(self.frame.down,"Moves the selected trigger down in sequence.",30)


	self.frame.remove = libTUI.Button(	self.frame.statusBar
										,triggers.addonId
										,self.description .. "_Down"
										,{width = 35, height=25}
										,"BOTTOMRIGHT",self.frame,"BOTTOMRIGHT",-4,-4
										,"\\images\\remove.png"
										,"\\images\\removeOver.png"
										,true)
	triggers.Tooltip(self.frame.remove,"Removes the selected trigger.\n check the checkbox to confirm",40)
	self.frame.remove.check:ClearAll()
	self.frame.remove.check:SetPoint("BOTTOMRIGHT",self.frame.remove,"BOTTOMLEFT",-5,4)

	self.frame.Closed = function()
		local self = triggers.Configuration.Notifications.Config
		triggers.Notifications:ShowOthersAfterConfigerz()
		self:Close()
	end
	self.frame.Moved = function()
		self.attached = false
	end
	self.OnDisable = function () end
	self.SetPositionAndSize = function () end
end

function Config:Close()
	self.visible = false
	self.frame:SetVisible(self.visible)
	triggers.Configuration.Notifications.NewFrameBorder:Show(false)
	-- remove handling of any callback handlers.
	self.OnDisable = function () end
	self.frame.export.Event.LeftClick = function () end
	self.frame.delete.Event.LeftClick = function () end
	self.frame.disabled.Changed = function () end
	self.frame.up.Event.LeftClick = function () end
	self.frame.down.Event.LeftClick = function () end
	self.frame.remove.Event.LeftClick = function () end
	self.frame.triggerList.Event.ItemSelect = function () end
	self.frame.trigger.onAdd = function () end
	self.frame.delete.Event.LeftClick = function () end
	self.OnNotificationChange = function () end
	self.OnTriggerChange = function () end
	self.SetPositionAndSize = function () end
	self.frame.trigger:Close()
	self.frame.notification:Close()
	triggers.TootipClose()
end

function Config:Show(notification)
	triggers.Notifications:HideOthersWhileConfigerz(notification)
	self.visible = true
	self.attached = true
	self.frame:SetVisible(self.visible)
	triggers.Configuration.Notifications.NewFrameBorder:Show(true,notification.frame)
	self.frame.trigger:Clear()
	self.frame.delete.check:SetChecked(false)
	self.frame.delete.Event.LeftClick = function (self)
		if(self.check:GetChecked() == true)then
			triggers.Notifications:DeleteNotification(notification)
			triggers.Configuration.Notifications.Config:Toggle()
		end
	end
	self.frame.delete:SetVisible(notification.deletable)
	self.frame.delete.check:SetVisible(notification.deletable)
	-- callback for Notification updating from the config window
	self.OnNotificationChange =  function()
		self.frame.notification:Update(notification)
		self.frame.triggerList:SetItems(notification:StringTriggerList())
		notification:UpdateImage()
		self.SetPositionAndSize(notification)
	end

	self.frame.notification:Set(notification,self.OnNotificationChange)

	-- callback for Adding a new trigger from the config window
	self.frame.trigger.onAdd = function()
		local t = triggers.Trigger.Create()
		self.frame.trigger:Update(t)
		notification:PrependTrigger(t)
		self.frame.triggerList:SetItems(notification:StringTriggerList())
		self.frame.trigger:Clear()
		notification:UpdateImage()
	end

	self.frame.triggerList:SetItems(notification:StringTriggerList())
	self.frame.triggerList.Event.ItemSelect = function(_, _, _, index)
		local trigger = notification.triggers[index]
		-- callback for Trigger udpating from the config window
		self.OnTriggerChange = function()
			self.frame.trigger:Update(trigger)
			self.frame.triggerList:SetItems(notification:StringTriggerList())
			self.frame.trigger:Clear()
			notification:UpdateImage()
		end
		self.frame.trigger:Set(trigger,self.OnTriggerChange)
		-- callback for removing this trigger
		self.frame.remove.Event.LeftClick = function()
			if(self.frame.remove.check:GetChecked() == true)then
				notification:RemoveTrigger(index)
				self.frame.triggerList:SetItems(notification:StringTriggerList())
			end
		end
		-- callback for moving this trigger down
		self.frame.down.Event.LeftClick = function()
			notification:TriggerDown(index)
			self.frame.triggerList:SetItems(notification:StringTriggerList())
			self.frame.triggerList:SetSelectedIndex(index + 1, true)
		end
		-- callback for moving this trigger Up
		self.frame.up.Event.LeftClick = function()
			notification:TriggerUp(index)
			self.frame.triggerList:SetItems(notification:StringTriggerList())
			self.frame.triggerList:SetSelectedIndex(index - 1, true)
		end
	end
	-- Callback for disabled flag changed
	self.frame.disabled.Changed = function()
		notification:Disable(self.frame.disabled.enabled)
	end
	-- callback for delete clicked
	self.frame.delete.Event.LeftClick = function()
		if(self.frame.delete.check:GetChecked() == true)then
			triggers.Notifications:DeleteNotification(notification)
			self:Close()
			self.frame.delete.check:SetChecked(false)
			triggers.Configuration.Notifications.NewFrameBorder:Show(false)
		end
	end
	-- callback for export clicked.
	self.frame.export.Event.LeftClick = function()
		triggers.ExportDialog:Show(notification)
	end
	-- callback from notification if disabled from doubleclick
	self.OnDisable = function(n,v)
		if(n==notification)then
			if(self.frame.disabled.enabled ~= v)then
				self.frame.disabled:Enable(v)
			end
		end
	end

	-- callback used to adjust notification size and position in window
	self.SetPositionAndSize = function(n)
		if(n ~= notification)then return end
		self.frame.notification:UpdateSizeAndPosition(n.size, n.position)

		if(self.attached == false)then return end
		local nTop = notification.position.top
		local nLeft = notification.position.left
		local nBottom = notification.position.top + notification.size
		local nRight =  notification.position.left + notification.size

		local nCenterX = (notification.size / 2) + nLeft
		local nCenterY = (notification.size / 2) + nTop

		local uiPCenterX = UIParent:GetWidth() / 2
		local uiPCenterY = UIParent:GetHeight() / 2

		local configUp = nCenterY > uiPCenterY
		local configLeft = nCenterX > uiPCenterX

		local top = 0
		local left = 0
		if(configUp)then
			top = nBottom - self.size.height
		else
			top = nTop
		end
		if(configLeft)then
			left = nLeft - self.size.width - 25
		else
			left = nRight + 25
		end
		if(top < 0)then
			top = 0
		end
		if(top + self.size.height > UIParent:GetHeight())then
			top = UIParent:GetHeight() - self.size.height
		end
		self.frame:ClearPoint("CENTER")
		self.frame:SetPoint("TOPLEFT",UIParent, "TOPLEFT", left, top)
	end
	self.frame.startsWith:SetChecked(true)
	self.SetPositionAndSize(notification)
end

triggers.Configuration.Notifications.Config = Config.Create()

Command.Event.Attach(Triggers.Event.ContextCreated , Config.UISetup, "Config.UISetup")
