--
-- Class ExportDialog
--
--   Manages a list of ExportDialog

local addonData, triggers = ...

local ExportDialog = {}

ExportDialog.__index = ExportDialog

function ExportDialog.Create()
	local obj = {}
	setmetatable(obj,ExportDialog)
	obj:Init()
	return obj
end

function ExportDialog:Init()
	self.size = { width = 670, height = 490 }
	self.description = "ExportDialog"
end

function ExportDialog.UISetup(handle,parent)
	self = triggers.ExportDialog
	self.parent = parent
	self.frame = libTUI.TUIClosableWindow(self.parent, self.description, self.size, "Export", 500)
	self.frame.Closed = function()
		local self = triggers.ExportDialog
		self.frame.text:SetKeyFocus(false)
	end

	self.frame.text = UI.CreateFrame("RiftTextfield", self.description .. "_Text", self.frame)
	self.frame.text:SetWidth(self.size.width - 10)
	self.frame.text:SetHeight(200)
	self.frame.text:SetMouseMasking("limited")
	self.frame.text:SetLayer(101)
	self.frame.text:SetPoint("TOPCENTER",self.frame.header,"BOTTOMCENTER")
	self.frame.text:SetVisible(true)
	self.frame.text:SetBackgroundColor(.3,.3,.3,1)


	self.frame.triggerListScrollView = UI.CreateFrame("SimpleScrollView", self.description .. "_TriggerList", self.frame)
	self.frame.triggerListScrollView:SetPoint("BOTTOMCENTER", self.frame, "BOTTOMCENTER",0,-40)
	self.frame.triggerListScrollView:SetWidth(self.size.width)
	self.frame.triggerListScrollView:SetHeight(200)
	self.frame.triggerListScrollView:SetBorder(1, .5, .5, .5, 1)
	self.frame.triggerListScrollView:SetBackgroundColor(0,0,0,1)
	self.frame.triggerList = UI.CreateFrame("SimpleList", self.description .. "_AbilityList_SimpleList", self.frame.triggerListScrollView)
	self.frame.triggerList:SetFontSize(14)
	self.frame.triggerList:SetSelectionMode("multi")
	self.frame.triggerList.Event.SelectionChange = function()
		local self = triggers.ExportDialog
		self:Compress()
	end
	self.frame.triggerListScrollView:SetContent(self.frame.triggerList)


	self.frame.selectAll = libTUI.Button(					 self.frame
																									,triggers.addonId                                 --parent
																									,self.description .. "_Share"                   --,id
																									,{width = 44, height=30}                      --,size
																									,"TOPLEFT"                                --,selfPoint
																									,self.frame.triggerListScrollView                    --,anchorFrame
																									,"BOTTOMLEFT"                                --,anchorPoint
																									,100                                           --,x
																									,7                                            --,y
																									,"\\images\\all.png"                          --,image
																									,"\\images\\allOver.png"                      --,hoverImage
																									,false)

	function self.frame.selectAll.Event:LeftClick()
		local self = triggers.ExportDialog
		self.frame.triggerList:SetSelectedValues({},true)
		local i = 1
		for id,value in pairs(self.triggers)do
			self.frame.triggerList:AddSelectedIndex(i, true)
			i = i + 1
		end
		self:Compress()
	end


	self.frame.selectNone = libTUI.Button(					 self.frame
																									,triggers.addonId                                 --parent
																									,self.description .. "_Share"                   --,id
																									,{width = 63, height=30}                      --,size
																									,"TOPRIGHT"                                --,selfPoint
																									,self.frame.triggerListScrollView                    --,anchorFrame
																									,"BOTTOMRIGHT"                                --,anchorPoint
																									,-100                                           --,x
																									,7                                            --,y
																									,"\\images\\none.png"                          --,image
																									,"\\images\\noneOver.png"                      --,hoverImage
																									,false)

	function self.frame.selectNone.Event:LeftClick()
		local self = triggers.ExportDialog
		self.frame.triggerList:SetSelectedValues({},true)

		self:Compress()
	end
end

function ExportDialog:Compress()
	local pack = {notification = {}, encounters = {}, abilities = {}}
	pack.notification = triggers.Notification.SerializeValues(self.notificationVoidOfTriggers)
	pack.notification.triggers = {}
	local triggerIds = self.frame.triggerList:GetSelectedValues()
	for _, triggerId in ipairs(triggerIds)do
		table.insert(pack.notification.triggers,self.triggers[triggerId].trigger:Serialize())
	end
	pack.notification.roles = nil
	local description = pack.notification.description
	local savedEncounterIDs = {}
	local savedAbilityIDs = {}
	if(pack.notification.triggers)then
		for _, trigger in ipairs(pack.notification.triggers)do
			local ability = libAbilityCatalog.Catalog:GetCatalogItem(trigger.abilityId)
			if(ability and savedAbilityIDs[ability.id] == nil)then
				savedAbilityIDs[ability.id] = true
				table.insert(pack.abilities,ability:Serialize())
			end
			if(trigger.encounterId)then
				local encounter = libEncounters.EncounterManager:GetEncounter(trigger.encounterId)
				if(encounter and savedEncounterIDs[encounter.id] == nil)then
					savedEncounterIDs[encounter.id] = true
					table.insert(pack.encounters,encounter:Serialize())
				end
			end
		end
	end
	local sendData = Utility.Serialize.Inline(pack)
	libTDebug.UIDebug:DEBUG(nil, "Export: Data:%s", sendData)
	local cd = zlib.deflate(zlib.BEST_COMPRESSION)(sendData, "finish")
	local encodedN ="TRIGGERS:[" .. description .. "]" .. libTUtility.EncodeBase64(cd)
	local tmp = {}
	for i = 1, encodedN:len(), 76 do
		table.insert(tmp, encodedN:sub(i, i + 76 - 1))
	end
	local result = table.concat(tmp, "\n")
	self.frame.text:SetText(result)
	self.frame.text:SetKeyFocus(true)
	self.frame.text:SetSelection(0,string.len(result))
	self.frame:SetVisible(true)
end

function ExportDialog:Show(notification)
	self.notificationVoidOfTriggers = notification:Serialize()
	self.triggers = {}
	local texts = {}
	local values = {}
	for _, trigger in ipairs(notification.triggers)do
		local t = trigger:toString()
		local v = trigger.id
		self.triggers[trigger.id] = {trigger = trigger, text = t}
		table.insert(texts,t)
		table.insert(values,v)
	end

	self.frame.triggerList:SetItems(texts,values)
	for i,_ in ipairs(texts)do
		self.frame.triggerList:AddSelectedIndex(i, true)
	end
	self:Compress()
end



Command.Event.Attach(Triggers.Event.ContextCreated , ExportDialog.UISetup, "ExportDialog.UISetup")


triggers.ExportDialog = ExportDialog.Create()