--
-- Class ImportDialog
--
--   Manages a list of ImportDialog

local addonData, triggers = ...

local ImportDialog = {}

ImportDialog.__index = ImportDialog

function ImportDialog.Create()
	local obj = {}
	setmetatable(obj,ImportDialog)
	obj:Init()
	return obj
end

function ImportDialog:Init()
	self.size = { width = 600, height = 610 }
	self.description = "ImportDialog"
	self.currentImport = nil
end

function ImportDialog.UISetup(handle,parent)
	local self = triggers.ImportDialog
	self.parent = parent
	self.frame = libTUI.TUIClosableWindow(self.parent, self.description, self.size, "Import", 500)
	self.frame.Closed = function()
		self = triggers.ImportDialog
		for _, i in pairs(self.frame.inputs)do
			if(i.SetKeyFocus)then
				i:SetKeyFocus(false)
			end
		end
	end

	self.frame.inputs = {}
	self.frame.inputs.importString = libTUI.Input(	self.frame                                    --parent
											,self.description .. "_importText"            --,id
											,"Import Text"                                --,labelText
											,80                                           --,labelWidth
											,"TOPLEFT"                                    --,selfPoint
											,self.frame.header          --,anchorFrame
											,"BOTTOMLEFT"                                   --,anchorPoint
											,0                                           --,x
											,0                                            --,y
											,self.size.width                                          --,inputWidth
											,""
											,true)
	self.frame.inputs.importString:SetHeight(100)
	self.frame.inputs.importString.background:SetHeight(100)
	self.frame.inputs.importString.Event.TextfieldChange = function() triggers.ImportDialog:ParseText() end

	self.frame.error = UI.CreateFrame("Text",self.description .. "_Error", self.frame)
	self.frame.error:SetWidth(self.size.width)
	self.frame.error:SetLayer(105)
	self.frame.error:SetVisible(true)
	self.frame.error:SetFontSize(15)
	self.frame.error:SetFontColor(1,1,0)
	self.frame.error:SetPoint("TOPLEFT", self.frame.inputs.importString, "BOTTOMLEFT",0,0)

	self.frame.abilityList = libTUI.ScrollingText(self.frame
						,self.description .. "_importAbilties"
						,"Abilities that will be imported"
						,self.size.width
						,"TOPLEFT"
						,self.frame.error
						,"BOTTOMLEFT"
						,0
						,5
						,self.size.width
						,100
						,""
						,true)

	self.frame.encounterList = libTUI.ScrollingText(self.frame
						,self.description .. "_importEncounters"
						,"Encounters that will be imported"
						,self.size.width
						,"TOPLEFT"
						,self.frame.abilityList.scrollView
						,"BOTTOMLEFT"
						,0
						,5
						,self.size.width
						,60
						,""
						,true)

	self.frame.triggerList = libTUI.ScrollingText(self.frame
						,self.description .. "_importEncounters"
						,"The notification that will be imported"
						,self.size.width
						,"TOPLEFT"
						,self.frame.encounterList.scrollView
						,"BOTTOMLEFT"
						,0
						,5
						,self.size.width
						,140
						,""
						,true)

	self.frame.import = libTUI.Button(self.frame
									,"LibTUtility"
									,self.description .. "_Okay"
									,{width = 67, height = 30}
									,"BOTTOMCENTER"
									,self.frame
									,"BOTTOMCENTER"
									,0
									,-5
									,"\\images\\import.png"
									,"\\images\\importOver.png"
									,false)
	self.frame.import:SetVisible(false)
	self.frame.import.Event.LeftClick = function()
		local self = triggers.ImportDialog
		self:Import()
	end	
end

function ImportDialog:SetError(err)
	self.frame.error:SetText(err)
end

function ImportDialog:ParseText()
	self.currentImport = nil
	self.frame.import:SetVisible(false)
	local prefix = "TRIGGERS"
	local text = self.frame.inputs.importString:GetText()
	local importPrefix = string.sub(text, 1, string.len(prefix))
	self.frame.error:SetText("")
	if(prefix ~= importPrefix)then return self:SetError("Error processing import text") end
	local binaryText = libTUtility.DecodeBase64(string.sub(text,string.find(text,"]") + 1,string.len(text)))
	if(binaryText == nil)then return self:SetError("Error processing import text(base64)") end
	local d, eof = zlib.inflate()(binaryText)
	if not eof then return self:SetError("Error processing import text (Decompress)") end
	libTDebug.UIDebug:DEBUG(nil, "Import: Data:%s", d)
	local table = loadstring("return " .. d)()
	if(table == nil)then return self:SetError("Error processing import text (LoadString)") end
	if(table.abilities == nil or table.notification == nil  or table.encounters == nil) then self:SetError("Error processing import text(bad table structure)") end
	if(#table.abilities == 0 and table.notification == nil and #table.encounters == 0) then return self:SetError("Error processing import text(nothing to import)") end
	libTDebug.UIDebug:AddTable("Import.Notification",table)
	local abilitiesString = ""
	local abilitiesHeight = 0
	if(table.abilities ~= nil and #table.abilities > 0)then
		abilitiesHeight = 18 * #table.abilities
		for _, ability in ipairs(table.abilities)do
			abilitiesString = string.format("%s<font color='#FFFF00'>%s : </font><font color='#88AAFF'>%s</font>\n",abilitiesString,ability.name,ability.description)
		end
	end
	self.frame.abilityList:SetText(abilitiesString,true)
	self.frame.abilityList:SetHeight(abilitiesHeight)

	local encountersString = ""
	local encountersHeight = 0
	if(table.encounters ~= nil and #table.encounters > 0)then
		encountersHeight = 18 * #table.encounters
		for _, encounter in ipairs(table.encounters)do
			encountersString = string.format("%s<font color='#FFFF00'>%s : </font><font color='#88AAFF'>%s</font>\n",encountersString,encounter.name,encounter.zone)
		end
	end
	self.frame.encounterList:SetText(encountersString,true)
	self.frame.encounterList:SetHeight(encountersHeight)

	local triggerString = ""
	local triggerHeight = 0
	if(table.notification ~= nil and #table.notification.triggers > 0)then
		local n = triggers.Notification.Deserialize(table.notification, true)
		triggerHeight = 18 * #n.triggers
		for _, trigger in ipairs(n.triggers)do
			local s = trigger:toString()
			s = string.gsub(s, ">","&gt;")
			s = string.gsub(s, "<","&lt;")
			triggerString = triggerString .. "<font color='#FFFF00'>" .. s .. "</font>\n"
		end
	end
	self.frame.triggerList:SetText(triggerString,true)
	self.frame.triggerList:SetHeight(triggerHeight)
	self.currentImport = table
	self.frame.import:SetVisible(true)
end

function ImportDialog:Import()
	local table = self.currentImport
	if(table.abilities ~= nil and #table.abilities > 0)then
		for _, ability in ipairs(table.abilities)do
			local myAbility = libAbilityCatalog.Catalog:GetCatalogItem(ability.id)
			if(myAbility == nil)then
				local newAbility = libAbilityCatalog.CatalogItem.Deserialize(ability.id, ability)
				libAbilityCatalog.Catalog:AddCatalogItem(newAbility)
			end
		end
	end
	if(table.encounters ~= nil and #table.encounters > 0)then
		for _, encounter in ipairs(table.encounters)do
			local myEncounter = libEncounters.EncounterManager:GetEncounter(encounter.id)
			if(myEncounter == nil)then
				local newEncounter = libEncounters.Encounter.Deserialize(encounter)
				newEncounter.id = encounter.id
				libEncounters.EncounterManager:AddEncounter(newEncounter)
			end
		end
	end
	if(table.notification ~= nil and #table.notification.triggers > 0)then
		local n = triggers.Notification.Deserialize(table.notification, true)
		local myn = triggers.Notifications:GetNotification(n.id)
		if(myn == nil)then
			myn = triggers.Notification.Deserialize(table.notification, true)
			triggers.Notifications:AddNotification(myn)
			myn.roles = nil
		else
			myn = triggers.Notification.Deserialize(table.notification, false, myn)
			for _, trigger in ipairs(n.triggers)do
				local myt = myn:GetTriggerById(trigger.id)
				if(myt == nil)then
					myt = triggers.Trigger.Deserialize(trigger)
					myn:AppendTrigger(myt)
				else
					myt = triggers.Trigger.Deserialize(trigger, myt)
				end
			end
		end
	end
	self.frame.error:SetText("Imported")
end

function ImportDialog:Show()
	self.frame:SetVisible(true)
end


Command.Event.Attach(Triggers.Event.ContextCreated , ImportDialog.UISetup, "ImportDialog.UISetup")
triggers.ImportDialog = ImportDialog.Create()