--
-- Class SharingDialog
--
--   Manages a list of SharingDialog

local addonData, triggers = ...

local SharingDialog = {}
local Sharing = {}
Sharing.Messages = {}

local triggerDefaults = "Default Triggers"


SharingDialog.__index = SharingDialog

function SharingDialog.Create()
	local obj = {}
	setmetatable(obj,SharingDialog)
	obj:Init()
	return obj
end

function SharingDialog:Init()
	self.size = { width = 730, height = 440 }
	self.description = "SharingDialog"

	self.users = {}
end

function SharingDialog:Show()
	self.users = {}
	self.frame:SetVisible(true)
	triggers.Configuration.Notifications.Config:Close(true)
	triggers.MapIcon:HideEditMode()
	Command.Message.Broadcast("party", nil, "Triggers_AWK", "")
	Command.Message.Broadcast("raid", nil, "Triggers_AWK", "")
	Command.Message.Broadcast("guild", nil, "Triggers_AWK", "")
end

function SharingDialog:UISetupPeople()
	self.frame.people =  UI.CreateFrame("Frame",self.description .. "People", self.frame)
	self.frame.people:SetVisible(false)
	self.frame.people:SetWidth(self.size.width)
	self.frame.people:SetHeight(self.size.height - 35)
	self.frame.people:SetMouseMasking("limited")
	self.frame.people:SetLayer(102)
	self.frame.people:SetPoint("TOPLEFT", self.frame.header, "BOTTOMLEFT")

	self.frame.people.inspectTypes = {}
	self.frame.people.inspectTypes.Interupts = libTUI.ToggleFrame.Create(self.frame.people
										,"Interupts", "Interupts"
										,nil
										,175
										,"TOPLEFT", self.frame.people, "TOPLEFT" ,10 ,15
										,false
										,false
										,libTUI.ToggleFrame.Borders.Left
										,{1,1,1,1})
	self.frame.people.inspectTypes.Purge = libTUI.ToggleFrame.Create(self.frame.people
										,"Purge", "Purge"
										,nil
										,175
										,"TOPLEFT", self.frame.people.inspectTypes.Interupts.label, "TOPRIGHT" ,10 ,0
										,false
										,false
										,libTUI.ToggleFrame.Borders.None
										,{1,1,1,1})
	self.frame.people.inspectTypes.DPSBuff = libTUI.ToggleFrame.Create(self.frame.people
										,"DPS Buff", "DPS Buff"
										,nil
										,175
										,"TOPLEFT", self.frame.people.inspectTypes.Purge.label, "TOPRIGHT" ,10 ,0
										,false
										,false
										,libTUI.ToggleFrame.Borders.Right
										,{1,1,1,1})

	self.frame.people.inspectTypes.Move = libTUI.ToggleFrame.Create(self.frame.people
										,"Move", "Move"
										,nil
										,175
										,"TOPLEFT", self.frame.people.inspectTypes.Interupts.label, "BOTTOMLEFT" ,0 ,5
										,false
										,false
										,libTUI.ToggleFrame.Borders.Left
										,{1,1,1,1})
	self.frame.people.inspectTypes.Shield = libTUI.ToggleFrame.Create(self.frame.people
										,"Shield", "Shield"
										,nil
										,175
										,"TOPLEFT", self.frame.people.inspectTypes.Move.label, "TOPRIGHT" ,10 ,0
										,false
										,false
										,libTUI.ToggleFrame.Borders.None
										,{1,1,1,1})
	self.frame.people.inspectTypes.Taunt = libTUI.ToggleFrame.Create(self.frame.people
										,"Taunt", "Taunt"
										,nil
										,175
										,"TOPLEFT", self.frame.people.inspectTypes.Shield.label, "TOPRIGHT" ,10 ,0
										,false
										,false
										,libTUI.ToggleFrame.Borders.Right
										,{1,1,1,1})
	self.frame.people.inspectTypes.Debilitate = libTUI.ToggleFrame.Create(self.frame.people
										,"Debilitate", "Debilitate"
										,nil
										,175
										,"TOPLEFT", self.frame.people.inspectTypes.Move.label, "BOTTOMLEFT" ,0 ,5
										,false
										,false
										,libTUI.ToggleFrame.Borders.Left
										,{1,1,1,1})
	self.frame.people.inspectTypes.Roots = libTUI.ToggleFrame.Create(self.frame.people
										,"Roots", "Roots"
										,nil
										,175
										,"TOPLEFT", self.frame.people.inspectTypes.Debilitate.label, "TOPRIGHT" ,10 ,0
										,false
										,false
										,libTUI.ToggleFrame.Borders.None
										,{1,1,1,1})
	self.frame.people.inspectTypes.Other = libTUI.ToggleFrame.Create(self.frame.people
										,"Other", "Other"
										,nil
										,175
										,"TOPLEFT", self.frame.people.inspectTypes.Roots.label, "TOPRIGHT" ,10 ,0
										,false
										,false
										,libTUI.ToggleFrame.Borders.Right
										,{1,1,1,1})

	for _, item in pairs(self.frame.people.inspectTypes)do
		item.onChanged = function()
			for _, i in pairs(self.frame.people.inspectTypes)do
				if(i ~= item)then
					i:Enable(false)
				end
			end
			self.inspectType = item
			self:RequestTriggers()
		end
	end

	self.inspectTypesMap = {
		[self.frame.people.inspectTypes.Interupts] = "Triggers-Interupt",
		[self.frame.people.inspectTypes.Purge] = "Triggers-Purge",
		[self.frame.people.inspectTypes.DPSBuff] = "Triggers-Fight",
		[self.frame.people.inspectTypes.Move] = "Triggers-Move",
		[self.frame.people.inspectTypes.Shield] = "Triggers-Shield",
		[self.frame.people.inspectTypes.Taunt] = "Triggers-Taunt",
		[self.frame.people.inspectTypes.Debilitate] = "Triggers-Debilitated",
		[self.frame.people.inspectTypes.Roots] = "Triggers-Rooted",
		[self.frame.people.inspectTypes.Other] = false,
	}

	self.frame.people.filterText = libTUI.Input( self.frame.people
										,self.description .. "_filterText"
										,"filter"
										,45
										,"BOTTOMLEFT", self.frame.people, "BOTTOMLEFT",10,0
										,200
										,""
										,false)

	self.frame.people.applyFilter = libTUI.Button(self.frame.people
									,triggers.addonId
									,self.description .. "_filterButton"
									,{width = 80, height = 25}
									,"TOPLEFT" ,self.frame.people.filterText.background ,"TOPRIGHT" ,25 , 0
									,"\\images\\apply.png"
									,"\\images\\applyOver.png"
									,false)

	self.frame.helpText = UI.CreateFrame("Text",self.description .. "_helptext", self.frame.people)
	self.frame.helpText:SetPoint("BOTTOMLEFT", self.frame.people.filterText.label, "TOPLEFT",0,-10 )
	self.frame.helpText:SetText("Color key: <font color='#FFFF00'>Yellow: Missing                    </font><font color='#FF8800'>Orange: Different                    </font><font color='#888888'>Grey: same</font>",true)

	self.frame.people.triggerListScrollView = UI.CreateFrame("SimpleScrollView", self.description .. "_TriggerList", self.frame.people)
	self.frame.people.triggerListScrollView:SetPoint("BOTTOMLEFT", self.frame.helpText, "TOPLEFT",0,-10 )
	self.frame.people.triggerListScrollView:SetWidth(710)
	self.frame.people.triggerListScrollView:SetHeight(200)
	self.frame.people.triggerListScrollView:SetBorder(1, .5, .5, .5, 1)
	self.frame.people.triggerListScrollView:SetBackgroundColor(0,0,0,1)
	self.frame.people.triggerList = UI.CreateFrame("SimpleList", self.description .. "_AbilityList_SimpleList", self.frame.people.triggerListScrollView)
	self.frame.people.triggerList:SetFontSize(14)
	self.frame.people.triggerListScrollView:SetContent(self.frame.people.triggerList)


	self.frame.people.userListScrollView = UI.CreateFrame("SimpleScrollView", self.description .. "_TriggerList", self.frame.people)
	self.frame.people.userListScrollView:SetPoint("TOPRIGHT", self.frame.people, "TOPRIGHT",-10,3)
	self.frame.people.userListScrollView:SetWidth(150)
	self.frame.people.userListScrollView:SetHeight(125)
	self.frame.people.userListScrollView:SetBorder(1, .5, .5, .5, 1)
	self.frame.people.userListScrollView:SetBackgroundColor(0,0,0,1)
	self.frame.people.userList = UI.CreateFrame("SimpleList", self.description .. "_AbilityList_SimpleList", self.frame.people.userListScrollView)
	self.frame.people.userList:SetFontSize(14)
	self.frame.people.userListScrollView:SetContent(self.frame.people.userList)
	self.frame.people.userList.dialog = self
	self.frame.people.userList:SetSelectionMode("single")
	self.frame.people.userList.Event.ItemClick = function(item,_,_)
		local self = triggers.SharingDialog
		self.selectedUser = item
		self.frame.people.userList.dialog:RequestTriggers()
	end

	self.frame.error = UI.CreateFrame("Text",self.description .. "_Error", self.frame)
	self.frame.error:SetWidth(550)
	self.frame.error:SetLayer(105)
	self.frame.error:SetVisible(true)
	self.frame.error:SetFontSize(15)
	self.frame.error:SetFontColor(1,1,0)
	--self.frame.error:SetBackgroundColor(0,0,0,1)
	self.frame.error:SetPoint("BOTTOMLEFT", self.frame.people.triggerListScrollView, "TOPLEFT",0,-10)

	self.selectedUser = nil
	self.inspectType = self.frame.people.inspectTypes.interupts
	self.frame.message = libTUI.Message(self.frame, "Message", "Would you like to create this?")
	self.frame.message:SetVisible(false)
	self:DisplayUsers()
end

function SharingDialog:DisplayEncounterNotifications(encounterId)
	local notifications = triggers.defaults.encounterNotifications[encounterId]
	local texts = {}
	local indents = {}
	local values = {}
	self.selectedEncounterId = encounterId
	for id, item in pairs(notifications)do
		local notification = triggers.defaults.defaultNotifications[id]
		if(notification)then
			notification.triggers = item.triggers
		else
			-- custom notification
			notification = item
		end
		local n = triggers.Notification.Deserialize(notification)
		if(#n.triggers > 0)then
			local description = n.description
			if(string.len(description) <= 0)then
				description = "Notification Group"
			end
			local myn = triggers.Notifications:GetNotification(n.id)
			table.insert(texts,description)
			table.insert(indents,1)
			table.insert(values,{notification = notification, myNotification = myn, trigger = nil, myTrigger = nil})
			for _, t in ipairs(n.triggers)do
				local lvl = 2
				local myt = nil
				if(myn ~= nil)then
					myt = myn:GetTriggerById(t.id)
					if(myt ~= nil and myt:toString() == t:toString())then
						lvl = 4
					elseif(myt ~= nil)then
						lvl = 3
					end
				end
				table.insert(texts,t:toString())
				table.insert(indents,lvl)
				table.insert(values,{notification = notification, myNotification = myn, trigger = t, myTrigger = myt, })
			end
		end
	end

	self.frame.bosses.triggerList:SetItems(texts,values,indents)
	self.frame.bosses.triggerList:SetLevelFontColor(1,1,1,1)
	self.frame.bosses.triggerList:SetLevelFontColor(2,1,1,0)
	self.frame.bosses.triggerList:SetLevelFontColor(3,1,.5,0)
	self.frame.bosses.triggerList:SetLevelFontColor(4,.7,.7,.7)
	self.frame.bosses.triggerList:SetLevelSelectable(1, true)
	self.frame.bosses.triggerList:SetLevelSelectable(2, true)
	self.frame.bosses.triggerList:SetLevelSelectable(3, true)
	self.frame.bosses.triggerList:SetLevelSelectable(4, false)
	self.frame.bosses.triggerList.Event.ItemClick = function(text,value,_)
		local self = triggers.SharingDialog
		libTDebug.UIDebug:AddTable("Shareing Boss-encounter Clicked",value)
		self.frame.message:SetVisible(true)
		if(value.trigger ~= nil)then
			self.frame.message:SetText(text .. "\r\nWould you like to import this trigger?")
		else
			self.frame.message:SetText("Would you like to import the Notification '" .. text .."'?")
		end
		self.frame.message.Event.okay = function()
			--local self = triggers.SharingDialog
			if(value.trigger == nil and value.myNotification == nil)then
				local newN = triggers.Notification.Deserialize(value.notification, true)
				triggers.Notifications:AddNotification(newN)
			elseif(value.trigger == nil and value.myNotification ~= nil)then
				triggers.Notification.Deserialize(value.notification, true, value.myNotification)
			elseif(value.trigger ~= nil and value.myNotification == nil)then
				local newN = triggers.Notification.Deserialize(value.notification, false)
				local t = triggers.Trigger.Deserialize(value.trigger)
				libTDebug.UIDebug:AddTable("Shareing Boss-encounter New Trigger",t)
				newN:AppendTrigger(t)
				triggers.Notifications:AddNotification(newN)
			elseif(value.trigger ~= nil and value.myNotification ~= nil and value.myTrigger ~= nil)then
				triggers.Notification.Deserialize(value.notification, false, value.myNotification)
				triggers.Trigger.Deserialize(value.trigger, value.myTrigger)
			elseif(value.trigger ~= nil and value.myNotification ~= nil)then
				local t = triggers.Trigger.Deserialize(value.trigger)
				value.myNotification:AppendTrigger(t)
			end
			self.frame.message:SetVisible(false)
			self:DisplayEncounterNotifications(encounterId)
		end
		self.frame.message.Event.cancel = function()
			local self = triggers.SharingDialog
			self.frame.message:SetVisible(false)
			self:DisplayEncounterNotifications(encounterId)
		end
	end
end

function SharingDialog:UISetupBosses()
	self.frame.bosses =  UI.CreateFrame("Frame",self.description .. "People", self.frame)
	self.frame.bosses:SetVisible(false)
	self.frame.bosses:SetWidth(self.size.width)
	self.frame.bosses:SetHeight(self.size.height - 35)
	self.frame.bosses:SetMouseMasking("limited")
	self.frame.bosses:SetLayer(102)
	self.frame.bosses:SetPoint("TOPLEFT", self.frame.header, "BOTTOMLEFT")


	self.frame.bosses.encountersListScrollView = UI.CreateFrame("SimpleScrollView", self.description .. "_TriggerList", self.frame.bosses)
	self.frame.bosses.encountersListScrollView:SetPoint("TOPRIGHT", self.frame.bosses, "TOPRIGHT",-10,3)
	self.frame.bosses.encountersListScrollView:SetWidth(300)
	self.frame.bosses.encountersListScrollView:SetHeight(150)
	self.frame.bosses.encountersListScrollView:SetBorder(1, .5, .5, .5, 1)
	self.frame.bosses.encountersListScrollView:SetBackgroundColor(0,0,0,1)
	self.frame.bosses.encountersList = UI.CreateFrame("SimpleList", self.description .. "_AbilityList_SimpleList", self.frame.bosses.encountersListScrollView)
	self.frame.bosses.encountersList:SetFontSize(14)
	self.frame.bosses.encountersListScrollView:SetContent(self.frame.bosses.encountersList)
	self.frame.bosses.encountersList.dialog = self
	self.frame.bosses.encountersList:SetSelectionMode("single")
	self.frame.bosses.encountersList.Event.ItemClick = function(item,value,_)
		local self = triggers.SharingDialog
		self:DisplayEncounterNotifications(value)
	end
	local encounterNames = {}
	local encounterIds = {}
	local encounterLevels = {}
	local encountersByZone = {}
	local zones = {}

	for encounterId, notification in pairs(triggers.defaults.encounterNotifications)do
		local encounter = libEncounters.EncounterManager:GetEncounter(encounterId)
		if(encounter)then
			if(not encountersByZone[encounter.zone])then
				encountersByZone[encounter.zone] = {}
			end
			table.insert(encountersByZone[encounter.zone],encounter)
		else
			libTDebug.UIDebug:INFO(nil,"Error loading encounterId = %s notificationId = ",encounterId, notification.id)
		end
	end
	for zone, _ in pairs(encountersByZone)do
		table.insert(zones,zone)
	end

	local sort_zones = function( a,b ) return a < b end
	table.sort( zones, sort_zones )

	local sort_encounters = function( a,b ) return a.name < b.name end
	for _, zoneItem in pairs(encountersByZone)do
		table.sort(zoneItem,sort_encounters)
	end

	for _, zoneName in ipairs(zones)do
		table.insert(encounterNames,zoneName)
		table.insert(encounterLevels, 1)
		table.insert(encounterIds, "")

		for _, zoneEncounter in ipairs(encountersByZone[zoneName])do
			local notifications = triggers.defaults.encounterNotifications[zoneEncounter.id]
			local hasTriggers = false
			if(notifications)then
				for _, notification in pairs(notifications)do
					if(#notification.triggers > 0)then
						hasTriggers = true
					end
				end
			end
			if(hasTriggers)then
				table.insert(encounterNames,zoneEncounter.name)
				table.insert(encounterLevels, 2)
				table.insert(encounterIds, zoneEncounter.id)
			end
		end
	end

	self.frame.bosses.encountersList:SetLevelFontColor(1, 1, 1, 0)
	self.frame.bosses.encountersList:SetLevelFontColor(2, 1, .5, 0)
	self.frame.bosses.encountersList:SetLevelSelectable(1, false)
	self.frame.bosses.encountersList:SetLevelSelectable(2, true)
	self.frame.bosses.encountersList:SetItems(encounterNames,encounterIds,encounterLevels)

	self.frame.bosses.triggerListScrollView = UI.CreateFrame("SimpleScrollView", self.description .. "_TriggerList", self.frame.bosses)
	self.frame.bosses.triggerListScrollView:SetPoint("BOTTOMLEFT", self.frame.bosses, "BOTTOMLEFT",10,-10 )
	self.frame.bosses.triggerListScrollView:SetWidth(710)
	self.frame.bosses.triggerListScrollView:SetHeight(230)
	self.frame.bosses.triggerListScrollView:SetBorder(1, .5, .5, .5, 1)
	self.frame.bosses.triggerListScrollView:SetBackgroundColor(0,0,0,1)
	self.frame.bosses.triggerList = UI.CreateFrame("SimpleList", self.description .. "_AbilityList_SimpleList", self.frame.bosses.triggerListScrollView)
	self.frame.bosses.triggerList:SetFontSize(14)
	self.frame.bosses.triggerListScrollView:SetContent(self.frame.bosses.triggerList)
end

function SharingDialog.UISetup(handle,parent)

	libTDebug.UIDebug:AddTable("settings",Inspect.Setting.List())

	local self = triggers.SharingDialog
	self.parent = parent
	self.frame = libTUI.TUIClosableWindow(self.parent, self.description, self.size, "Sharing is caring")

	self.frame.shareType = {}

	self.frame.toggleType = libTUI.ToggleFrame.Create(self.frame
										,"Bosses", "People"
										,nil
										,182
										,"TOPLEFT", self.frame.header, "TOPLEFT" ,10 ,5
										,false
										,false
										,libTUI.ToggleFrame.Borders.All
										,{1,1,1,1})

	self.frame.toggleType.onChanged = function()
		if(self.frame.toggleType.enabled)then
			self.frame.people:SetVisible(false)
			self.frame.bosses:SetVisible(true)
		else
			self.frame.people:SetVisible(true)
			self.frame.bosses:SetVisible(false)
		end
	end
	self:UISetupPeople()
	self:UISetupBosses()
	self.frame.people:SetVisible(true)
	self.frame.bosses:SetVisible(false)

end

function SharingDialog:RequestTriggers()
	self.frame.error:SetText("")
	self.frame.people.triggerList:SetItems({})
	self.NotificationId = nil
	if (self.selectedUser == triggerDefaults  and self.inspectType ~= nil) then
		-- default triggers
		local fetchId = self.inspectTypesMap[self.inspectType]
		if(fetchId)then
			libTDebug.UIDebug:DEBUG(nil, "Default TriggerList")

			local dn = triggers.Notification.Deserialize(triggers.defaults.defaultNotifications[fetchId])
			local n = triggers.Notifications:GetNotification(dn.id)
			local tt = dn:StringTriggerTable()
			self:ParseTriggers(tt,n)
			self.frame.people.triggerList.Event.ItemSelect = function(_,text,triggerId,_)
				local self = triggers.SharingDialog
				self.frame.message:SetVisible(true)
				self.frame.message:SetText(text .. "\r\nWould you like to import this trigger?")
				self.frame.message.Event.okay = function()
					local self = triggers.SharingDialog
					self.frame.message:SetVisible(false)
					local t = n:GetTriggerById(triggerId)
					if(t == nil)then
						t = dn:GetTriggerById(triggerId)
						n:PrependTrigger(t)
					else
						local ct = dn:GetTriggerById(triggerId)
						triggers.Trigger.Deserialize(ct:Serialize(), t)
					end
					n = triggers.Notifications:GetNotification(dn.id)
					tt = dn:StringTriggerTable()
					self:ParseTriggers(tt,n)
				end
				self.frame.message.Event.cancel = function()
					local self = triggers.SharingDialog
					self.frame.message:SetVisible(false)
				end
			end
		else
			self.frame.error:SetText("No Others for Triggers Defaults")
		end
	elseif (self.selectedUser ~= nil and self.inspectType ~= nil)then
		-- users from guild or raids
		local fetchId = self.inspectTypesMap[self.inspectType]
		if(fetchId)then
			libTDebug.UIDebug:DEBUG(nil, "Requesting TriggerList - From:%s Trigger:%s", self.selectedUser, fetchId)
			Command.Message.Send(self.selectedUser, "Triggers_RequestTriggerList",fetchId, Sharing.Callback)
			self.NotificationId = fetchId
			self.frame.people.triggerList.Event.ItemSelect = function(_,t,triggerId,_)
				local self = triggers.SharingDialog
				local n = triggers.Notifications:GetNotification(self.NotificationId)
				local tt = n:GetTriggerById(triggerId)
				if(tt == nil)then
					self.frame.message:SetText(t .. "\r\nWould you like to import this trigger?")
				else
					local tstring = tt:toString()
					self.frame.message:SetText(tstring .. "\r\nWould you like to replace the above trigger with the below?\n\n" .. t)
				end
				self.frame.message:SetVisible(true)
				self.frame.message.Event.okay = function()
					local self = triggers.SharingDialog

					self.frame.message:SetVisible(false)
					libTDebug.UIDebug:DEBUG(nil, "Requesting TriggerDetails - To:%s Trigger:%s", self.selectedUser, triggerId)
					Command.Message.Send(self.selectedUser, "Triggers_RequestTriggerDetails", self.NotificationId .. "|" .. triggerId, Sharing.Callback)
				end
				self.frame.message.Event.cancel = function()
					local self = triggers.SharingDialog
					self.frame.message:SetVisible(false)
				end
			end
		else
			libTDebug.UIDebug:DEBUG(nil, "Requesting Notifications By Role - To:%s ", self.selectedUser)
			Command.Message.Send(self.selectedUser, "Triggers_RequestRoles","", Sharing.Callback)
			self.search = {}
			self.search.selectedRole = nil
			self.search.requestedNotification = nil
			self.search.requestedTrigger = nil
		end
	end
end

function Sharing.Messages.Triggers_RequestRoles(sender, data, msgType)
	libTDebug.UIDebug:INFO(nil, "Share: Triggers_RequestRoles- From:%s", sender)
	local uniqueRoles = triggers.Notifications:GetUniqueRoles()

	libTMessaging.SendMessage(sender, uniqueRoles,"Triggers_Roles", Sharing.Callback)
end

function Sharing.Messages.Triggers_Roles(sender, data, msgType)
	libTDebug.UIDebug:INFO(nil, "Share: Triggers_NotificationList- From:%s", sender)
	local self = triggers.SharingDialog
	self.notificationRoleList = libTMessaging.ParseMessage(data, self.frame.error)
	if(self.notificationRoleList == nil)then return end

	self:ParseRoles(self.notificationRoleList, self.frame.people.filterText:GetText())
end

function SharingDialog.MessageReceived(handle, sender, msgType, channel, id, data)
	if(sender ~= triggers.UnitId and Sharing.Messages[id] ~= nil)then
		Sharing.Messages[id](sender,data, msgType)
	end
end

function SharingDialog:DisplayUsers()
	local names = {triggerDefaults}
	for name, _ in pairs(self.users)do
		table.insert(names,name)
	end
	self.frame.people.userList:SetItems(names)
end

function SharingDialog:AddUser(user, msgType)
	if(self.users[user] == nil)then
		self.users[user] = msgType
	end
	if(self.users[user] ~= "guild" and msgType == "guild") then
		self.users[user] = msgType
	end
	self:DisplayUsers()
end

function Sharing.Messages.Triggers_AWK(sender, data, msgType)
	libTDebug.UIDebug:INFO(nil, "Share: Request AWK- From:%s", sender)
	Command.Message.Send(sender, "Triggers_ACK", "", Sharing.Callback)
end

function Sharing.Messages.Triggers_ACK(sender, data, msgType)
	libTDebug.UIDebug:INFO(nil, "Share: Request ACK- From:%s", sender)
	triggers.SharingDialog:AddUser(sender, msgType)
end

function Sharing.Messages.Triggers_RequestTriggerList(sender, data, msgType)
	libTDebug.UIDebug:DEBUG(nil, "Share: Trigger List Request- From:%s Trigger:%s", sender, data)
	local n = triggers.Notifications:GetNotification(data)
	local tt = n:StringTriggerTable()
	local s = ""
	for id, v in pairs(tt)do
		s = s .. id .. "|" .. v .. "|"
	end
	libTDebug.UIDebug:DEBUG(nil, "Share: Trigger List Request Data:%s", s)
	local cd = zlib.deflate(zlib.BEST_COMPRESSION)(s, "finish")
	Command.Message.Send(sender, "Triggers_TriggerList", cd, Sharing.Callback)
end

function SharingDialog:ParseTriggers(t, n, f)
	self.currentTriggerTable = t
	self.currentNotification = n
	local filter = libTUtility.Trim(f)
	local texts = {}
	local values = {}
	local indents = {}
	for id, v in pairs(t) do
		local lvl = 1
		local trigger = n:GetTriggerById(id)
		if(trigger ~= nil and trigger:toString() == v)then
			lvl = 3
		elseif(trigger ~= nil)then
			lvl = 2
		end
		if(filter == nil or filter == "" or string.find(string.lower(v), string.lower(filter)))then
			table.insert(indents,lvl)
			table.insert(texts,v)
			table.insert(values,id)
		end
	end
	self.frame.people.triggerList:SetItems(texts,values,indents)
	self.frame.people.triggerList:SetLevelFontColor(1,1,1,0)
	self.frame.people.triggerList:SetLevelFontColor(2,1,.5,0)
	self.frame.people.triggerList:SetLevelFontColor(3,.7,.7,.7)
	self.frame.people.triggerList:SetLevelSelectable(3, false)
	self.frame.people.applyFilter.Event.LeftClick = function()
		self:ParseTriggers(self.currentTriggerTable,self.currentNotification,self.frame.people.filterText:GetText())
	end
end

-- response from user - data is a list of triggers for not other.
function Sharing.Messages.Triggers_TriggerList(sender, data, msgType)
	libTDebug.UIDebug:DEBUG(nil, "Share: Trigger List Response- From:%s", sender)
	local self = triggers.SharingDialog
	local n = triggers.Notifications:GetNotification(self.NotificationId)
	if(sender == self.selectedUser)then
		local d, eof = zlib.inflate()(data)

		if not eof then
			libTDebug.UIDebug:DEBUG(nil, "Share: There is an error decompressing the string")
			self.frame.error:SetText("There is an error decompressing the string")
			return
		end
		libTDebug.UIDebug:DEBUG(nil, "Share: Trigger List Response Data:%s", d)
		local t = libTUtility.StringSplitToTable(d,"|")

		self:ParseTriggers(t,n)
	end
end

function Sharing.Messages.Triggers_RequestTriggerDetails(sender, data, msgType)
	local t = libTUtility.StringSplit(data, "|")
	local nId = t[1]
	local tId = t[2]

	local notification = triggers.Notifications:GetNotification(nId)
	local trigger = notification:GetTriggerById(tId)
	local triggerTable = trigger:Serialize()
	triggerTable.notificationId = nId

	libTMessaging.SendMessage(sender, triggerTable,"Triggers_TriggerDetails", Sharing.Callback)
end

function Sharing.Messages.Triggers_TriggerDetails(sender ,data, msgType)
	libTDebug.UIDebug:DEBUG(nil, "Share: Triggers_TriggerDetails Response- From:%s", sender)
	local self = triggers.SharingDialog
	returnData = libTMessaging.ParseMessage(data, self.frame.error)
	if(returnData == nil)then return end

	local notification = triggers.Notifications:GetNotification(returnData.notificationId)
	if(notification == nil)then
		libTDebug.UIDebug:DEBUG(nil, "Share: Notification not found")
		self.frame.error:SetText("Notification not found")
		return
	end
	local trigger = notification:GetTriggerById(returnData.id)
	local newTrigger = false
	if(trigger == nil)then
		newTrigger = true
		trigger = triggers.Trigger.Create()
	end
	triggers.Trigger.Deserialize(returnData,trigger)
	if(newTrigger)then
		notification:PrependTrigger(trigger)
	end
	local encounterId = returnData.encounterId

	local encounter = libEncounters.EncounterManager:GetEncounter(encounterId)
	if(encounter == nil)then
		libTDebug.UIDebug:DEBUG(nil, "Requesting Encounter - From:%s Encounter:%s", self.selectedUser, returnData.encounterId)
		Command.Message.Send(self.selectedUser, "Triggers_RequestEncounterDetails",returnData.encounterId, Sharing.Callback)
	end

	local ability = libAbilityCatalog.Catalog:GetCatalogItem(trigger.abilityId)
	if(ability == nil)then
		libTDebug.UIDebug:DEBUG(nil, "Requesting Ability - From:%s Ability:%s", self.selectedUser, trigger.abilityId)
		Command.Message.Send(self.selectedUser, "Triggers_RequestAbilityDetails",trigger.abilityId, Sharing.Callback)
	end
	self:RequestTriggers()
end

function Sharing.Messages.Triggers_RequestAbilityDetails(sender, data, msgTypes)
	libTDebug.UIDebug:DEBUG(nil, "Share: Triggers_RequestAbilityDetails - From:%s ability:%s", sender,data)
	local ability = libAbilityCatalog.Catalog:GetCatalogItem(data)
	local abilityTable = ability:Serialize()

	libTMessaging.SendMessage(sender, abilityTable,"Triggers_AbilityDetails", Sharing.Callback)
end

function Sharing.Messages.Triggers_AbilityDetails(sender, data, msgTypes)
	libTDebug.UIDebug:DEBUG(nil, "Share: Triggers_AbilityDetails - From:%s", sender,data)

	local self = triggers.SharingDialog
	local d, eof = zlib.inflate()(data)
	if not eof then
		libTDebug.UIDebug:DEBUG(nil, "Share: There is an error decompressing the string")
		self.frame.error:SetText("There is an error decompressing the string")
		return
	end

	local returnData = loadstring("return " .. d)()
	local ability = libAbilityCatalog.CatalogItem.Deserialize(returnData.id, returnData)

	libAbilityCatalog.Catalog:AddCatalogItem(ability)
end

function Sharing.Messages.Triggers_RequestEncounterDetails(sender, data, msgType)
	libTDebug.UIDebug:DEBUG(nil, "Share: Triggers_RequestEncounterDetails - From:%s Encounter:%s", sender,data)
	local encounter = libEncounters.EncounterManager:GetEncounter(data)
	local encounterTable = encounter:Serialize()

	libTMessaging.SendMessage(sender, encounterTable,"Triggers_EncounterDetails", Sharing.Callback)
end

function Sharing.Messages.Triggers_EncounterDetails(sender, data, msgType)
	libTDebug.UIDebug:DEBUG(nil, "Share: Triggers_EncounterDetails Response- From:%s", sender)

	local self = triggers.SharingDialog
	returnData = libTMessaging.ParseMessage(data, self.frame.error)
	if(returnData == nil)then return end

	local encounter = libEncounters.Encounter.Deserialize(returnData)

	libEncounters.EncounterManager:AddEncounter(encounter)
end

function SharingDialog:ParseRoles(t, f)
	self.currentTable = t
	local filter = libTUtility.Trim(f)
	local texts = {}
	local values = {}
	local indents = {}
	for _, v in ipairs(t) do
		local lvl = 1
		if(filter == nil or filter == "" or string.find(string.lower(v), string.lower(filter)))then
			table.insert(indents,lvl)
			table.insert(texts,v)
			table.insert(values,"ROLE")
		end
	end
	self.frame.people.triggerList:SetItems(texts,values,indents)
	self.frame.people.triggerList:SetLevelFontColor(1,1,1,1)
	self.frame.people.triggerList.Event.ItemSelect = function(_,role,_,_)
		self.search.selectedRole = role
		libTDebug.UIDebug:DEBUG(nil, "Requesting Notification Role %s- To:%s ", role, self.selectedUser)
		Command.Message.Send(self.selectedUser, "Triggers_RequestNotificationsByRole",role, Sharing.Callback)
	end
	self.frame.people.applyFilter.Event.LeftClick = function()
		self:ParseRoles(self.currentTable,self.frame.people.filterText:GetText())
	end
end

function Sharing.Messages.Triggers_RequestNotificationsByRole(sender, data, msgType)

	libTDebug.UIDebug:INFO(nil, "Share: Triggers_RequestNotificationsByRole- From:%s Role:%s", sender, data)
	local notifications
	if(data == "All Roles")then
		notifications = triggers.Notifications:GetByNoRolePopulated(data)
	else
		notifications = triggers.Notifications:GetByRoleMatch(data)
	end
	local triggerTexts = {}
	for _, n in pairs(notifications)do
		local triggerList = {}
		for _, trigger in ipairs(n.triggers) do
			table.insert(triggerList, { id=trigger.id, text = trigger:toString()} )
		end
		table.insert(triggerTexts, { notificationId = n.id, triggers = triggerList, description=n.description, displayIf = triggers.DisplayIf.toString(n.displayIf) })
	end
	local response = { role = data,  texts = triggerTexts }

	libTMessaging.SendMessage(sender, response,"Triggers_NotificationsByRole", Sharing.Callback)
end

function Sharing.Messages.Triggers_NotificationsByRole(sender, data, msgType)
	libTDebug.UIDebug:INFO(nil, "Share: Triggers_NotificationByRole- From:%s", sender)
	local self = triggers.SharingDialog

	local responseData = libTMessaging.ParseMessage(data, self.frame.error)
	if(responseData == nil)then return end

	self:Notifications(responseData,self.frame.people.filterText:GetText())
end

function SharingDialog:Notifications(t, f)
	self.currentTable = t
	local filter = libTUtility.Trim(f)
	local texts = {}
	local values = {}
	local indents = {}
	for _, v in ipairs(t.texts) do
		local notificationMatch = true
		local triggerMatch = true
		if(filter ~= nil and filter ~= "")then
			notificationMatch = false
			triggerMatch = false
			if (string.find(string.lower(v.description), string.lower(filter)))then
				notificationMatch = true
			else
				for _, triggerInfo in ipairs(v.triggers) do
					if (string.find(string.lower(triggerInfo.text), string.lower(filter)))then
						triggerMatch = true
					end
				end
			end
		end
		if(notificationMatch or triggerMatch)then
			local lvl = 1
			table.insert(indents,lvl)
			local description = v.description
			if(string.len(description) <= 0)then
				description = "Notification Group"
			end
			table.insert(texts, string.format("%s (%s triggers match)",description,v.displayIf))
			table.insert(values,{ notificationId = v.notificationId })
			local n = triggers.Notifications:GetNotification(v.notificationId)
			local notFiltered = true
			for _, triggerInfo in ipairs(v.triggers) do
				lvl = 2
				if(n ~= nill)then
					local trigger = n:GetTriggerById(triggerInfo.id)
					if(trigger ~= nil and trigger:toString() == triggerInfo.text)then
						lvl = 4
					elseif(trigger ~= nil)then
						lvl = 3
					end
				end
				table.insert(indents,lvl)
				table.insert(texts,triggerInfo.text)
				table.insert(values,{ notificationId =  v.notificationId, triggerId = triggerInfo.id })
			end
		end
	end
	self.frame.people.triggerList:SetItems(texts,values,indents)
	self.frame.people.triggerList:SetLevelFontColor(1,1,1,1)
	self.frame.people.triggerList:SetLevelFontColor(2,1,1,0)
	self.frame.people.triggerList:SetLevelFontColor(3,1,.5,0)
	self.frame.people.triggerList:SetLevelFontColor(4,.7,.7,.7)
	self.frame.people.triggerList.Event.ItemSelect = function(_,text,triggerInfo,_)
		self.search.requestedNotification = triggerInfo.notificationId
		self.search.requestedTrigger = triggerInfo.triggerId

		self.frame.message:SetVisible(true)
		if(self.search.requestedTrigger ~= nil)then
			self.frame.message:SetText(text .. "\r\nWould you like to import this trigger?")
		else
			self.frame.message:SetText("Would you like to import the Notification '" .. text .."'?")
		end
		self.frame.message.Event.okay = function()
			local self = triggers.SharingDialog
			self.frame.message:SetVisible(false)
			libTDebug.UIDebug:DEBUG(nil, "Requesting Notification ID %s- To:%s ", self.search.requestedNotification, self.selectedUser)
			Command.Message.Send(self.selectedUser, "Triggers_RequestNotificationsById", self.search.requestedNotification, Sharing.Callback)
		end
		self.frame.message.Event.cancel = function()
			local self = triggers.SharingDialog
			self.frame.message:SetVisible(false)
		end

	end
	self.frame.people.applyFilter.Event.LeftClick = function()
		self:Notifications(self.currentTable,self.frame.people.filterText:GetText())
	end
end

function Sharing.Messages.Triggers_RequestNotificationsById(sender, data, msgType)
	libTDebug.UIDebug:INFO(nil, "Share: Triggers_RequestNotificationsById- From:%s notificationID:%s", sender,data)
	local self = triggers.SharingDialog
	local notification = triggers.Notifications:GetNotification(data)
	ntosend = notification:Serialize()
	ntosend.clientSize = {width = math.floor(UIParent:GetWidth()), height = math.floor(UIParent:GetHeight())}

	libTMessaging.SendMessage(sender, ntosend,"Triggers_NotificationsById", Sharing.Callback)
end

function Sharing.Messages.Triggers_NotificationsById(sender, data, msgType)
	libTDebug.UIDebug:INFO(nil, "Share: Triggers_NotificationsById- From:%s", sender)
	local self = triggers.SharingDialog

	local notification = libTMessaging.ParseMessage(data, self.frame.error)
	if(notification == nil)then return end

	local n = triggers.Notifications:GetNotification(notification.id)

	if(n == nil and  self.search.requestedTrigger == nil)then
		n = triggers.Notification.Deserialize(notification)
	elseif(n==nil)then
		n = triggers.Notification.Deserialize(notification, false)
	end

	local importNotification = triggers.Notification.Deserialize(notification)
	if (notification.clientSize ~= nil)then
		importNotification:Resize(notification.clientSize)
	end
	if(self.search.requestedTrigger ~= nil)then
		local t = importNotification:GetTriggerById(self.search.requestedTrigger)
		n:PrependTrigger(t)
	end
	triggers.Notifications:AddNotification(n)
	-- parse all triggers in the notification received,  insures we are getting all information
	-- could have used the generated notifcation(n) but that does not have all the users triggers.
	for _, trigger in pairs(notification.triggers)do
		local encounter = libEncounters.EncounterManager:GetEncounter(trigger.encounterId)
		if(encounter == nil and trigger.encounterId ~= nil and string.len(trigger.encounterId) > 0) then
			libTDebug.UIDebug:DEBUG(nil, "Requesting Encounter - From:%s Encounter:%s", self.selectedUser, trigger.encounterId)
			Command.Message.Send(self.selectedUser, "Triggers_RequestEncounterDetails",trigger.encounterId, Sharing.Callback)
		end
		local ability = libAbilityCatalog.Catalog:GetCatalogItem(trigger.abilityId)
		if(ability == nil and trigger.abilityId ~= nil and string.len(trigger.abilityId) > 0)then
			libTDebug.UIDebug:DEBUG(nil, "Requesting Ability - From:%s Ability:%s", self.selectedUser, trigger.abilityId)
			Command.Message.Send(self.selectedUser, "Triggers_RequestAbilityDetails",trigger.abilityId, Sharing.Callback)
		end
	end

	self.search = {}
	self:Notifications(self.currentTable,self.frame.people.filterText:GetText())
end

function Sharing.Callback(failure,message)
	if(failure)then
		libTDebug.UIDebug:INFO(nil, "Share: (Callback) %s", message)
	end
end

for message, _ in pairs(Sharing.Messages) do
	Command.Message.Accept(nil, message)
end

Command.Event.Attach(Event.Message.Receive, SharingDialog.MessageReceived, "SharingDialog.MessageReceived")
Command.Event.Attach(Triggers.Event.ContextCreated , SharingDialog.UISetup, "SharingDialog.UISetup")


triggers.SharingDialog = SharingDialog.Create()
