--
--  Class Expression

local addonData, triggers = ...

triggers.Expression = {}
local Expression = triggers.Expression
Expression.__index = Expression

Expression.rolesMap = {
		["dps"] = libTConstants.Roles.DPS,
		["tank"] =  libTConstants.Roles.TANK,
		["heal"] =  libTConstants.Roles.HEALER,
		["support"] =  libTConstants.Roles.SUPPORT
	}

function Expression.Create(notification, inCombat)
	if notification.disabled then return nil end
	local obj = {}
	setmetatable(obj,Expression)
	obj.notification = notification

	-- exit if the spec is not set, means user has no points in role - error
	if(not obj.notification:SpecMatch(obj:Spec())) then
		libTDebug.UIDebug:INFO(nil,"Notification: %s drop notification due to role match %s no match %s", obj.notification.id,obj.notification.roles, obj:Spec())
		return
	end
	libTDebug.UIDebug:INFO(nil,"Notification: %s Included", obj.notification.id)

	-- Create the obj.CheckTriggers function based on when the trigger should fire
	if(notification.displayIf == triggers.DisplayIf.ANY)then
		-- when the trigger is display if any, itterate until first tigger matches
		obj.CheckTriggers = function(self,state)
			for _, value in ipairs(self.triggers) do
				if(value.Match(self,value.trigger,state))then
					state.displayAbilityId = value.trigger.abilityId
					state.text = value.trigger.text
					state.color = value.trigger.color
					state.opacity = value.trigger.opacity
					state.visible = true
					state.showSpark = value.trigger.showSpark
					return state
				end
				-- clear state for next
			state = {}
			end
			return state
		end
	elseif(notification.displayIf == triggers.DisplayIf.ALL)then
		-- All triggers must match return once first does not
		obj.CheckTriggers = function(self,state)
			local i = 1
			while(i <= #self.triggers and self.triggers[i].Match(self,self.triggers[i].trigger,state) == true) do
				state = {}
				i = i + 1
			end
			state.displayImage = self.triggers[1].trigger.abilityId
			state.visible = i > #self.triggers
			return state
		end
	elseif(notification.displayIf == triggers.DisplayIf.NONE)then
		-- all triggers must not match return if one does
		obj.CheckTriggers = function(self,state)
			local i = 1
			while(i <= #self.triggers and self.triggers[i].Match(self,self.triggers[i].trigger,state) == false) do
				i = i + 1
				state = {}
			end
			state.visible = i == #self.triggers
			return state
		end
	end
	local player = Inspect.Unit.Detail("player")
	local currentRole = Expression.rolesMap[player.role]

 	--register all buffs with buff Manager
 	-- we ignore combat flags or anything that is dynamic.
	for _, trigger in ipairs(notification.triggers) do
		(function()
			if(trigger.type == libTConstants.CatalogItemType.BUFF) then
				if (trigger.disabled) then return end
				if (currentRole ~= nil and bit.band(trigger.roles,currentRole) ~= currentRole)then return end
				--noticing buffs that do not have the corrisponding correct ability id
				--if (trigger.filtered and trigger.type == libTConstants.CatalogItemType.BUFF and trigger.selfCast)then
				--	 if(trigger.abilityNew and not libAbilityCatalog.AbilityManager.currentAbilities[trigger.abilityNew]) then return end
				--end
				libAbilityCatalog.BuffManager:Register(triggers.addonId, trigger.abilityId, trigger.unit)
			end
		end)()
	end

	-- if CombatOnly then we must check if in Combat otherwise, we can skip
	-- the execution of this function and assign Show the next step
	if(not notification.combatOnly or inCombat)then
		obj.Show = obj.CheckTriggers
	else
		return nil
	end

	obj.triggers = {}
	for _, trigger in ipairs(notification.triggers) do
		(function()
			libTDebug.UIDebug:INFO(nil,"Trigger: %s", trigger.id)
		-- only add the triggers to the expression when not disabled and our current role matches
			if (trigger.disabled) then libTDebug.UIDebug:INFO("Drop Trigger is disabled") return end
			if (currentRole ~= nil and bit.band(trigger.roles,currentRole) ~= currentRole)then libTDebug.UIDebug:INFO("Drop Trigger role missmatch") return end
			if (not obj:EncounterMatch(trigger.encounterId)) then libTDebug.UIDebug:INFO("Drop Trigger Encounter missmatch") return end
			if (trigger.filtered and trigger.type == libTConstants.CatalogItemType.ABILITY and not libAbilityCatalog.AbilityManager.currentAbilities[trigger.abilityId]) then libTDebug.UIDebug:INFO("Drop Trigger ability filtered, missing ability") return end

			if(trigger.type == libTConstants.CatalogItemType.BUFF) then
				table.insert(obj.triggers, {
					["trigger"] = trigger,
					["Match"] = function (self,trigger,state)
												local unit, buff, relationMatch, now = self:UnitBuff(trigger)
												--print(string.format("%s %s %s",unit,buff,relationMatch))
												if not relationMatch then return false end
												if not buff then return self:NoBuff(trigger) end

												if trigger.timer and buff.expires ~= nil then
													state.timeLeft = buff.expires - now
												end
												if trigger.stacker and buff.stack ~= nil then
													state.stacks = buff.stack
												end
												if not (trigger.stacker or trigger.timer)then
													return trigger.buffState ~= libTConstants.BuffState.MISSING
												else
													if(trigger.timer and trigger.stacker)then
														return (trigger.timer and ((state.timeLeft or 0) - trigger.time) * trigger.timerCompare >= 0) or
																	 (trigger.stacker and ((state.stacks or 0) - (trigger.stacks or 0)) * trigger.stackCompare >= 0)
													elseif trigger.timer then
														return (trigger.timer and ((math.floor(state.timeLeft or 0)) - (trigger.time or 0)) * trigger.timerCompare >= 0)
													elseif trigger.stacker then
														return (trigger.stacker and ((state.stacks or 0) - (trigger.stacks or 0)) * trigger.stackCompare >= 0)
													end
												end
										  end})
			elseif(trigger.type == libTConstants.CatalogItemType.ABILITY) then
				table.insert(obj.triggers, {
					["trigger"] = trigger,
					["Match"] = function (self,trigger,state)
												local onCD = libAbilityCatalog.AbilityManager.ability[trigger.abilityId] ~= nil
												if(onCD)then
													local timeLeft = libAbilityCatalog.AbilityManager.ability[trigger.abilityId] - Inspect.Time.Frame()
													if(trigger.timer)then
														state.timeLeft = timeLeft
														return (((math.floor(state.timeLeft or 0)) - (trigger.time or 0)) * trigger.timerCompare >= 0)
													end
												else
													state.timeLeft = nil
													if(trigger.abilityState == libTConstants.AbilityState.COOLDOWN)then
														return false
													else
														return true
													end
												end
												return false
											end })
			elseif(trigger.type == libTConstants.CatalogItemType.RESOURCE) then
				table.insert(obj.triggers, {
					["trigger"] = trigger,
					["Match"] = function (self,trigger,state)
												return false
											end })
			elseif(trigger.type == libTConstants.CatalogItemType.CASTBAR) then
				table.insert(obj.triggers, {
					["trigger"] = trigger,
					["Match"] = function (self,trigger,state)
												local unit = libAbilityCatalog.GetUnit(trigger.unit)
												if(unit)then
													if(libAbilityCatalog.unitsRelation[unit] == trigger.relation)then

														 local castDetails = libAbilityCatalog.CastbarManager.casting[unit]
														 return castDetails and castDetails.abilityName == trigger.name
													end
												end
												return false
											end })
			end
		end)()
	end
	if(#obj.triggers > 0)then
		return obj
	end
	return
end

function Expression:NoBuff(trigger)
	if trigger.buffState == libTConstants.BuffState.MISSING then
		return true and ((not trigger.timer and not trigger.stacker) or
										 (trigger.timer and trigger.timerCompare < 0) or
										 (trigger.stacker and trigger.stackCompare < 0))
	end
	return false
end

function Expression:UnitBuff(trigger)
		local unit = libAbilityCatalog.GetUnit(trigger.unit)

		local relationMatch = nil
		local buff = nil
		if(unit) then
			relationMatch = libAbilityCatalog.unitsRelation[unit] == trigger.relation or trigger.unit == libTConstants.Unit.PLAYER
			if(libAbilityCatalog.BuffManager.units[unit] ~= nil)then
				if(libAbilityCatalog.BuffManager.units[unit][trigger.abilityId] ~= nil)then
					if(trigger.selfCast)then
						buff = libAbilityCatalog.BuffManager.units[unit][trigger.abilityId].player
					else
						buff = libAbilityCatalog.BuffManager.units[unit][trigger.abilityId].others
					end
				end
			end
		end
		--libTDebug.UIDebug:DEBUG(nil, "UnitBuff: %s %s %s %s %s %s", unit, buff, tostring(relationMatch), libAbilityCatalog.unitsRelation[unit], trigger.relation, trigger.unit)
		return unit, buff, relationMatch, Inspect.Time.Frame()
end

function Expression:InCombat()
	return triggers.ExpressionsManager.inCombat
end

function Expression:Spec()
		local roleList = Inspect.Role.List()
		local slot = Inspect.TEMPORARY.Role()
		if(slot)then
			self.role = roleList[string.format("h%016X", slot - 1)]
		end
	  return self.role
end

function Expression:EncounterMatch(id)
	if(id == nil or string.len(id) == 0 ) then return true end
	if(triggers.ExpressionsManager.encounter == nil)then return false end
	return triggers.ExpressionsManager.encounter.id == id
end