--
-- Class ExpressionsManager
--
--   Manages a list of ExpressionsManager

local addonData, triggers = ...

local ExpressionsManager = {}
ExpressionsManager.__index = ExpressionsManager

function ExpressionsManager.Create()
	local obj = {}
	setmetatable(obj,ExpressionsManager)
	obj:Init()
	return obj
end

function ExpressionsManager:Init()
	self.batchCount = 10
	self.expressions = {}
	self.index = 1
	self.enabled = false
	self.inCombat = false
	self.nextUpdate =Inspect.Time.Frame()
	self.lastCPUUpdate = Inspect.Time.Frame()
	libTDebug.UIDebug:AddTable("ExpressionsManager.expressions",self.units)
end

function ExpressionsManager:BuildExpressions()
	-- clear the registered buffs we are watching
	libAbilityCatalog.BuffManager:UnregisterAll(triggers.addonId)
	self.expressions = {}
	libTDebug.UIDebug:AddTable("ExpressionsManager.expressions",self.units)
	self.index = 1

	for _, notification in pairs(triggers.Notifications.items) do
		notification:SetViewState()
		local e = triggers.Expression.Create(notification, self.inCombat)
		if(e ~= nil)then
			table.insert(self.expressions, {notification = notification,
								  										expression = e,
								  										state = {  } })
		end
	end
	self.index = 1
end

function ExpressionsManager:EnableExpressions(enable)
	self.enabled = enable
	self.expressions = {}
	self.index = 1

	if(self.enabled) then
		self:BuildExpressions()
	else
		triggers.Notifications:HideNotificitions()
	end
end

function ExpressionsManager.ExpressionsRun()
	local self = triggers.ExpressionsManager

  local now = Inspect.Time.Frame()
  if(self.nextUpdate < now)then
  	if #self.expressions > 0 then
	  	local batchEnd = self.index + self.batchCount
			for i = self.index,batchEnd do

		  	local item = self.expressions[self.index]
		  	local newState = item.expression:Show( {} )
		  	item.notification:SetViewState(newState)
			if self.index == #self.expressions then
				self.index = 1
				break
			else
				self.index = self.index + 1
			end
		  end
		end
		self.nextUpdate = Inspect.Time.Frame() + .1
	end
end

function ExpressionsManager.RoleChange(handle,slot)
	local self = triggers.ExpressionsManager
	if(self.enabled)then
		self:BuildExpressions()
	end
end

function ExpressionsManager.NotificationsLoaded(handle, units)
	local self = triggers.ExpressionsManager
	self.loaded = true
	self:EnableExpressions(true)
end

function ExpressionsManager.EncounterStart(handle, encounter)
	local self = triggers.ExpressionsManager
	self.inCombat = true
	self.encounter = encounter
	if(self.enabled)then
		self:BuildExpressions()
	end
end

function ExpressionsManager.EncounterEnd(handle)
	local self = triggers.ExpressionsManager
	self.inCombat = false
	self.encounter = nil
	if(self.enabled)then
		self:BuildExpressions()
	end
end

triggers.ExpressionsManager = ExpressionsManager.Create()

Command.Event.Attach(Event.System.Update.Begin, ExpressionsManager.ExpressionsRun, "ExpressionsManager.Update")
Command.Event.Attach(Event.TEMPORARY.Role,ExpressionsManager.RoleChange,"ExpressionsManager.RoleChange")

Command.Event.Attach(Triggers.Event.NotificationsLoaded , ExpressionsManager.NotificationsLoaded, "ExpressionsManager.NotificationsLoaded")
Command.Event.Attach(libEncounters.Event.EncounterStart, ExpressionsManager.EncounterStart, "ExpressionsManager.EncounterStart")
Command.Event.Attach(libEncounters.Event.EncounterEnd, ExpressionsManager.EncounterEnd, "ExpressionsManager.EncounterEnd")

