--
-- Class Catalog
--
--   Manages a list of Catalog

libAbilityCatalog = ...

local Catalog = {}
Catalog.__index = Catalog

function Catalog.Create()
	local obj = {}
	setmetatable(obj,Catalog)
	obj:Init()
	return obj
end

function Catalog:Init()
	self.items = {}
	libTDebug.UIDebug:AddTable("Catalog.items",self.items)
	self.version = 1.3
	self.queue = {byId={}, byName={}, byBuffTypeId={}}
	self.lastProcessed = Inspect.Time.Frame()
	self.ignoreNames = {}
	self.ignoreNames["Changing Role"] = true
	self.Upgrade = "Unfortunatly LibAbilityCatalog has upgraded\n\nIn doing so, the Triggers you have setup will need\n to be reviewed as they may no longer work.\n\nSpecificaly buff tracking has been changed so casting\n and ability triggers should be in working order.\n\n In most cases this means re-selecting the buff\n in the dropdown for each trigger again.\n\nThese new changes include code optimizations.\n  I hope you all see increased performance.\n  I also hope that these changes will increase reliablility\n  in the notifications and better awareness of abilities\n  that apply multiple buffs of the same name."
end

function Catalog.OnVariablesLoaded(event, addonId)
	self = libAbilityCatalog.Catalog
	if addonId ~= libAbilityCatalog.addonId then return end
	LIBA_CATALOG = LIBA_CATALOG or {version = self.version}
	if(LIBA_CATALOG.version == self.version)then
		for key, value in pairs(LIBA_CATALOG) do
			if(key ~= "version")then
				local item = libAbilityCatalog.CatalogItem.Deserialize(key, value)
				self.items[item.id] = item
			end
		end
	else
		self.UIContext = UI.CreateContext("LAMainContext")
		self.frame = UI.CreateFrame("SimpleWindow","Upgrade", self.UIContext)
		self.frame:SetMouseMasking("limited")
		self.frame:SetLayer(101)
		self.frame.config = self
		self.frame:SetPoint("CENTER",UIParent,"CENTER")
		self.frame:SetVisible(true)
		self.frame:SetCloseButtonVisible(true)
		self.frame.Upgrade = UI.CreateFrame("Text", "UpgradeText" .. "_text", self.frame)
		self.frame.Upgrade:SetPoint("TOPLEFT",self.frame,"TOPLEFT",20,60)
		self.frame.Upgrade:SetVisible(true)
		self.frame.Upgrade:SetText(self.Upgrade,true)
		self.frame.Upgrade:SetFontSize(15)

		LIBA_CATALOG = {version = self.version}
	end
end

function Catalog.OnVariablesSaving(event, addonId)
	if addonId ~= libAbilityCatalog.addonId then return end
	self = libAbilityCatalog.Catalog
	LIBA_CATALOG = {version = LIBA_CATALOG.version or self.version}

	for id, item in pairs(self.items) do
			local section = item:Serialize()
			LIBA_CATALOG[id] = section
	end
end

function Catalog:GetImage(id)
	local catalogItem = self.items[id]
	if( catalogItem ) then
		return catalogItem.icon
	end
	return nil
end

function Catalog:GetCatalogItem(id)
	return  self.items[id]
end

function Catalog:AddCatalogItem(item)
	self.items[item.id] = item
end

function Catalog:GetCatalogItemStartingWith(start,type, contains)
	start = string.lower(start)
	local searchValues = {}
	local pattern = string.format("^%s.*",start)
	if(contains)then
		pattern = string.format("^.*%s.*",start)
	end
	for id, value in pairs(self.items) do
		local name = string.lower(value.name)
		if(type == libTConstants.CatalogItemType.ALL or bit.band(value.type,type)==type)then
			if (string.match(name,pattern) ~= nil) then
				table.insert(searchValues,value)
			end
		end
	end

	local sort_func = function( a,b ) return a.name < b.name end
	table.sort( searchValues, sort_func )
	local searchNames = {}
	local searchIds = {}
	for _, v in ipairs(searchValues)do
		table.insert(searchNames,v.name .. " - " .. (v.description or ""))
		table.insert(searchIds,v.id)
	end
	return searchNames,searchIds
end

function Catalog.SettingsLoaded(handle,settings)
	libAbilityCatalog.Catalog.aggressiveBuffsOn = nil
	 Catalog.AggressiveBuffsOn(settings.aggressiveBuffs)
end

function Catalog.Cast(handle, unit, casted)
	local self = libAbilityCatalog.Catalog
	if (casted == nil) then return end
	local id = casted.abilityNew or casted.abilityName
	if(not self.ignoreNames[casted.abilityName] and not self.items[id])then
		local casterDetail = Inspect.Unit.Detail(unit)
		local c = libAbilityCatalog.CatalogItem.Create()
		c.id = id
		c.name = casted.abilityName
		c.icon = casted.icon
		c.type = libTConstants.CatalogItemType.CASTBAR
		if(casted.abilityNew ~= nil)then
			c.type = bit.bor(c.type,libTConstants.CatalogItemType.ABILITY)
		end
		c.abilityId = casted.abilityNew
		c.casted = true
		local casterName = ""
		if(casterDetail ~= nil and not (casterDetail.player == true))then
			casterName = string.format("(%s) -", casterDetail.name)
		end
		local interuptable = ""
		if(casted.uninterruptible)then
			interuptable = "uninterruptible "
		end
		c.description = string.format("%sduration(%d) %s%s",casterName, casted.duration or 0,interuptable,casted.description or "")
		self.items[id] = c
	elseif(self.items[id])then
		self.items[id].casted = true
		self.items[id].type = bit.bor(self.items[id].type,libTConstants.CatalogItemType.CASTBAR)
	end
end

function Catalog.Buff(handle, buffTypeId, unit, buffId)
	local self = libAbilityCatalog.Catalog
	if(buffTypeId ~= nil and not self.items[buffTypeId] and buffTypeId ~= true)then
		local details = Inspect.Buff.Detail(unit, buffId)
		local casterDetail = Inspect.Unit.Detail(details.caster)
		local b = libAbilityCatalog.CatalogItem.Create()
		b.id = buffTypeId
		b.name = details.name
		b.icon = details.icon
		b.abilityId = details.abilityNew
		b.type = libTConstants.CatalogItemType.BUFF
		local casterName = ""
		if(casterDetail ~= nil and not casterDetail.player)then
			casterName = string.format("(%s)", casterDetail.name)
		end
		local curse = ""
		if(details.curse)then curse = "curse " end
		local debuff = ""
		if(details.debuff)then debuff = "debuff " end
		local disease = ""
		if(details.disease)then disease = "disease " end
		local poison = ""
		if(details.poison)then poison = "poison " end
		b.description = string.format("%s%s%s%s%s-duration(%d) %s",casterName,curse,debuff,disease,poison,details.duration or 0, details.description or "")
		self.items[b.id] = b
	elseif(self.items[buffTypeId])then
		self.items[buffTypeId].type = bit.bor(self.items[buffTypeId].type,libTConstants.CatalogItemType.BUFF)
	end
end

function Catalog.Ability(handle, ability)
	local self = libAbilityCatalog.Catalog
	if(not self.items[ability])then
		local details = Inspect.Ability.New.Detail(ability)
		if(details and not self.ignoreNames[details.name])then
			local a = libAbilityCatalog.CatalogItem.Create()
			a.id = ability
			a.name = details.name
			a.abilityId = details.ability
			a.icon = details.icon
			a.type = libTConstants.CatalogItemType.ABILITY
			a.description = details.description or "Ability - May be CD for multi-use abilities like masteries"
			self.items[a.id] = a
		end
	elseif(self.items[ability])then
		self.items[ability].type = bit.bor(self.items[ability].type,libTConstants.CatalogItemType.ABILITY)
	end
end

libAbilityCatalog.Catalog = Catalog.Create()

Command.Event.Attach(Event.Addon.SavedVariables.Load.End, Catalog.OnVariablesLoaded, "Event.Addon.SavedVariables.Load.End")
Command.Event.Attach(Event.Addon.SavedVariables.Save.Begin, Catalog.OnVariablesSaving, "Event.Addon.SavedVariables.Save.Begin")

Command.Event.Attach(libAbilityCatalog.Event.Cast, Catalog.Cast, "Catalog.Cast")
Command.Event.Attach(libAbilityCatalog.Event.Buff, Catalog.Buff, "Catalog.Buff")
Command.Event.Attach(libAbilityCatalog.Event.Ability, Catalog.Ability, "Catalog.Ability")

