--
-- Class EncounterManager
--
--   Manages a list of EncounterManager
libEncounters = ...

local EncounterManager = {}
EncounterManager.__index = EncounterManager

libEncounters.Event.Trigger.EncounterStart, libEncounters.Event.EncounterStart = Utility.Event.Create(libEncounters.addonId, "EncounterStart")
libEncounters.Event.Trigger.EncounterEnd, libEncounters.Event.EncounterEnd = Utility.Event.Create(libEncounters.addonId, "EncounterEnd")

function EncounterManager.Create()
	local obj = {}
	setmetatable(obj,EncounterManager)
	obj:Init()
	return obj
end

local groupSpec = {
	["player"] = true,
	["group01"]= true,
	["group02"]= true,
	["group03"]= true,
	["group04"]= true,
	["group05"]= true,
	["group06"]= true,
	["group07"]= true,
	["group08"]= true,
	["group09"]= true,
	["group10"]= true,
	["group11"]= true,
	["group12"]= true,
	["group13"]= true,
	["group14"]= true,
	["group15"]= true,
	["group16"]= true,
	["group17"]= true,
	["group18"]= true,
	["group19"]= true,
	["group20"]= true,
}

function EncounterManager:Init()
	self.items = {}
	self.group = {}
	self.player = Inspect.Unit.Lookup("player")
	self.currentEncounter = nil
	self.inCombat = false
	self.unitsInCombat = 0
end

function EncounterManager:AddEncounter(e)
	self.items[e.id] = e
	libTDebug.UIDebug:AddTable("EncounterManager.encounters",self.items)
end

function EncounterManager:GetEncounterNames()
	local names = {}
	for id, details in pairs(self.items) do
		table.insert(names,details.name)
	end
	return names
end

function EncounterManager:GetEncounter(id)
	return self.items[id]
end

function EncounterManager:GetEncountersByZone()
	local zoneNames = {}
	local zones = {}
	local texts = {}
	local values = {}
	local indent = {}
	for _,value in pairs(self.items)do
		if(not zones[value.zone])then
			table.insert(zoneNames,value.zone)
			zones[value.zone] = {}
		end
		table.insert(zones[value.zone],value)
	end
	local sort_func = function( a,b ) return a < b end
	table.sort( zoneNames, sort_func )
	for _, zone in ipairs(zoneNames)do
		local encounters = zones[zone]
		sort_func = function( a,b ) return a.name < b.name end
		table.sort( encounters, sort_func )
		table.insert(texts,zone)
		table.insert(values,"ZONE")
		table.insert(indent,1)
		for _, encounter in ipairs(encounters)do
			table.insert(texts,encounter.name)
			table.insert(values,encounter.id)
			table.insert(indent,2)
		end
	end
	return texts,values,indent
end

function EncounterManager:GetEncountersStartingWith(start)
	local values = {}
	local pattern = string.format("^%s.*",start)

	for id, value in pairs(self.items) do
		local name = string.lower(value.name)
		if (string.match(name,pattern) ~= nil) then
			table.insert(values,value)
		end
	end

	local sort_func = function( a,b ) return a.name < b.name end
	table.sort( values, sort_func )
	local names = {}
	for _, v in ipairs(values)do
		table.insert(names,v.name)
	end
	return names,values
end


function EncounterManager:GetEncounters()
	local values = {}
	local pattern = string.format("^%s.*",start)

	for id, value in pairs(self.items) do
		table.insert(values,value)
	end

	local sort_func = function( a,b ) return a.name < b.name end
	table.sort( values, sort_func )
	local names = {}
	for _, v in ipairs(values)do
		table.insert(names,v.name)
	end
	return names,values
end

function EncounterManager.OnVariablesLoaded(event, addonId)
	local self = libEncounters.EncounterManager
	if addonId ~= libEncounters.addonId then return end
	LIBE_ENC = LIBE_ENC or {}
	local merged = libEncounters.VersionUpdate(LIBE_ENC)
	for _, value in pairs(merged) do
		local e = libEncounters.Encounter.Deserialize(value)
		self.items[e.id]=e
	end
	libTDebug.UIDebug:AddTable("EncounterManager.encounters",self.items)
end

function EncounterManager.OnVariablesSaving(event, addonId)
	if addonId ~= libEncounters.addonId then return end
	local self = libEncounters.EncounterManager
	LIBE_ENC = LIBE_ENC or {}
	for id, item in pairs(self.items) do
		LIBE_ENC[id] = item:Serialize()
	end
end

function EncounterManager.CombatEnter(handle)
	local self = libEncounters.EncounterManager
	local currentCombat = self.inCombat
	self.inCombat = true
	self.currentEncounter = nil
	for id, item in pairs(self.items)do
		if(item:UnitMatch(libEncounters.UnitManager.combatUnits))then
			self.currentEncounter = item
			print("Encounter Begin : " .. item.name)
			break
		end
	end
	if(currentCombat ~= self.inCombat)then
		libEncounters.Event.Trigger.EncounterStart(self.currentEncounter)
	end
end

function EncounterManager.CombatLeave(handle)
	local self = libEncounters.EncounterManager
	if(self.inCombat)then
		self.inCombat = false
		if(self.currentEncounter ~= nil)then
			self.currentEncounter = nil
			print("Encounter End")
		end
		libEncounters.Event.Trigger.EncounterEnd()
	end
end

function EncounterManager.CombatChange(handle, units)
	local self = libEncounters.EncounterManager
	local unitSs = Inspect.Unit.Lookup(units)

	for id, unitSpec in pairs(unitSs) do
		if(groupSpec[unitSpec])then
			if(units[id])then
				self.unitsInCombat = self.unitsInCombat + 1
			else
				self.unitsInCombat = self.unitsInCombat - 1
			end
		end
	end
	if(units[self.player])then
		-- combat enter
		EncounterManager.CombatEnter(nil)
	elseif(units[self.player] ~= nil and not units[self.player] and self.inCombat and self.unitsInCombat == 0)then
		EncounterManager.CombatLeave(nil)
	end
end


function EncounterManager.RunFirst(handle, units)
	-- crazyness - this was needed to initialize some values.
	local self = libEncounters.EncounterManager
	for id, unitSpecifier in pairs(units)do
		if(unitSpecifier == "player")then
			self.player = id
			local d = Inspect.Unit.Detail("player")
			if(d.combat)then
				EncounterManager.CombatEnter(nil)
			else
				EncounterManager.CombatLeave(nil)
			end
		end
	end
	Command.Event.Detach(Event.Unit.Availability.Full, EncounterManager.RunFirst)
end

libEncounters.EncounterManager = EncounterManager.Create()

Command.Event.Attach(Event.Addon.SavedVariables.Load.End, EncounterManager.OnVariablesLoaded, "EncounterManager.OnVariablesLoaded")
Command.Event.Attach(Event.Addon.SavedVariables.Save.Begin, EncounterManager.OnVariablesSaving, "EncounterManager.OnVariablesSaving")
Command.Event.Attach(Event.SafesUnitLib.Raid.Combat.Enter, EncounterManager.CombatEnter, "Raid Combat Enter")
Command.Event.Attach(Event.SafesUnitLib.Raid.Combat.Leave, EncounterManager.CombatLeave, "Raid Combat Leave")
Command.Event.Attach(Event.Unit.Detail.Combat,EncounterManager.CombatChange,"EncounterManager.CombatChange")

Command.Event.Attach(Event.Unit.Availability.Full,EncounterManager.RunFirst, "EncounterManager.RunFirst")
