
libEncounters = ...
libEncounters.addonId = "libEncounters"
libEncounters.Event = {}
libEncounters.Event.Trigger = {}
libEncounters.LibSUnit = Inspect.Addon.Detail("SafesUnitLib").data

local LEMain = {}
LEMain.__index = LEMain

function LEMain.Create()
	local obj = {}
	setmetatable(obj,LEMain)
	obj:Init()
	return obj
end

function LEMain:Init()
	self.SlashCommands = {
		["config"] = { function() self:DisplayConfig() end, "Configuration." },
		["clearUnits"] = { function() libEncounters.UnitManager:UnitsClear() end, "Clears cached Units" },
	}
	self.description ="LibEncounters"
	self.size = { width = 800, height = 350 }
	print("encounter /encounter")
end

function LEMain:Split(inputstr, sep)
        if sep == nil then
                sep = "%s"
        end
        local t={} ; i=1
        for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
                t[i] = str
                i = i + 1
        end
        return t
end

function LEMain:DisplayConfig()
	local items, values = libEncounters.UnitManager:GetUnits()
	if(#items == 0)then
		table.insert(items,"Empty")
		table.insert(values,"Empty")
	end
	self.frame.units.list:SetItems(items,values)

	items, values = libEncounters.EncounterManager:GetEncounters()
	if(#items == 0)then
		table.insert(items,"Empty")
		table.insert(values,"Empty")
	end
	self.frame.encounters.list:SetItems(items,values)
	self.frame:SetVisible(true)
	self.frame.create:Show()
	self.frame.clear:Hide()
  self.frame.name:SetText("")
  self.frame.zone:SetText("")
  self.frame.raidSize:SetText("")
	self.frame.units.list:SetEnabled(true)

end

function LEMain.OnSlash(handle, parameterString)
	local self = libEncounters.LEMain
	local parameters =  self:Split(parameterString)
	local parameter = parameters[1]
	if (self.SlashCommands[parameter] ~= nil) then
		self.SlashCommands[parameter][1](parameters)
	else
		for key, value in pairs(self.SlashCommands) do
			print(string.format("/encounter %s - %s", key, value[2]))
		end
	end
end


function LEMain:UISetup()
	self.UIContext = UI.CreateContext("LibEncountersContext")
	self.frame = libTUI.TUIClosableWindow(self.UIContext, self.description, self.size, "Encounters")

	self.frame.encounters = {}
	self.frame.encounters.width = 240

	self.frame.encounters.scrollView = UI.CreateFrame("SimpleScrollView", self.description .. "_list",self.frame)
	self.frame.encounters.scrollView:SetPoint("TOPLEFT", self.frame.header, "BOTTOMLEFT",10,38)
	self.frame.encounters.scrollView:SetWidth(self.frame.encounters.width)
	self.frame.encounters.scrollView:SetHeight(250)
	self.frame.encounters.scrollView:SetShowScrollbar(false)
	self.frame.encounters.scrollView:SetBackgroundColor(0,0,0,1)
	self.frame.encounters.list = UI.CreateFrame("SimpleList", self.description .. "_AbilityList_SimpleList", self.frame.encounters.scrollView)
	self.frame.encounters.list:SetFontSize(14)

	self.frame.encounters.scrollView:SetContent(self.frame.encounters.list)
	self.frame.encounters.scrollView:SetVisible(true)

	self.frame.encounters.list.Event.ItemClick = function(item,value,_)
		local self = libEncounters.LEMain.frame.list

		libEncounters.LEMain.frame.name:SetText(value.name .. "")
		libEncounters.LEMain.frame.zone:SetText((value.zone or "") .. "")
		libEncounters.LEMain.frame.raidSize:SetText((value.raidSize or "").. "")

		local names = {}
		for tuid, name in pairs(value.units)do
		  table.insert(names,name)
		end
		if(#names == 0)then
			table.insert(names,"Empty")
		end
		libEncounters.LEMain.frame.units.list:SetItems(names)
		libEncounters.LEMain.frame.create:SetVisible(false)
		libEncounters.LEMain.frame.clear:SetVisible(true)
		libEncounters.LEMain.frame.units.list:SetEnabled(false)
  end

	self.frame.encounters.text = UI.CreateFrame("Text", self.description .. "_HeaderText", self.frame)
	self.frame.encounters.text:SetPoint("BOTTOMCENTER",self.frame.encounters.scrollView, "TOPCENTER",0,-3)
	self.frame.encounters.text:SetText("Encounters")
	self.frame.encounters.text:SetFontColor(1,1,1)
	self.frame.encounters.text:SetWidth(100)
	self.frame.encounters.text:SetFontSize(18)


	self.frame.units = {}
	self.frame.units.width = 240

	self.frame.units.scrollView = UI.CreateFrame("SimpleScrollView", self.description .. "_list",self.frame)
	self.frame.units.scrollView:SetPoint("TOPRIGHT", self.frame.header, "BOTTOMRIGHT",-10,38)
	self.frame.units.scrollView:SetWidth(self.frame.units.width)
	self.frame.units.scrollView:SetHeight(250)
	self.frame.units.scrollView:SetShowScrollbar(false)
	self.frame.units.scrollView:SetBackgroundColor(0,0,0,1)
	self.frame.units.list = UI.CreateFrame("SimpleList", self.description .. "_AbilityList_SimpleList", self.frame.units.scrollView)
	self.frame.units.list:SetFontSize(14)
	self.frame.units.list:SetSelectionMode("multi")
	self.frame.units.scrollView:SetContent(self.frame.units.list)
	self.frame.units.scrollView:SetVisible(true)

	self.frame.units.list.Event.ItemClick = function(item,unit,_)
		local self = libEncounters.LEMain.frame.list
		local encounterName = libEncounters.LEMain.frame.name:GetText()
		if(string.len(encounterName) <= 0 and item ~= nil) then
			 libEncounters.LEMain.frame.name:SetText(item .. "")
			 libEncounters.LEMain.frame.zone:SetText(unit.zone .. "")
			 libEncounters.LEMain.frame.raidSize:SetText(unit.raidMemebers .. "")
		end
	end

	self.frame.units.text = UI.CreateFrame("Text", self.description .. "_HeaderText", self.frame)
	self.frame.units.text:SetPoint("BOTTOMCENTER",self.frame.units.scrollView, "TOPCENTER",0,-3)
	self.frame.units.text:SetText("Avalible Units")
	self.frame.units.text:SetFontColor(1,1,1)
	self.frame.units.text:SetWidth(120)
	self.frame.units.text:SetFontSize(18)




	self.frame.name = libTUI.Input( self.frame
																,self.description .. "_Name"
																,"Encounter Name"
																,250
																,"TOPCENTER"
																,self.frame.header
																,"BOTTOMCENTER"
																,0
																,10
																,250
																,""
																,true)

	self.frame.zone = libTUI.Input( self.frame
																,self.description .. "_Zone"
																,"Zone"
																,250
																,"TOPLEFT"
																,self.frame.name.background
																,"BOTTOMLEFT"
																,0
																,10
																,250
																,""
																,true)
	self.frame.raidSize = libTUI.Input( self.frame
																,self.description .. "_RaidSize"
																,"Raid Size"
																,250
																,"TOPLEFT"
																,self.frame.zone.background
																,"BOTTOMLEFT"
																,0
																,10
																,250
																,""
																,true)
	self.frame.clear = libTUI.Button( self.frame
																		,libEncounters.addonId
																		,self.description .. "_Clear"
																		,{width = 102, height=39}
																		,"BOTTOMCENTER"
																		,self.frame
																		,"BOTTOMCENTER"
																		,0
																		,-10
																		,"\\images\\clear.png"
																		,"\\images\\clearOver.png" )
	self.frame.clear.config = self
	function self.frame.clear.Event:LeftClick()
		self.config:DisplayConfig()
	end

	self.frame.clear:Hide()
	self.frame.create = libTUI.Button( self.frame
																		,libEncounters.addonId
																		,self.description .. "_Clear"
																		,{width = 102, height=39}
																		,"BOTTOMCENTER"
																		,self.frame
																		,"BOTTOMCENTER"
																		,0
																		,-10
																		,"\\images\\create.png"
																		,"\\images\\createOver.png" )
	self.frame.create.config = self
	function self.frame.create.Event:LeftClick()
			 local name = self.config.frame.name:GetText()
			 local units = self.config.frame.units.list:GetSelectedValues()
			 if(string.len(name) > 0 and #units > 0)then
			 	 local encounter = libEncounters.Encounter.Create()

			 	 encounter.name = name
			 	 encounter.units = {}
			 	 for _, unit in ipairs(units) do
			 	 	encounter.units[unit.type] = unit.name
			 	 end
			 	 encounter.zone = self.config.frame.zone:GetText()
			 	 encounter.raidSize = self.config.frame.raidSize:GetText()
			 	 libEncounters.EncounterManager:AddEncounter(encounter)
			 	 self.config:DisplayConfig()
			 else
				print("Name empty or no units selected")
			 end
	end

end

function LEMain.OnVariablesLoaded(event, addonId)
	if addonId ~= libEncounters.addonId then return end
	libEncounters.LEMain:UISetup()
end



libEncounters.LEMain = LEMain.Create()
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, LEMain.OnVariablesLoaded, "LEMain.OnVariablesLoaded")

Command.Event.Attach(Command.Slash.Register("encounter"), LEMain.OnSlash, "SlashRegisterCommand.libEncounters")
