libEncounters = ...

local UnitManager = {}
UnitManager.__index = UnitManager

function UnitManager.Create()
	local obj = {}
	setmetatable(obj,UnitManager)
	obj:Init()
	return obj
end

function UnitManager:Init()
	self.cache = {}
	self.combatUnits = {}
	libTDebug.UIDebug:AddTable("UnitManager.combatUnits",self.combatUnits)
end

function UnitManager.ZoneChange(handle, units)
	self = libEncounters.UnitManager
	for unitId, zone in pairs(units) do
		if(zone and unitId == self.player)then
			self.cache = {}
			local z = Inspect.Zone.Detail(zone)
			self.zone = z.name
		end
	end
end

function UnitManager:UnitsClear()
	self.cache = {}
end

function UnitManager:GetUnits()
	local values = {}
	local pattern = string.format("^%s.*",start)

	for id, value in pairs(self.cache) do
		table.insert(values,value)
	end

	local sort_func = function( a,b ) return a.name < b.name end
	table.sort( values, sort_func )
	local names = {}
	for _, v in ipairs(values)do
		table.insert(names,v.name)
	end
	return names,values
end

function UnitManager.TargetChange(handle, unit)
	local self = libEncounters.UnitManager
	if(unit)then
		local detail = Inspect.Unit.Detail(unit)
		if(detail and detail.type)then
			if(not self.cache[detail.type])then
				detail.zone = self.zone
				local members = libEncounters.LibSUnit.Raid.Members
				if(not (members > 0))then
					members = "Not Grouped"
				end
				detail.raidMemebers = members
				self.cache[detail.type] = detail
			end
			self.combatUnits[detail.type] = detail
		end
	end
end

function UnitManager.RunFirst()
	local self = libEncounters.UnitManager
	self.player = Inspect.Unit.Lookup("player")
	local d = Inspect.Unit.Detail("player")
	local z = Inspect.Zone.Detail(d.zone)
	self.zone = z.name
	Command.Event.Detach(Event.Unit.Availability.Full, UnitManager.RunFirst)
end

function UnitManager.EncounterEnd()
	local self = libEncounters.UnitManager
	self.combatUnits = {}
	libTDebug.UIDebug:AddTable("UnitManager.combatUnits",self.combatUnits)
end

function UnitManager.EncounterStart()
	local self = libEncounters.UnitManager
end

libEncounters.UnitManager = UnitManager.Create()

local groupSpec = {
	"player",
	"group01",
	"group02",
	"group03",
	"group04",
	"group05",
	"group06",
	"group07",
	"group08",
	"group09",
	"group10",
	"group11",
	"group12",
	"group13",
	"group14",
	"group15",
	"group16",
	"group17",
	"group18",
	"group19",
	"group20",
}

for _, grp in ipairs(groupSpec) do
	Command.Event.Attach(Library.LibUnitChange.Register(grp .. ".target"), UnitManager.TargetChange, "libEncounters.UnitManager.TargetChange")
end

Command.Event.Attach(Event.Unit.Detail.Zone,UnitManager.ZoneChange, "UnitManager.ZoneChange")
Command.Event.Attach(Event.Unit.Availability.Full,UnitManager.RunFirst, "UnitManager.RunFirst")
Command.Event.Attach(libEncounters.Event.EncounterStart, UnitManager.EncounterStart, "UnitManager.EncounterStart")
Command.Event.Attach(libEncounters.Event.EncounterEnd, UnitManager.EncounterEnd, "UnitManager.EncounterEnd")

