--
--  Class UIDebug

libTDebug = ...

local TEvent = {}
TEvent.Trigger = {}

TEvent.Trigger.DEBUG, TEvent.DEBUG = Utility.Event.Create("LibTUtility", "DEBUG")
TEvent.Trigger.INFO, TEvent.INFO = Utility.Event.Create("LibTUtility", "INFO")


local UIDebug = {}
UIDebug.__index = UIDebug

function UIDebug.Create()
	local obj = {}
	setmetatable(obj,UIDebug)
	obj:Init()
	return obj
end

function UIDebug:Init()
	self.size = { width = 800, height = 400 }
	self.description = "UIDebug"
	self.log = {}
	self.tableMap = {}
	self.lastMsg = ""
end

function UIDebug.UISetup(handle)
	local self = libTDebug.UIDebug
	self.UIContext = UI.CreateContext("UIDebugContext")
	self.frame = libTUI.TUIClosableWindow(self.UIContext, self.description, self.size, "Debug")
	self.frame.inputs = {}

	self.frame.inputs.NONE = libTUI.ToggleFrame.Create(self.frame
										,"None", "None"
										,nil
										,110
										,"TOPLEFT", self.frame.header, "BOTTOMLEFT" ,110 ,5
										,false
										,false
										,libTUI.ToggleFrame.Borders.Left)

	self.frame.inputs.INFO = libTUI.ToggleFrame.Create(self.frame
										,"Info", "Info"
										,nil
										,110
										,"TOPLEFT", self.frame.inputs.NONE.label, "TOPRIGHT" ,5 ,0
										,false
										,false
										,libTUI.ToggleFrame.Borders.None)

	self.frame.inputs.DEBUG = libTUI.ToggleFrame.Create(self.frame
										,"Debug", "Debug"
										,nil
										,110
										,"TOPLEFT", self.frame.inputs.INFO.label, "TOPRIGHT" ,5 ,0
										,false
										,false
										,libTUI.ToggleFrame.Borders.Right)

	self.frame.inputs.NONE:Enable(true)
	self.frame.inputs.INFO:Enable(false)
	self.frame.inputs.DEBUG:Enable(false)
	if(TDEBUG.LOG == "INFO")then
		self.frame.inputs.NONE:Enable(false)
		self.frame.inputs.DEBUG:Enable(false)
	elseif(TDEBUG.LOG == "DEBUG")then
		self.frame.inputs.NONE:Enable(false)
		self.frame.inputs.INFO:Enable(false)
	end
	self.frame.inputs.NONE.onChanged = function()
		UIDebug.Detach()
		TDEBUG.LOG = "NONE"
		self.frame.inputs.INFO:Enable(false)
		self.frame.inputs.DEBUG:Enable(false)
	end
	self.frame.inputs.INFO.onChanged = function()
		UIDebug.Detach()
		TDEBUG.LOG = "INFO"
		Command.Event.Attach(TEvent.INFO,UIDebug.OnINFO, "UIDebug.OnINFO")
		self.frame.inputs.NONE:Enable(false)
		self.frame.inputs.DEBUG:Enable(false)
	end
	self.frame.inputs.DEBUG.onChanged = function()
		UIDebug.Detach()
		TDEBUG.LOG = "DEBUG"
		Command.Event.Attach(TEvent.INFO,UIDebug.OnINFO, "UIDebug.OnINFO")
		Command.Event.Attach(TEvent.DEBUG,UIDebug.OnDEBUG, "UIDebug.OnDEBUG")
		self.frame.inputs.NONE:Enable(false)
		self.frame.inputs.INFO:Enable(false)
	end

	self.frame.inputs.Log = libTUI.Input(self.frame
											,self.description .. "_Log"
											,"Messages"
											,self.size.width
											,"TOPLEFT"
											,self.frame.header
											,"BOTTOMLEFT"
											,5
											,15
											,self.size.width - 250
											,""
											,true)
	self.frame.inputs.Log:SetHeight(315)
	self.frame.inputs.Log.background:SetHeight(315)


	self.frame.inputs.Refresh  = libTUI.Button(self.frame
											,"LibTUtility"
											,self.description .. "_Refresh"
											,{width = 30, height = 30}
											,"TOPRIGHT"
											,self.frame.inputs.Log
											,"TOPRIGHT"
											,0
											,-35
											,"\\images\\Refresh.png"
											,"\\images\\RefreshOver.png"
											,false)
	function self.frame.inputs.Refresh.Event:LeftClick()
		self = libTDebug.UIDebug
		self.frame.inputs.Log:SetText(table.concat(self.log,"\n"))
	end

	self.frame.inputs.Clear  = libTUI.Button(self.frame
											,"LibTUtility"
											,self.description .. "_Refresh"
											,{width = 30, height = 30}
											,"TOPRIGHT"
											,self.frame.inputs.Refresh
											,"TOPRIGHT"
											,-35
											,0
											,"\\images\\Clear.png"
											,"\\images\\ClearOver.png"
											,false)
	function self.frame.inputs.Clear.Event:LeftClick()
		self = libTDebug.UIDebug
		self.log = {}
		self.frame.inputs.Log:SetText(table.concat(self.log,"\n"))
	end

	self.frame.debugTableListScrollView = UI.CreateFrame("SimpleScrollView", self.description .. "_debugTableList",self.frame)
	self.frame.debugTableListScrollView:SetPoint("TOPLEFT", self.frame.inputs.Log, "TOPRIGHT",10,-35)
	self.frame.debugTableListScrollView:SetWidth(235)
	self.frame.debugTableListScrollView:SetHeight(self.size.height - 40)
	self.frame.debugTableListScrollView:SetShowScrollbar(false)

	self.frame.debugTableList = UI.CreateFrame("SimpleList", self.description .. "_AbilityList_SimpleList", self.frame.debugTableListScrollView)
	self.frame.debugTableList:SetFontSize(14)
	self.frame.debugTableListScrollView:SetContent(self.frame.debugTableList)
	self.frame.debugTableListScrollView:SetVisible(true)

	self.frame.debugTableList.Event.ItemClick = function(name, value, _)
		libTDebug.UIDebug.frame.inputs.Log:SetText(libTUtility.TableToString(value))
	end
end

function UIDebug.Detach()
	Command.Event.Detach(TEvent.INFO,UIDebug.OnINFO, "UIDebug.OnINFO")
	Command.Event.Detach(TEvent.DEBUG,UIDebug.OnDEBUG, "UIDebug.OnDEBUG")
end

function UIDebug.OnINFO(handle,prettytable, message, ...)
	self = libTDebug.UIDebug
	if(message ~= nil)then
		local v = ""
		if(... ~= nil)then
			v = "INFO: " .. string.format(message,...)
		else
			v = "INFO: " .. message
		end
		if(self.lastMsg ~= v)then
			self.lastMsg = v
			table.insert(self.log,v)
		end
	end
end

function UIDebug.OnDEBUG(handle,prettytable, message, ...)
	self = libTDebug.UIDebug
	if(message ~= nil)then
		local v = ""
		if(... ~= nil)then
			v = "DEBUG: " .. string.format(message,...)
		else
			v = "DEBUG: " .. message
		end
		if(self.lastMsg ~= v)then
			self.lastMsg = v
			table.insert(self.log,v)
		end
	end
end

function UIDebug:Show()
	self.frame.inputs.Log:SetText(table.concat(self.log,"\n"))
	self.frame:SetVisible(true)

	local names = {}
	local tableItems =  {}

	for name, _ in pairs(self.tableMap)do
		table.insert(names,name)
	end

	local sort_func = function( a ,b ) return a < b end
	table.sort(names, sort_func )

	if(#names > 0) then
		for _,name in ipairs(names)do
			table.insert(tableItems,self.tableMap[name])
		end

		self.frame.debugTableList:SetItems(names,tableItems)
	end
end

function UIDebug:AddTable(name,tbl)
	self.tableMap[name] = tbl
end

function UIDebug:INFO(prettytable, message, ...)
	TEvent.Trigger.INFO(prettytable, message, ...)
end

function UIDebug:DEBUG(prettytable, message, ...)
	TEvent.Trigger.DEBUG(prettytable, message, ...)
end

libTDebug.UIDebug = UIDebug.Create()

function UIDebug.OnVariablesLoaded(event, addonId)
	if addonId ~= "LibTUtility" then return end
	TDEBUG = TDEBUG or {}
	if(TDEBUG.LOG == "INFO")then
		Command.Event.Attach(TEvent.INFO,UIDebug.OnINFO, "UIDebug.OnINFO")
	elseif(TDEBUG.LOG == "DEBUG")then
		Command.Event.Attach(TEvent.INFO,UIDebug.OnINFO, "UIDebug.OnINFO")
		Command.Event.Attach(TEvent.DEBUG,UIDebug.OnDEBUG, "UIDebug.OnDEBUG")
	end
end

function UIDebug.OnVariablesSaving(event, addonId)
	if addonId ~= "LibTUtility" then return end
	TDEBUG = TDEBUG or {}
end

Command.Event.Attach(Event.Addon.Load.End, UIDebug.UISetup, "UIDebug.UISetup")
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, UIDebug.OnVariablesLoaded, "UIDebug.OnVariablesLoaded")
Command.Event.Attach(Event.Addon.SavedVariables.Save.Begin, UIDebug.OnVariablesSaving, "UIDebug.OnVariablesSaving")
