libTUI = ...
local height = 26

function libTUI.EditFieldDefaults(frame,itemWidth)
	frame:SetWidth(itemWidth)
	frame:SetHeight(20)
	frame:SetLayer(102)
	frame:SetVisible(true)
end

function libTUI.RadioButton(parent
							,id
							,text
							,selfPoint
							,anchorFrame
							,anchorPoint
							,x
							,y)
	local r = UI.CreateFrame("SimpleRadioButton", id  .. "_RadioButton", parent)
	r:SetText(text)
	r:SetFontSize(15)
	r:SetLayer(parent:GetLayer() + 1)
	r:SetPoint(selfPoint,anchorFrame, anchorPoint,x,y)
	function r:Hide()
		self:SetVisible(false)
	end
	function r:Show()
		self:SetVisible(true)
	end
	return r
end



local ToggleFrame = {}
ToggleFrame.Borders = {
	["None"] = 0,
	["Left"] = 1,
	["Right"] = 2,
	["All"] = 3
}

ToggleFrame.__index = ToggleFrame

function ToggleFrame.Create(parent
							,label, disabledText
							,linkedFrame
							,labelWidth
							,selfPoint ,anchorFrame ,anchorPoint ,x ,y
							,enabled
							,red
							,borders
							,disabledFontColor)
	local obj = {}
	setmetatable(obj,ToggleFrame)

	obj:Init(parent
			,label, disabledText
			,linkedFrame
			,labelWidth
			,selfPoint ,anchorFrame ,anchorPoint ,x ,y
			,enabled
			,red
			,borders
			,disabledFontColor)
	return obj
end

function ToggleFrame:Init(parent
							,label, disabledText
							,linkedFrame
							,labelWidth
							,selfPoint ,anchorFrame ,anchorPoint ,x ,y
							,enabled
							,red
							,borders
							,disabledFontColor)
	self.disabledFontColor = disabledFontColor
	self.borders = borders

	self.label = UI.CreateFrame("Frame", parent:GetName() .. "_Center_" .. label, parent)
	self.label:SetHeight(height)
	self.label:SetWidth(labelWidth)
	self.label:SetPoint(selfPoint, anchorFrame, anchorPoint,x,y)
	self.label:SetLayer(parent:GetLayer() + 1)
	self.label:SetVisible(true)

	self.background = UI.CreateFrame("Frame", parent:GetName() .. "_back_" .. label, self.label)

	if(bit.band(self.borders,ToggleFrame.Borders.Left) == ToggleFrame.Borders.Left)then
		self.left = UI.CreateFrame("Texture", parent:GetName() .. "_Left_" .. label, self.label)
		self.left:SetHeight(height)
		self.left:SetWidth(15)
		self.left:SetPoint("TOPLEFT", self.label, "TOPLEFT",0,0)
		self.left:SetLayer(parent:GetLayer() + 1)
		self.left:SetVisible(true)

		self.background:SetPoint("TOPLEFT", self.left, "TOPRIGHT")
		self.background:SetPoint("BOTTOMLEFT", self.left, "BOTTOMRIGHT")
	else
		self.background:SetPoint("TOPLEFT", self.label, "TOPLEFT")
		self.background:SetPoint("BOTTOMLEFT", self.label, "BOTTOMLEFT")
	end

	self.text = UI.CreateFrame("Text", parent:GetName() .. "_Label_" .. label, self.label)
	self.text:SetLayer(parent:GetLayer() + 3)
	self.text:SetVisible(true)
	self.text:SetText(label)
	self.text:SetFontSize(15)
	self.text:SetFontColor(0,0,0,1)
	self.text:SetPoint("BOTTOMCENTER", self.label, "BOTTOMCENTER",0,-1)

	if(bit.band(self.borders,ToggleFrame.Borders.Right) == ToggleFrame.Borders.Right)then
		self.right = UI.CreateFrame("Texture", parent:GetName() .. "_Right_" .. label, self.label)
		self.right:SetHeight(height)
		self.right:SetWidth(15)
		self.right:SetPoint("TOPRIGHT", self.label, "TOPRIGHT",0,0)
		self.right:SetLayer(parent:GetLayer() + 1)
		self.right:SetVisible(true)

		self.background:SetPoint("TOPRIGHT", self.right, "TOPLEFT")
		self.background:SetPoint("BOTTOMRIGHT", self.right, "BOTTOMLEFT")
	else
		self.background:SetPoint("TOPRIGHT", self.label, "TOPRIGHT")
		self.background:SetPoint("BOTTOMRIGHT", self.label, "BOTTOMRIGHT")
	end

	self.background:SetLayer(parent:GetLayer() + 2)
	self.background:SetVisible(true)

	self.linkedFrame = linkedFrame
	self.disabledText = disabledText
	self.labelText = label
	self.enabled = enabled
	self.red = red
	self.label.Event.LeftClick = function()
		self:Enable(not self.enabled)
		if(self.onChanged)then
			self.onChanged(self)
		end
		if(self.Changed)then
			self.Changed(self)
		end
	end

	if(linkedFrame)then
		linkedFrame:SetPoint("TOPLEFT", self.label, "TOPRIGHT",0,0)
	end

	self:Enable(self.enabled)
end

function ToggleFrame:SetVisible(show)
	self.label:SetVisible(show)
end

function ToggleFrame:Enable(enabled)
	self.enabled = enabled
	if(enabled)then
		if(self.red)then
			self.background:SetBackgroundColor(.7,.001,.001,1)
			if(bit.band(self.borders,ToggleFrame.Borders.Left) == ToggleFrame.Borders.Left)then
				self.left:SetTexture("LibTUtility", "\\images\\redLeftRound.png")
			end
			if(bit.band(self.borders,ToggleFrame.Borders.Right) == ToggleFrame.Borders.Right)then
				self.right:SetTexture("LibTUtility", "\\images\\redRightRound.png")
			end
		else
			self.background:SetBackgroundColor(1,1,1,1)
			if(bit.band(self.borders,ToggleFrame.Borders.Left) == ToggleFrame.Borders.Left)then
				self.left:SetTexture("LibTUtility", "\\images\\toolTipLeft.png")
			end
			if(bit.band(self.borders,ToggleFrame.Borders.Right) == ToggleFrame.Borders.Right)then
				self.right:SetTexture("LibTUtility", "\\images\\toolTipRight.png")
			end
		end
		self.text:SetText(self.labelText)
		if(self.linkedFrame)then
			self.linkedFrame:SetVisible(true)
		end
	else
		self.background:SetBackgroundColor(.2,.2,.2,1)
		if(bit.band(self.borders,ToggleFrame.Borders.Left) == ToggleFrame.Borders.Left)then
			self.left:SetTexture("LibTUtility", "\\images\\greyLeftRound.png")
		end
		if(bit.band(self.borders,ToggleFrame.Borders.Right) == ToggleFrame.Borders.Right)then
			self.right:SetTexture("LibTUtility", "\\images\\greyRightRound.png")
		end
		self.text:SetText(self.disabledText)
		if(self.linkedFrame)then
			self.linkedFrame:SetVisible(false)
		end
	end
	if(self.disabledFontColor and enabled == false)then
		self.text:SetFontColor(self.disabledFontColor[1],self.disabledFontColor[2],self.disabledFontColor[3],self.disabledFontColor[4])
	else
		self.text:SetFontColor(0,0,0,1)
	end
end

function ToggleFrame:GetEnabled()
	return self.enabled
end

libTUI.ToggleFrame = ToggleFrame


function libTUI.Label( parent
						,id
						,text
						,width
						,selfPoint
						,anchorFrame
						,anchorPoint
						,x
						,y)
	local l = UI.CreateFrame("Text", id .. "_Label", parent)
	l:SetWidth(width)
	l:SetHeight(height)
	l:SetMouseMasking("limited")
	l:SetLayer(parent:GetLayer() + 1)
	l:SetVisible(true)
	l:SetText(text)
	l:SetFontSize(15)
	l:SetPoint(selfPoint, anchorFrame, anchorPoint,x,y)
	l:SetFontColor(.3,.55,.85)
	function l:Hide()
		self:SetVisible(false)
	end
	function l:Show()
		self:SetVisible(true)
	end
	return l
end

function libTUI.Slider( parent
						,id
						,labelText
						,labelWidth
						,selfPoint
						,anchorFrame
						,anchorPoint
						,x
						,y
						,sliderWidth
						,defaultValue)

	local s = UI.CreateFrame("RiftSlider", id  .. "_Slider", parent)
	s.input = libTUI.Input(parent, id, labelText, labelWidth, selfPoint, anchorFrame, anchorPoint,x,y, 40, tostring(defaultValue))
	s:SetWidth(sliderWidth)
	s:SetPoint("TOPLEFT",s.input, "TOPRIGHT",20,5)


	function s:Hide()
		self:SetVisible(false)
		if(self.label ~= nil)then
			self.label:SetVisible(false)
		end
	end
	function s:Show()
		self:SetVisible(true)
		if(self.label ~= nil)then
			self.label:SetVisible(true)
		end
	end
	return s
end

function libTUI.Input( parent
						,id
						,labelText
						,width
						,selfPoint
						,anchorFrame
						,anchorPoint
						,x
						,y
						,inputWidth
						,defaultValue
						,vertical)
	local b = UI.CreateFrame("Frame",id .. "frame", parent)
	b:SetVisible(true)
	b:SetHeight(height)
	b:SetWidth(inputWidth)
	b:SetLayer(parent:GetLayer() + 1)
	local t = UI.CreateFrame("RiftTextfield", id, parent)
	if(labelText ~= nil)then
		if(vertical)then
			t.label = libTUI.Label(parent, id, labelText, width,selfPoint, anchorFrame, anchorPoint,x,y)
			b:SetPoint("TOPLEFT",t.label, "BOTTOMLEFT",0,-3)
			t:SetPoint("CENTER",b,"CENTER",0,1)
			t.label:SetFontColor(.3,.55,.85)
		else
			t.label = libTUI.Label(parent, id, labelText, width,selfPoint, anchorFrame, anchorPoint,x,y)
			b:SetPoint("TOPLEFT",t.label, "TOPRIGHT",0,-2)
			t:SetPoint("CENTER",b,"CENTER",0,1)
			t.label:SetFontColor(.3,.55,.85)
		end
	else
		b:SetPoint(selfPoint, anchorFrame, anchorPoint,x,y-2)
		t:SetPoint("CENTER", b, "CENTER",0,0)
	end

	t:SetWidth(inputWidth - 10)
	t:SetVisible(true)
	t:SetLayer(parent:GetLayer() + 2)
	t:SetHeight(22)
	t:SetText(defaultValue)

	b:SetBackgroundColor(0.1, 0.1, 0.1, 1)
	t.background = b

	function t:Hide()
		self:SetVisible(false)
		if(self.label ~= nil)then
			self.label:SetVisible(false)
		end
	end
	function t:Show()
		self:SetVisible(true)
		if(self.label ~= nil)then
			self.label:SetVisible(true)
		end
	end
	return t
end


function libTUI.InputPoint( parent
							,id
							,labelTop
							,labelLeft
							,width
							,selfPoint
							,anchorFrame
							,anchorPoint
							,x
							,y
							,inputWidth)
	local t = UI.CreateFrame("RiftTextfield", id, parent)
	t.left = UI.CreateFrame("RiftTextfield",id, parent)
	if(labelTop ~= nil)then
		t.labelTop = libTUI.Label(parent, id, labelTop, width,selfPoint, anchorFrame, anchorPoint,x,y)
		t:SetPoint("TOPLEFT",t.labelTop, "TOPRIGHT")
	else
		t:SetPoint(selfPoint, anchorFrame, anchorPoint,x,y)
	end

	if(labelLeft ~= nil)then
		t.labelLeft = libTUI.Label(parent, id, labelLeft, width,selfPoint, anchorFrame, anchorPoint,x,y + 30)
		t.left:SetPoint("TOPLEFT",t,"BOTTOMLEFT",0,10)
	else
		t:SetPoint(selfPoint, anchorFrame, anchorPoint,x,y)
	end

	libTUI.EditFieldDefaults(t,inputWidth)
	libTUI.EditFieldDefaults(t.left,inputWidth)

	t:SetBackgroundColor(0.2, 0.2, 0.2, 1)
	t.left:SetBackgroundColor(0.2, 0.2, 0.2, 1)
	Library.LibSimpleWidgets.SetBorder(t, 1, 0.5, 0.5, 0.5, 1)
	Library.LibSimpleWidgets.SetBorder(t.left, 1, 0.5, 0.5, 0.5, 1)
	function t:Hide()
		self:SetVisible(false)
		self.left:SetVisible(false)
		if(self.labelTop ~= nil)then
			self.labelTop:SetVisible(false)
		end
		if(self.labelLeft ~= nil)then
			self.labelLeft:SetVisible(false)
		end
	end
	function t:Show()
		self:SetVisible(true)
		self.left:SetVisible(true)
		if(self.labelTop ~= nil)then
			self.labelTop:SetVisible(true)
		end
		if(self.labelLeft ~= nil)then
			self.labelLeft:SetVisible(true)
		end
	end
	return t
end


function libTUI.ScrollingText(parent
						,id
						,labelText
						,width
						,selfPoint
						,anchorFrame
						,anchorPoint
						,x
						,y
						,scrollingWidth
						,scrollingHeight
						,text
						,vertical)

	local t = UI.CreateFrame("Text",id, parent)
	t.scrollView = UI.CreateFrame("SimpleScrollView", id .. "_scroll" , parent)
	t.scrollView:SetWidth(scrollingWidth)
	t.scrollView:SetHeight(scrollingHeight)
	t.scrollView:SetBorder(.5, .5, .5, .5, 1)
	t.scrollView:SetBackgroundColor(0.2, 0.2, 0.2, 1)
	t.scrollView:SetVisible(true)

	if(labelText ~= nil)then
		if(vertical)then
			t.label = libTUI.Label(parent, id, labelText, width,selfPoint, anchorFrame, anchorPoint,x,y)
			t.scrollView:SetPoint("TOPLEFT",t.label, "BOTTOMLEFT",0,-3)
		else
			t.label = libTUI.Label(parent, id, labelText, width,selfPoint, anchorFrame, anchorPoint,x,y)
			t.scrollView:SetPoint("TOPLEFT",t.label, "TOPRIGHT")
		end
	else
		t.scrollView:SetPoint(selfPoint, anchorFrame, anchorPoint,x,y)
	end
	t.scrollView:SetContent(t)
	if(text~=nil)then
		t:SetText(text)
	end
	return t
end

function libTUI.CheckBox(parent
						,id
						,label
						,labelWidth
						,selfPoint
						,anchorFrame
						,anchorPoint
						,x
						,y)
	local c = UI.CreateFrame("RiftCheckbox", id, parent)
	c:SetWidth(20)
	c:SetHeight(20)
	c:SetLayer(parent:GetLayer() + 1)

	if(label ~= nil)then
		c.label = libTUI.Label(parent,id, label, labelWidth,selfPoint, anchorFrame, anchorPoint,x + 20,y)
		c:SetPoint("TOPRIGHT", c.label, "TOPLEFT")
	else
		c:SetPoint(selfPoint, anchorFrame, anchorPoint,x,y)
	end

	c:SetVisible(true)
	function c:Hide()
		self:SetVisible(false)
		if(self.label ~= nil)then
			self.label:SetVisible(false)
		end
	end
	function c:Show()
		self:SetVisible(true)
		if(self.label ~= nil)then
			self.label:SetVisible(true)
		end
	end
	return c
end
function libTUI.Button(parent
						,addonId
						,id
						,size
						,selfPoint
						,anchorFrame
						,anchorPoint
						,x
						,y
						,image
						,hoverImage
						,confirm)

	local b = UI.CreateFrame("Texture",id, parent)
	b:SetWidth(size.width)
	b:SetHeight(size.height)
	b:SetVisible(true)
	b:SetLayer(parent:GetLayer() + 1)
	b:SetMouseMasking("limited")
	b:SetPoint(selfPoint,anchorFrame, anchorPoint,x ,y)
	b:SetTexture(addonId,image)
	b.image = image
	b.hoverImage = hoverImage

	b.Event.MouseIn = function(self)
  	self:SetTexture(addonId,self.hoverImage)
	end
	b.Event.MouseOut = function (self)
		self:SetTexture(addonId,self.image)
	end
	if (confirm ~= nil and confirm == true)then
		b.check = libTUI.CheckBox(	b
									,id .. "_Confirm"
									,nil
									,nil
									,"LEFTCENTER"
									,b
									,"RIGHTCENTER"
									,10
									,0)
	end
	function b:Hide()
		self:SetVisible(false)
		if(self.check ~= nil)then
			self.label:Hide()
		end
	end
	function b:Show()
		self:SetVisible(true)
		if(self.check ~= nil)then
			self.label:Show()
		end
	end
	return b
end


function libTUI.TUIClosableWindow(parent, description, size, headerText, layer)
	local f = UI.CreateFrame("Frame",description, parent)
	f:SetWidth(size.width)
	f:SetHeight(size.height)
	f:SetMouseMasking("limited")
	f:SetLayer(layer or 101)
	Library.LibSimpleWidgets.SetBorder("rounded", f, 1,1,1,1)
	f:SetBackgroundColor(0.05,0.05,0.05,.95)
	f:SetPoint("CENTER",UIParent, "CENTER")
	f:SetVisible(false)
	f.header = UI.CreateFrame("Frame",description .. "_Header", f)
	f.header:SetWidth(size.width)
	f.header:SetHeight(20)
	f.header:SetLayer(layer or 101)
	f.header:SetVisible(true)
	f.header:SetPoint("TOPCENTER", f, "TOPCENTER")
	f.header:SetBackgroundColor(.15,.15,.15, 1)
	f.header:SetHeight(40)
	f.header.frame = f
	f.header.text = UI.CreateFrame("Text", description .. "_HeaderText", f.header)
	f.header.text:SetPoint("TOPCENTER", f.header, "TOPCENTER",0,5)
	f.header.headerText = headerText
	f.header.text:SetText(f.header.headerText)
	f.header.text:SetFontColor(1,1,0)
	f.header.text:SetFontSize(20)
	function  f.Closed()
	end
	function f.Moved()
	end

	function f.header.Event:LeftDown()
		self.grab = true
		self.moved = false
		m = Inspect.Mouse()
		self.grabOffset = { x = m.x - f:GetLeft(), y = m.y - f:GetTop() }
	end

	function f.header.Event:MouseMove(x,y)
		if self.grab then
			f:ClearPoint("CENTER")
			f:SetPoint("TOPLEFT", UIParent, "TOPLEFT", (x - self.grabOffset.x) , (y - self.grabOffset.y))
			self.moved = true
			f.Moved()
		end
	end

	function f.header.Event:LeftUp()
		self.grab = false
	end

	function f.header.Event:LeftClick()
		if self.moved == false then
			f:SetVisible(false)
			f.Closed()
		end
		self.text:SetText(self.headerText)
		self.text:SetFontColor(1,1,0)
		self.moved = false
	end

	function f.header.Event:MouseIn()
		self.text:SetText("< close >")
		self.text:SetFontColor(1,0,0)
	end

	function f.header.Event:MouseOut()
		self.text:SetText(self.headerText)
		self.text:SetFontColor(1,1,0)
	end

	return f
end

local Msg = {}
Msg.__index = Msg
function Msg:SetVisible(visible)
	self.UI.smoke:SetVisible(visible)
end

function Msg:SetText(t)
	self.UI.text:SetText(t)
end

function libTUI.Message(parent, description, m)
	local message = {}
	setmetatable(message,Msg)
	message.UI = {}
	message.Event = {}
	message.Event.okay = function ()
	end
	message.Event.cancel = function ()
	end
	message.UI.smoke = UI.CreateFrame("Frame",description .. "_smoke", parent)
	message.UI.smoke:SetAllPoints(parent)
	message.UI.smoke:SetLayer(parent:GetLayer() + 10)
	message.UI.smoke:SetBackgroundColor(0,0,0, .9)
	message.UI.smoke:SetMouseMasking("limited")

	message.UI.smoke.Event.MouseMove = function () end
	message.UI.smoke.Event.LeftUp = function () end
	message.UI.smoke.Event.LeftClick = function () end


	message.UI.text = UI.CreateFrame("Text", description .. "_text", message.UI.smoke)
	message.UI.text:SetPoint("CENTER",message.UI.smoke,"CENTER",0,-50)
	message.UI.text:SetLayer(message.UI.smoke:GetLayer() + 1)
	message.UI.text:SetText(m)
	message.UI.text:SetFontSize(14)

	message.UI.okay = libTUI.Button(message.UI.text
									,"LibTUtility"
									,self.description .. "_Okay"
									,{width = 67, height = 30}
									,"CENTER"
									,message.UI.text
									,"CENTER"
									,-70
									,90
									,"\\images\\import.png"
									,"\\images\\importOver.png"
									,false)
	message.UI.okay:SetLayer(parent:GetLayer() + 1)
	message.UI.okay.msg = message
	message.UI.okay.Event.LeftClick = function(self)
		self.msg.Event.okay()
	end

	message.UI.cancel = libTUI.Button(message.UI.text
									,"LibTUtility"
									,self.description .. "_Okay"
									,{width = 69, height = 30}
									,"CENTER"
									,message.UI.text
									,"CENTER"
									,70
									,90
									,"\\images\\cancel.png"
									,"\\images\\cancelOver.png"
									,false)
	message.UI.cancel:SetLayer(parent:GetLayer() + 1)
	message.UI.cancel.msg = message
	message.UI.cancel.Event.LeftClick = function(self)
		self.msg.Event.cancel()
	end

	return message
end