--
-- Class ToolTip
--
--   Manages a list of ToolTip
local libTUI = ...
local ToolTip = {}
libTUI.ToolTip = ToolTip

ToolTip.__index = ToolTip

local gridCount= 50
local scale = 3

function ToolTip.Create(uiContext)
	local obj = {}
	setmetatable(obj,ToolTip)
	obj:Init(uiContext)
	return obj
end

function ToolTip:Init(uiContext)
	self.size = { width = 300, height = 60 }
	self.description = "ToolTip"
	self.visible = false
	self.lastCheckCPU = 0
	self.cor = nil
	self:UISetup(uiContext)
end

function ToolTip:UISetup(parent)
	self.parent = parent
	self.frame = UI.CreateFrame("Frame",self.description, parent)
	self.frame:SetWidth(self.size.width)
	self.frame:SetHeight(self.size.height)
	self.frame:SetMouseMasking("limited")
	self.frame:SetLayer(998)
	--Library.LibSimpleWidgets.SetBorder("rounded", self.frame, 1,1,1,1)
	self.frame:SetBackgroundColor(1,1,1,1)
	self.frame:SetPoint("TOPLEFT",UIParent,"CENTER", self.size.width / 2 * -1, self.size.height / 2 * -1)
	self.frame:SetVisible(true)

	self.frame.chatPoint = UI.CreateFrame("Texture",self.description .. "Point", self.frame)
	self.frame.chatPoint:SetWidth(60)
	self.frame.chatPoint:SetHeight(15)
	self.frame.chatPoint:SetPoint("TOPLEFT", self.frame, "BOTTOMLEFT",10,0)
	self.frame.chatPoint:SetLayer(999)
	self.frame.chatPoint:SetVisible(true)
	self.frame.chatPoint:SetTexture("LibTUtility", "\\images\\toolTip.png")

	self.frame.left = UI.CreateFrame("Texture",self.description .. "Point", self.frame)
	self.frame.left:SetPoint("TOPRIGHT", self.frame, "TOPLEFT")
	self.frame.left:SetPoint("BOTTOMRIGHT", self.frame, "BOTTOMLEFT")
	self.frame.left:SetLayer(999)
	self.frame.left:SetVisible(true)
	self.frame.left:SetTexture("LibTUtility", "\\images\\toolTipLeft.png")

	self.frame.right = UI.CreateFrame("Texture",self.description .. "Point", self.frame)
	self.frame.right:SetPoint("TOPLEFT", self.frame, "TOPRIGHT")
	self.frame.right:SetPoint("BOTTOMLEFT", self.frame, "BOTTOMRIGHT")
	self.frame.right:SetLayer(999)
	self.frame.right:SetVisible(true)
	self.frame.right:SetTexture("LibTUtility", "\\images\\toolTipRight.png")

	self.frame.text = UI.CreateFrame("Text",self.description .. "Text", self.frame)

	self.frame.text:SetPoint("CENTER",self.frame,"CENTER")
	self.frame.text:SetVisible(true)
	self.frame.text:SetText("Testing")
	self.frame:SetVisible(false)
	self.frame.text:SetFontSize(14)
	self.frame.text:SetFontColor(0, 0, 0)
end

local updateHandler = nil

function ToolTip:Show(show,top,left, text, height, width)
	local showAt = Inspect.Time.Frame() + 1
	Command.Event.Detach(Event.System.Update.Begin,updateHandler, "Tooltip3SecondDisplay")

	if(not show)then
		self.frame:SetVisible(false)
		return
	end

	local updateHandler = function()
		local now = Inspect.Time.Frame()
		if(now > showAt)then
			self.frame.text:SetText(text)
			self.frame:SetHeight(height or self.size.height)
			self.frame:SetWidth(width or self.size.width)
			self.frame.left:SetWidth((height or self.size.height) / 2)
			self.frame.right:SetWidth((height or self.size.height) / 2)

			self.frame:SetPoint("TOPLEFT",UIParent,"TOPLEFT", left, top - self.frame:GetHeight() - 15)
			self.frame:SetVisible(true)
			Command.Event.Detach(Event.System.Update.Begin,updateHandler, "Tooltip3SecondDisplay")
			self.frame.text.Event.LeftDown = function()
				self.frame:SetVisible(false)
			end
		end
	end
	Command.Event.Attach(Event.System.Update.Begin, updateHandler, "Tooltip3SecondDisplay")
end
