local addonData, triggers = ...
extTriggers = triggers
triggers.addonId = "triggers"

Triggers = {}
Triggers.Event = {}
Triggers.Event.Trigger = {}
Triggers.Event.Trigger.ContextCreated, Triggers.Event.ContextCreated = Utility.Event.Create(triggers.addonId, "ContextCreated")
Triggers.Event.Trigger.NotificationsLoaded, Triggers.Event.NotificationsLoaded = Utility.Event.Create(triggers.addonId, "NotificationsLoaded")
local addonData, triggers = ...


local BWMain = {}
BWMain.__index = BWMain

function BWMain.Create()
	local obj = {}
	setmetatable(obj,BWMain)
	obj:Init()
	return obj
end

function BWMain:Init()
	self.SlashCommands = {
		["config"] = { function(parameters) triggers.BWMain:ParseConfig(parameters) end, "Sets config settings /config [SettingName] [Value]" },
		["debug"] = { function() libTDebug.UIDebug:Show() end, "shows Debug screen" },
		["locate"] = { function() triggers.MapIcon:Locate() end, "Moves button to center screen" },
		["clean"] =  { function() triggers.BWMain:Clean() end, "Removes all default triggers leaving empty notifications." },
		["install"] =  { function() triggers.BWMain:Install() end, "Adds all default triggers to default notifications.  Can be used to update triggers." },
	}
 	self.lastCheckCPU =Inspect.Time.Frame()
 	self.settingsType = {
 		["MaxStaticTextSize"] = "numeric",
 		["StackTextSize"] = "percent",
 		["StaticTextSize"] = "percent",
 		["TimerTextSize"] = "percent",
 		["ShowDecimalTimer"] = "bool",
 		["FontWeight"] = "percent"
 	}
end

local defaultSettings = {

}

function BWMain:Install()
	print("Installing default triggers.  Current triggers will be updated.")
	for _, id in ipairs(triggers.defaults.sharedNotifications) do
		local dn = triggers.defaults.defaultNotifications[id]
		local n = triggers.Notifications:GetNotification(id)
		for _, t in ipairs(dn.triggers)do
			local myt = n:GetTriggerById(t.id)
			local exists = myt or false
			myt = triggers.Trigger.Deserialize(t, myt)
			if(not exists)then
				n:AppendTrigger(myt)
			end
		end
	end
end

function BWMain:Clean()
	print("Removing default triggers from notifications.")
	for _, id in ipairs(triggers.defaults.sharedNotifications) do
		local dn = triggers.defaults.defaultNotifications[id]
		local n = triggers.Notifications:GetNotification(id)
		for _, t in ipairs(dn.triggers)do
			n:RemoveTriggerById(t.id)
		end
	end
end

function BWMain:ParseConfig(parameters)
	if(parameters[2] ~= nil)then
		if(parameters[3] == nil and TRIGGER_CONFIG.Settings[parameters[2]] ~= nil)then
			print(string.format("%s = %s",parameters[2],TRIGGER_CONFIG.Settings[parameters[2]]))
		elseif(parameters[3] ~= nil)then
			if(self.settingsType[parameters[2]] == "numeric")then
				if(tonumber(parameters[3])~=nil)then
					TRIGGER_CONFIG.Settings[parameters[2]] = tonumber(parameters[3])
				else
					print("Invalid format, please enter a number")
				end
			elseif(self.settingsType[parameters[2]] == "bool")then
				local value = string.lower(parameters[3])
				if(value == "y" or value == "1" or value =="true" or value == "yes")then
					TRIGGER_CONFIG.Settings[parameters[2]] = true
				else
					TRIGGER_CONFIG.Settings[parameters[2]] = false
				end
			elseif(self.settingsType[parameters[2]] == "percent")then
				local value = tonumber(parameters[3])
				if(value <= 1) then
					TRIGGER_CONFIG.Settings[parameters[2]] = value
				elseif(value < 100)then
					TRIGGER_CONFIG.Settings[parameters[2]] = value / 100
				end
			end
		end
	end
end

function triggers.Tooltip(parent,text, height, width, mouseIn, mouseOut)
	function parent.Event:MouseIn()
		local t = self:GetTop()
		local l = self:GetLeft()
		triggers._tooltip:Show(true,t,l, text, height, width)
		if(mouseIn)then
			mouseIn(parent)
		end
	end
	function parent.Event:MouseOut()
		triggers._tooltip:Show(false)
		if(mouseOut)then
			mouseOut(parent)
		end
	end
end

function triggers.TootipClose()
	triggers._tooltip:Show(false)
end

function BWMain:ToggleSetting(id)
	TRIGGER_CONFIG[id] = not TRIGGER_CONFIG[id]
	print(string.format("%s changed to %s", id, tostring(TRIGGER_CONFIG[id])))
end

function BWMain:UISetup()
	self.UIContext = UI.CreateContext("BWMainContext")
	triggers.ContextMenu:AddMenu(triggers.MenuItem_NewNotification)
	triggers.ContextMenu:AddMenu(triggers.MenuItem_NewAbility)
	triggers.ContextMenu:AddMenu(triggers.MenuItem_Sharing)
	triggers.ContextMenu:AddMenu(triggers.MenuItem_Debug)
	triggers.ContextMenu:AddMenu(triggers.MenuItem_CPUGraph)
	triggers.ContextMenu:AddMenu(triggers.MenuItem_Import)
	triggers.ContextMenu:AddMenu(triggers.menuItem_ShowFilter)

	Triggers.Event.Trigger.ContextCreated(self.UIContext)
	triggers._tooltip = libTUI.ToolTip.Create(self.UIContext)

	local detail = Inspect.Unit.Detail("player")
	triggers.UnitId = detail.name -- .. "@" .. Inspect.Shard().name
end

function BWMain.OnVariablesLoaded(event, addonId)
	if addonId ~= triggers.addonId then return end
	TRIGGER_CONFIG = TRIGGER_CONFIG or defaultSettings
	TRIGGER_CONFIG.Settings = TRIGGER_CONFIG.Settings or {}
	if(TRIGGER_CONFIG.Settings.ShowDecimalTimer==nil)then
		TRIGGER_CONFIG.Settings.ShowDecimalTimer = true
	end
	TRIGGER_CONFIG.Settings.FontWeight = TRIGGER_CONFIG.Settings.FontWeight or .6
	TRIGGER_CONFIG.Settings.MaxStaticTextSize = TRIGGER_CONFIG.Settings.MaxStaticTextSize or 100
	TRIGGER_CONFIG.Settings.StaticTextSize = TRIGGER_CONFIG.Settings.StaticTextSize or .8
	TRIGGER_CONFIG.Settings.StackTextSize = TRIGGER_CONFIG.Settings.StackTextSize or .8
	TRIGGER_CONFIG.Settings.TimerTextSize = TRIGGER_CONFIG.Settings.TimerTextSize or .8

	LibVersionCheck.register(addonData.toc.Identifier, addonData.toc.Version)
end

function BWMain.OnVariablesSaving(event, addonId)
	if addonId ~= triggers.addonId then return end
	TRIGGER_CONFIG = TRIGGER_CONFIG or defaultSettings
end

function BWMain:Split(inputstr, sep)
        if sep == nil then
                sep = "%s"
        end
        local t={} ; i=1
        for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
                t[i] = str
                i = i + 1
        end
        return t
end

function BWMain.OnSlash(handle, parameterString)
	local self = triggers.BWMain
	local parameters =  self:Split(parameterString)
	local parameter = parameters[1]
	if (parameter == "Export")then
		print("exporting")
		triggers.defaults.ExportDefaultNotificationsByEncounter()
	elseif(self.SlashCommands[parameter] ~= nil) then
		self.SlashCommands[parameter][1](parameters)
	else
		for key, value in pairs(self.SlashCommands) do
			print(string.format("/triggers %s - %s", key, value[2]))
		end
	end
end

function BWMain.RunFirst(handle, units)
	-- crazyness - this was needed to initialize some values.
	local self = triggers.BWMain
	for id, unitSpecifier in pairs(units)do
		if(unitSpecifier == "player")then
			triggers.BWMain:UISetup()
			Command.Event.Detach(Event.Unit.Availability.Full, BWMain.RunFirst)
		end
	end
end

triggers.BWMain = BWMain.Create()

Command.Event.Attach(Event.Addon.SavedVariables.Load.End, BWMain.OnVariablesLoaded, "Event.Addon.SavedVariables.Load.End")
Command.Event.Attach(Event.Addon.SavedVariables.Save.Begin, BWMain.OnVariablesSaving, "Event.Addon.SavedVariables.Save.Begin")
Command.Event.Attach(Event.Unit.Availability.Full,BWMain.RunFirst, "BWMain.RunFirst")
Command.Event.Attach(Command.Slash.Register("triggers"), BWMain.OnSlash, "SlashRegisterCommand.Buffs")




