--
--  Class ContextMenu

local addonData, triggers = ...

local ContextMenu = {}
ContextMenu.__index = ContextMenu

function ContextMenu.Create()
	local obj = {}
	setmetatable(obj,ContextMenu)
	obj:Init()
	return obj
end

function ContextMenu:Init()
	self.items = {}
	self.count = 0
	self.width = 200
	self.frameOutset = 120
	self.itemHeight = 33
	self.description = "ContextMenu"
	self.parent = nil
end

function ContextMenu:UISetup(parent)
	self.frame = UI.CreateFrame("Frame",self.description, parent)
	self.frame.parent = parent
	self.frame:SetWidth(self.width)
	self.frame:SetHeight(#self.items * self.itemHeight + self.frameOutset)
	self.frame:SetMouseMasking("limited")
	self.frame:SetLayer(2)
	TRIGGER_CONFIG.ContextMenu = TRIGGER_CONFIG.ContextMenu or {x= 0, y= -(#self.items * self.itemHeight + self.frameOutset)}
	self.frame:SetPoint("TOPLEFT", parent, "TOPLEFT",TRIGGER_CONFIG.ContextMenu.x, TRIGGER_CONFIG.ContextMenu.y)
	self.frame:SetBackgroundColor(0,0,0,.8)
	Library.LibSimpleWidgets.SetBorder("rounded", self.frame, 1,1,1,1)
	self.frame:SetVisible(false)

	self.frame.header = UI.CreateFrame("Frame", self.description .. "_HeaderContainer", self.frame)
	self.frame.header:SetWidth(self.width)
	self.frame.header:SetHeight(self.itemHeight)
	self.frame.header:SetMouseMasking("limited")
	self.frame.header:SetLayer(3)
	self.frame.header:SetVisible(true)
	self.frame.header:SetPoint("TOPLEFT", self.frame, "TOPLEFT")
	self.frame.bottomItem = self.frame.header
	self.frame.header:SetBackgroundColor(.1,.1,.1, 1)
	self.frame.header.text = UI.CreateFrame("Text", self.description .. "_HeaderText", self.frame.header)
	self.frame.header.text:SetPoint("TOPCENTER", self.frame.header, "TOPCENTER")
	self.frame.header.text:SetText("Triggers")
	self.frame.header.text:SetFontColor(1,1,0)
	self.frame.header.text:SetFontSize(20)
	self.frame.menuItems = {}
	self.frame.header.frame = self.frame

	for _, item in ipairs(self.items) do
		local menuItem = nil
		if(item.selfDrawn)then
			menuItem = item.handler:CunstructMenuItem(self.frame, self.frame.bottomItem, self.width)
		else
			menuItem = self:ConstructMenuItem(item, self.frame, self.frame.bottomItem)
		end
		self.frame.bottomItem = menuItem
		self.frame.menuItems[item.description] = menuItem
	end

	function self.frame.header.Event:MouseIn()
		triggers.ContextMenu.frame.header.text:SetText("< close menu >")
		triggers.ContextMenu.frame.header.text:SetFontColor(1,0,0)
	end
	function self.frame.header.Event:MouseOut()
		triggers.ContextMenu.frame.header.text:SetText("Triggers")
		triggers.ContextMenu.frame.header.text:SetFontColor(1,1,0)
	end

	function self.frame.header.Event:LeftClick()
		if self.moved == false  then
			triggers.ContextMenu.frame:SetVisible(false)
		end
		self.moved = false
	end

	function self.frame.header.Event:LeftUp()
		TRIGGER_CONFIG.ContextMenu = TRIGGER_CONFIG.ContextMenu or {}
		TRIGGER_CONFIG.ContextMenu.x = self.frame:GetLeft() - self.frame.parent:GetLeft()
		TRIGGER_CONFIG.ContextMenu.y = self.frame:GetTop() - self.frame.parent:GetTop()
		self.frame:SetPoint("TOPLEFT", self.frame.parent, "TOPLEFT",TRIGGER_CONFIG.ContextMenu.x , TRIGGER_CONFIG.ContextMenu.y)
		self.grab = false
	end

	function self.frame.header.Event:LeftDown()
		self.grab = true
		self.moved = false
		m = Inspect.Mouse()
		self.grabOffset = { x = m.x - self:GetLeft(), y = m.y - self:GetTop() }
	end

	function self.frame.header.Event:MouseMove(x,y)
		if self.grab then
			self.frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", (x - self.grabOffset.x) , (y - self.grabOffset.y))
			self.moved = true
		end
	end
end

function ContextMenu:ConstructMenuItem(item, parent, alignFrame)
	local menuItemElement = UI.CreateFrame("Frame", self.description .. "_" .. item.description .. "Container", parent)
	menuItemElement:SetWidth(self.width)
	menuItemElement:SetHeight(self.itemHeight)
	menuItemElement:SetMouseMasking("limited")
	menuItemElement:SetLayer(3)
	menuItemElement:SetVisible(true)
	menuItemElement:SetPoint("TOPLEFT", alignFrame, "BOTTOMLEFT")
	menuItemElement.menuItem = item
	menuItemElement:SetBackgroundColor(0,0,0,0)
	menuItemElement.text = UI.CreateFrame("Text", self.description .. "_HeaderText", menuItemElement)
	menuItemElement.text:SetPoint("TOPLEFT", menuItemElement, "TOPLEFT", 10, 3)
	menuItemElement.text:SetText(item.description)
	menuItemElement.text:SetFontSize(18)

	function menuItemElement.Event:MouseOut()
		self.text:SetFontColor(1,1,1,1)
	end

	function menuItemElement.Event:MouseIn()
		self.text:SetFontColor(0,1,.5, 1)
	end

	function menuItemElement.Event:LeftClick()
		triggers.ContextMenu.frame:SetVisible(false)
		if(self.menuItem.handler ~= nil)then
			self.menuItem.handler:MenuClicked()
		end
	end

	return menuItemElement
end

function ContextMenu:Toggle()
	self.frame:SetVisible(not self.frame:GetVisible())
end

function ContextMenu:AddMenu(menuItem)
	table.insert(self.items, { description = menuItem.description, handler = menuItem, selfDrawn = menuItem.selfDrawn })
end

triggers.ContextMenu = ContextMenu.Create()

