--
--  Class MapIcon

local addonData, triggers = ...

local MapIcon = {}
MapIcon.__index = MapIcon

function MapIcon.Create()
	local obj = {}
	setmetatable(obj,MapIcon)
	obj:Init()
	return obj
end

function MapIcon:Init()
	self.items = {}
	self.width = 40
	self.height = 40
	self.description = "MapIcon"
end

function MapIcon:Locate()
	self.frame:ClearPoint("CENTER")
	self.frame:SetPoint("TOPLEFT", UIParent, "CENTER", 0,0)

end

function MapIcon.UISetup(handle,parent)
	self = triggers.MapIcon
	self.frame = UI.CreateFrame("Frame",self.description, parent)
	self.frame:SetWidth(self.width)
	self.frame:SetHeight(self.height)
	self.frame:SetMouseMasking("limited")
	self.frame:SetLayer(1)
	self.frame:SetVisible(true)
	self.frame:SetPoint("CENTER", UIParent, "CENTER", 0,0)

	self.frame.texture = UI.CreateFrame("Texture", self.description .. "_Texture", self.frame)
	self.frame.texture:SetWidth(self.width)
	self.frame.texture:SetHeight(self.height)
	self.frame.texture:SetPoint("CENTER",self.frame, "CENTER")
	self.frame.texture:SetLayer(0)
	self.frame.texture:SetTexture("triggers", "images\\button.png")
	self.frame.texture:SetVisible(true)

	TRIGGER_CONFIG.MapIcon = TRIGGER_CONFIG.MapIcon or { x = self.frame:GetLeft(), y = self.frame:GetTop() }
	self.frame:ClearPoint("CENTER")
	self.frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", TRIGGER_CONFIG.MapIcon.x, TRIGGER_CONFIG.MapIcon.y)

	function self.frame.Event:LeftDown()
		self.grab = true
		self.moved = false
		m = Inspect.Mouse()
		self.grabOffset = { x = m.x - self:GetLeft(), y = m.y - self:GetTop() }
	end

	function self.frame.Event:MouseMove(x,y)
		if self.grab then
			self:SetPoint("TOPLEFT", UIParent, "TOPLEFT", (x - self.grabOffset.x) , (y - self.grabOffset.y))
			self.moved = true
		end
	end

	function self.frame.Event:LeftUp()
		TRIGGER_CONFIG.MapIcon = TRIGGER_CONFIG.MapIcon or {}
		TRIGGER_CONFIG.MapIcon.x = self:GetLeft()
		TRIGGER_CONFIG.MapIcon.y = self:GetTop()
		self.grab = false
	end

	function self.frame.Event:RightClick()
		triggers.ContextMenu:Toggle()
	end

	function self.frame.Event:LeftClick()
		if self.moved == false then
			triggers.MapIcon:EditMode()
		end
		self.moved = false
	end

	triggers.ContextMenu:UISetup(self.frame)
end

function MapIcon:HideEditMode()
	triggers.Notifications:EditMode(false)
	self.frame.texture:SetTexture("triggers", "images\\button.png")
end

function MapIcon:ShowEditMode()
	triggers.Notifications:EditMode(true)
	self.frame.texture:SetTexture("triggers", "images\\buttonEditMode.png")
end

function MapIcon:EditMode()
	triggers.Notifications:ToggleEdit()
	if(triggers.Notifications.isEditMode)then
		self.frame.texture:SetTexture("triggers", "images\\buttonEditMode.png")
	else
		triggers.Configuration.Notifications.Config:Close()
		self.frame.texture:SetTexture("triggers", "images\\button.png")
	end
end

triggers.MapIcon = MapIcon.Create()

Command.Event.Attach(Triggers.Event.ContextCreated , MapIcon.UISetup, "MapIcon.UISetup")