--
--  Class Notification

local addonData, triggers = ...

triggers.Notification = {}
local Notification = triggers.Notification
Notification.__index = Notification

Notification.startSize = 128

triggers.DisplayIf = {
    ANY = 1,
    ALL = 2,
    NONE = 3,
        toString = function (value)
            local map = {"ANY","ALL","NONE"}
            return map[value]
        end
}

function Notification.Create(resource, image)
    local obj = {}
    setmetatable(obj,Notification)
    obj:Init()
    obj.id = libTUtility.uuid()
    if(resource ~= nil and image ~= nil)then
        obj.image = resource .. "|" .. image
    end
    return obj
end

function Notification:Init()
    self.shareGroupType = nil
    self.image = nil
    self.size = Notification.startSize
    self.position = { top = 0, left = 0 }
    self.opacity = 100
    self.text = nil
    self.displayIf = triggers.DisplayIf.ANY
    self.combatOnly = false
    self.fontSize = "25"
    self.abilityIcon = true
    self.disabled = false
    self.triggers = {}
    self.description = "Notification"
    self.roles = ""
    self.deletable = true
    self.description = ""
    self.layer = 1
    self.state = {text = false,
                                stacks = false,
                                timer = false,
                                opacity = 1,
                                timerText = "",
                                stackText = "",
                                headerPoint = "CENTER",
                                timeRemainingPoint = "TOPRIGHT",
                                stacksPoint = "BOTTOMLEFT",
                                textMask = "",
                                color = { 1, 1, 1 } }
    self.color = { 1, 1, 1 }
end

function Notification:SetTextColor(color)
    self.frame.stacks:SetFontColor(color[1],color[2],color[3],1)
    self.frame.header:SetFontColor(color[1],color[2],color[3],1)
    self.frame.timeRemaining:SetFontColor(color[1],color[2],color[3],1)
end

                                        --{timeLeft = nil,
                                        --stacks = nil,
                                        --now = now,
                                        --text = nil,
                                        --color = nil,
                                        --opacity = nil,
                                        --displayAbilityId = nil,
                                        --visible = false}
function Notification:SetViewState(newState)
    if not self.initialized then return end
    newState = newState or {}
    if(newState.visible)then
        if newState.timeLeft == nil or newState.timeLeft <= 0 then
            newState.newOpacity = (newState.opacity or self.opacity) * .01
            newState.timerText = nil
        elseif(newState.timeLeft >= 1)then
            newState.newOpacity = (newState.opacity or self.opacity) * .01
            newState.timerText = string.format("%d",  math.floor(newState.timeLeft))
        elseif(newState.showSpark)then
            newState.newOpacity = (newState.opacity or self.opacity) * newState.timeLeft * .01
            if(TRIGGER_CONFIG.Settings.ShowDecimalTimer)then
            	newState.timerText = string.format("%d", newState.timeLeft * 10 )
            else
            	newState.timerText = "0"
            end
        else
            newState.newOpacity = (newState.opacity or self.opacity) * .01
            if(TRIGGER_CONFIG.Settings.ShowDecimalTimer)then
	            newState.timerText = string.format("%d", newState.timeLeft * 10 )
            else
            	newState.timerText = "0"
            end
        end
        if  newState.stacks ~= nil and newState.stacks > 0 then
            newState.stackText = string.format("%d", newState.stacks)
        end
        if(newState.displayAbilityId and self.abilityIcon)then
            local i = libAbilityCatalog.Catalog:GetImage(newState.displayAbilityId)
            if(i)then
                newState.image = "Rift|" .. libAbilityCatalog.Catalog:GetImage(newState.displayAbilityId)
            end
        elseif(not self.abilityIcon)then
            newState.image = self.image
        end
        if(newState.color == nil)then
            newState.color = {}
            newState.color[1] = self.color[1]
            newState.color[2] = self.color[2]
            newState.color[3] = self.color[3]
        end
    else
        newState.stackText = nil
        newState.text = nil
        newState.timerText = nil
        newState.newOpacity = self.opacity
        newState.color = {}
        newState.image = self.image
        newState.color[1] = self.color[1]
        newState.color[2] = self.color[2]
        newState.color[3] = self.color[3]
    end
    local visiblilityChange = false
    if(newState.visible ~= self.state.visible)then
        visiblilityChange = true
        self.state.visible = newState.visible
        local visible = self.state.visible or false
        self.frame:SetVisible(visible)
        self.frame.texture:SetVisible(visible)
    end
    local textChange = false
    local timeChange = false
    local stacksChange = false
    local textColorChange = newState.color[1] ~= self.state.color[1] or newState.color[2] ~= self.state.color[2] or newState.color[3] ~= self.state.color[3]
    if(textColorChange)then
        self.state.color[1] = newState.color[1]
        self.state.color[2] = newState.color[2]
        self.state.color[3] = newState.color[3]
        self:SetTextColor(self.state.color)
    end
    if(newState.timerText ~= self.state.timerText)then
        self.state.timerText = newState.timerText
        self.frame.timeRemaining:SetText(self.state.timerText or "")
        self.frame.timeRemaining:SetVisible(self.state.timerText and true or false)
        timeChange = true
    end
    if(newState.stackText ~= self.state.stackText)then
        self.state.stackText = newState.stackText
        self.frame.stacks:SetText(self.state.stackText or "")
        self.frame.stacks:SetVisible(self.state.stackText and true or false)
        stacksChange = true
    end

    if((newState.text or self.text) ~= self.state.text)then
        self.state.text = newState.text or self.text
        self.frame.header:SetText(self.state.text or "")
        self.frame.header:SetVisible(self.state.text and true or false)
        textChange = true
    end

    if(newState.image ~= self.state.image)then
        self.state.image = newState.image
        if(self.state.image)then
            local imageInfo = libTUtility.StringSplit(self.state.image, "|")
            self.frame.texture:SetTexture(imageInfo[1],imageInfo[2])
        end
    end
    if(newState.newOpacity ~= self.state.opacity)then
        self.state.opacity = newState.newOpacity
        self.frame.texture:SetAlpha(self.state.opacity)
    end
    if(textChange or timeChange or stacksChange)then
        self:RepositionText()
    end
end

function Notification:RepositionText()
    if(self.state.text and self.state.timerText and self.state.stackText)then
        -- all
        if(self.state.textMask == "ALL") then return end
        self.frame.header:ClearPoint(self.state.headerPoint)
        self.state.headerPoint = "CENTER"
        self.frame.header:SetPoint(self.state.headerPoint, self.frame, self.state.headerPoint)
        self.frame.header:SetFontSize(self.size * .35)
        self.frame.timeRemaining:ClearPoint(self.state.timeRemainingPoint)
        self.state.timeRemainingPoint = "TOPRIGHT"
        self.frame.timeRemaining:SetPoint(self.state.timeRemainingPoint, self.frame, self.state.timeRemainingPoint)
        self.frame.timeRemaining:SetFontSize(self.size * .275)
        self.frame.stacks:ClearPoint(self.state.stacksPoint)
        self.state.stacksPoint = "BOTTOMLEFT"
        self.frame.stacks:SetPoint(self.state.stacksPoint, self.frame, self.state.stacksPoint)
        self.frame.stacks:SetFontSize(self.size * .275)

        self.state.textMask = "ALL"
    elseif(self.state.text and (self.state.timerText or self.state.stackText))then
        -- text and one other
        if(self.state.timerText)then
            if(self.state.textMask == "TEXT:TIMER") then return end
            self.frame.header:ClearPoint(self.state.headerPoint)
            self.state.headerPoint = "BOTTOMCENTER"
            self.frame.header:SetPoint(self.state.headerPoint, self.frame, self.state.headerPoint)
            self.frame.timeRemaining:ClearPoint(self.state.timeRemainingPoint)
            self.state.timeRemainingPoint = "TOPRIGHT"
            self.frame.timeRemaining:SetPoint(self.state.timeRemainingPoint, self.frame, self.state.timeRemainingPoint)
            self.frame.timeRemaining:SetFontSize(self.size * .45)
            self.state.textMask = "TEXT:TIMER"
        else
            if(self.state.textMask == "TEXT:STACKS") then return end
            self.frame.header:ClearPoint(self.state.headerPoint)
            self.state.headerPoint = "TOPCENTER"
            self.frame.header:SetPoint(self.state.headerPoint, self.frame, self.state.headerPoint)
            self.frame.stacks:ClearPoint(self.state.stacksPoint)
            self.state.stacksPoint = "BOTTOMLEFT"
            self.frame.stacks:SetPoint(self.state.stacksPoint, self.frame, self.state.stacksPoint)
            self.frame.stacks:SetFontSize(self.size * .45)
            self.state.textMask = "TEXT:STACKS"
        end
        self.frame.header:SetFontSize(self.size * .5)
    elseif(self.state.text)then
        -- only text
        if(self.state.textMask == "TEXT") then return end
        self.frame.header:ClearPoint(self.state.headerPoint)
        self.state.headerPoint = "CENTER"
        self.frame.header:SetPoint(self.state.headerPoint, self.frame, self.state.headerPoint)
        local size = self.size * TRIGGER_CONFIG.Settings.StaticTextSize
        if(size > TRIGGER_CONFIG.Settings.MaxStaticTextSize)then
            size = TRIGGER_CONFIG.Settings.MaxStaticTextSize
        end
        self.frame.header:SetFontSize(size)
        self.state.textMask = "TEXT"
    elseif(self.state.timerText and self.state.stackText)then
        -- both timer and stacks
        if(self.state.textMask == "TIMER:STACKS") then return end
        self.frame.stacks:ClearPoint(self.state.stacksPoint)
        self.state.stacksPoint = "BOTTOMLEFT"
        self.frame.stacks:SetPoint(self.state.stacksPoint, self.frame, self.state.stacksPoint)
        self.frame.stacks:SetFontSize(self.size * .45)
        self.frame.timeRemaining:ClearPoint(self.state.timeRemainingPoint)
        self.state.timeRemainingPoint = "TOPRIGHT"
        self.frame.timeRemaining:SetPoint(self.state.timeRemainingPoint, self.frame, self.state.timeRemainingPoint)
        self.frame.timeRemaining:SetFontSize(self.size * .45)
        self.state.textMask = "TIMER:STACKS"
    elseif(self.state.timerText or self.state.stackText)then
        -- either timer or stacks
        if(self.state.timerText)then
            self.frame.timeRemaining:ClearPoint(self.state.timeRemainingPoint)
            self.state.timeRemainingPoint = "CENTER"
            self.frame.timeRemaining:SetPoint(self.state.timeRemainingPoint, self.frame, self.state.timeRemainingPoint)
            self.frame.timeRemaining:SetFontSize(self.size * TRIGGER_CONFIG.Settings.TimerTextSize)
            self.state.textMask = "TIMER"
        else
            if(self.state.textMask == "STACKS") then return end
            self.frame.stacks:ClearPoint(self.state.stacksPoint)
            self.state.stacksPoint = "CENTER"
            self.frame.stacks:SetPoint(self.state.stacksPoint, self.frame, self.state.stacksPoint)
            self.frame.stacks:SetFontSize( self.size * TRIGGER_CONFIG.Settings.StackTextSize)
            self.state.textMask = "STACKS"
        end
    end
end

function Notification:UpdateImage()
    local image
    if(self.abilityIcon)then
        for _,trigger in ipairs(self.triggers) do
            image = libAbilityCatalog.Catalog:GetImage(trigger.abilityId or trigger.name)
            if(image)then
                self.frame.texture:SetTexture("Rift",image)
                break

            end
        end
    else
        if(self.image)then
            local imageInfo = libTUtility.StringSplit(self.image, "|")
            self.frame.texture:SetTexture(imageInfo[1],imageInfo[2])
        else
            self.frame.texture:SetTexture(triggers.addonId,"\\images\\empty.png")
        end
    end
    self.frame:SetWidth(self.size)
    self.frame:SetHeight(self.size)
    self.frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", self.position.left, self.position.top)
end

function Notification:GetTrigger(index)
    return self.triggers[index]
end

function Notification:GetTriggerById(id)
    for _,v in ipairs(self.triggers) do
        if(v.id == id)then
            return v
        end
    end
    return nil
end

function Notification:AppendTrigger(trigger)
    table.insert(self.triggers,trigger)
end

function Notification:PrependTrigger(trigger)
    table.insert(self.triggers,1,trigger)
end

function Notification:RemoveTrigger(index)
    table.remove(self.triggers,index)
end

function Notification:RemoveTriggerById(id)
    local deleteIndex = nil
    for i, t in ipairs(self.triggers)do
        if(t.id == id)then
            deleteIndex = i
            break
        end
    end
    if(deleteIndex ~= nil)then
        self:RemoveTrigger(deleteIndex)
    end
end

function Notification:TriggerUp(index)
    if(index > 1)then
        local t = self.triggers[index]
        self:RemoveTrigger(index)
        index = index - 1
        table.insert(self.triggers,index,t)
    end
end

function Notification:TriggerDown(index)
    if(index < #self.triggers)then
        local t = self.triggers[index]
        self:RemoveTrigger(index)
        index = index + 1
        table.insert(self.triggers,index,t)
    end
end

function Notification:StringTriggerList()
    local r = {}
    for _,v in ipairs(self.triggers) do
        local t = v:toString()
        table.insert(r,t)
    end
    return r
end

function Notification:StringTriggerTable()
    local r = {}
    for _,v in ipairs(self.triggers) do
        local t = v:toString()
        r[v.id] = t
    end
    return r
end

function Notification.SerializeValues(n)
    local section = {}

    section.image = n.image
    section.size = n.size
    section.position = n.position
    section.opacity = n.opacity
    if(n.text ~= nil)then
        n.text = libTUtility.Trim(n.text)
        if(string.len(n.text) <= 0)then
            n.text = nil
        end
    end
    section.text = n.text
    section.roles = n.roles
    section.color =  { n.color[1], n.color[2], n.color[3] }

    section.id = n.id
    section.displayIf = n.displayIf
    section.combatOnly =n.combatOnly
    section.abilityIcon = n.abilityIcon
    section.disabled = n.disabled
    section.deletable = n.deletable
    section.description = n.description
    section.layer = n.layer or 1

    return section
end

function Notification:Serialize()
    local clone = triggers.Notification.SerializeValues(self)
    if(#self.triggers > 0)then
        clone.triggers = {}
        for _, trigger in ipairs(self.triggers) do
            table.insert(clone.triggers, trigger:Serialize())
        end
    end
    return clone
end

function Notification:Resize(clientSize)
    local nw = math.floor(UIParent:GetWidth())
    local nh = math.floor(UIParent:GetHeight())
    local cw = clientSize.width
    local ch = clientSize.height


    local rw = nw / cw
    local rh = nh / ch

    self.position.left = math.floor(self.position.left * rw)
    self.position.top = math.floor(self.position.top * rh)
    self.size = math.floor(self.size * rw)
end

function Notification:SetLayer(layer)
	self.layer = layer
	self.frame:SetLayer(self.layer)
	self.frame.texture:SetLayer(self.layer + 20)
	self.frame.header:SetLayer(self.layer + 30)
	self.frame.timeRemaining:SetLayer(self.layer + 30)
	self.frame.stacks:SetLayer(self.layer + 30)
	self.frame.mask:SetLayer(self.layer + 40)
end

function Notification.Deserialize(section, parseTriggers, s)
    if(not s)then
        s = Notification.Create()
        if(section.image)then
            local imageInfo = libTUtility.StringSplit(section.image , "|")
            if(#imageInfo ~= 2)then
                section.image = triggers.addonId .. "|" .. section.image
            end
        end
        s.image = section.image
        s.size = section.size
        s.position = section.position
        s.opacity = section.opacity
        if(section.text ~= nil)then
            section.text = libTUtility.Trim(section.text)
            if(string.len(section.text) <= 0)then
                section.text = nil
            end
        end
        s.text = section.text
        s.roles = section.roles
        if(section.color ~= nil)then
            s.color = { section.color[1], section.color[2], section.color[3] }
        else
            s.color = {1,1,1}
        end
        s.layer = section.layer or 1
    end

    s.id = section.id
    s.displayIf = section.displayIf
    s.combatOnly = section.combatOnly
    s.abilityIcon = section.abilityIcon
    s.disabled = section.disabled
    s.deletable = section.deletable or false
    s.description = section.description or ""

    if(section.triggers ~= nil and parseTriggers ~= false)then
        for _, trigger in ipairs(section.triggers) do
            local existingTrigger = s:GetTriggerById(trigger.id)
            if(existingTrigger)then
                triggers.Trigger.Deserialize(trigger,existingTrigger)
            else
                t = triggers.Trigger.Deserialize(trigger)
                table.insert(s.triggers,t)
            end
        end
    end
    return s
end

function Notification:UISetup(parent)
    self.parent = parent
    local blur = (self.size - 15) / 300 * 10
    self.frame = UI.CreateFrame("Frame",self.description, self.parent)
    self.frame:SetWidth(self.size)
    self.frame:SetHeight(self.size)
    self.frame:SetMouseMasking("limited")
    self.frame:SetLayer(self.layer)
    self.frame:SetVisible(false)
    self.frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", self.position.left, self.position.top)
    self.frame:SetBackgroundColor(0,0,0,0)

    self.frame.texture = UI.CreateFrame("Texture",self.description .. "_Texture", self.frame)
    self.frame.texture:SetMouseMasking("limited")
    self.frame.texture:SetLayer(self.layer + 20)
    self.frame.texture:SetVisible(false)
    self.frame.texture:SetAllPoints()
    self.frame.texture:SetAlpha(self.opacity *.01)
    self:UpdateImage()

    self.frame.header = UI.CreateFrame("Text",self.description .. "_StaticText", self.frame)
    self.frame.header:SetLayer(self.layer + 30)
    self.frame.header:SetPoint("CENTER", self.frame, "CENTER")
    self.frame.header:SetVisible(false)
    self.frame.header:SetEffectGlow({colorA=1 * TRIGGER_CONFIG.Settings.FontWeight,colorB=0,colorG=0,colorR=0,blurX=blur ,blurY=blur,strength=10 * TRIGGER_CONFIG.Settings.FontWeight})

    self.frame.timeRemaining = UI.CreateFrame("Text",self.description .. "_RemainingTime", self.frame)
    self.frame.timeRemaining:SetLayer(self.layer + 30)
    self.frame.timeRemaining:SetPoint("TOPRIGHT", self.frame, "TOPRIGHT")
    self.frame.timeRemaining:SetVisible(true)
    self.frame.timeRemaining:SetVisible(false)
    self.frame.timeRemaining:SetEffectGlow({colorA=1 * TRIGGER_CONFIG.Settings.FontWeight,colorB=0,colorG=0,colorR=0,blurX=blur,blurY=blur,strength=10 * TRIGGER_CONFIG.Settings.FontWeight})

    self.frame.stacks = UI.CreateFrame("Text",self.description .. "_stacks", self.frame)
    self.frame.stacks:SetLayer(self.layer + 30)
    self.frame.stacks:SetPoint("BOTTOMLEFT", self.frame, "BOTTOMLEFT")
    self.frame.stacks:SetVisible(false)
    self.frame.stacks:SetEffectGlow({colorA=1 * TRIGGER_CONFIG.Settings.FontWeight,colorB=0,colorG=0,colorR=0,blurX=blur,blurY=blur,strength=10 * TRIGGER_CONFIG.Settings.FontWeight})

    self:RepositionText()

    self.frame.mask = UI.CreateFrame("Frame", self.description .. "_mask", self.frame)
    self.frame.mask:SetMouseMasking("limited")
    self.frame.mask:SetLayer(self.layer + 40)
    self.frame.mask:SetVisible(false)
    self.frame.mask:SetAllPoints()
    self.frame.mask:SetBackgroundColor(1,1,1,.2)
    self.frame.mask.notification = self
    self.frame.mask.shift = false
    self.frame.mask.dblClickElapse = Inspect.Time.Frame()

    function self.frame.mask.Event:LeftDown()
        self.grab = true
        m = Inspect.Mouse()
        self.grabOffset = { x = m.x - self:GetLeft(), y = m.y - self:GetTop() }
    end

    function self.frame.mask.Event:MouseMove(x,y)
        if self.grab then
            self.notification.position.left = (x - self.grabOffset.x)
            self.notification.position.top = (y - self.grabOffset.y)
            self.notification.frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", self.notification.position.left, self.notification.position.top)
            triggers.Configuration.Notifications.Config.SetPositionAndSize(self.notification)
        end
    end

    function self.frame.mask.Event:LeftUp()
        self.grab = false
        local now = Inspect.Time.Frame()
        if((now - self.dblClickElapse) < .4)then
            -- double click
            self.notification:Disable(not self.notification.disabled)
        end
        self.dblClickElapse = now

        local n = triggers.Notifications:FindLikeEdge(self.notification)
        if(n)then
            -- snap to calculate new position
            self.notification.position.left = n.x
            self.notification.position.top = n.y
            self.notification.frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", self.notification.position.left, self.notification.position.top)
        end

        triggers.Configuration.Notifications.Config.SetPositionAndSize(self.notification)
    end

    function self.frame.mask.Event:WheelBack()
        if(self.notification.size > 40)then
            self.notification:SetSize(self.notification.size - 10)
            triggers.Configuration.Notifications.Config.SetPositionAndSize(self.notification)
        end
    end

    function self.frame.mask.Event:WheelForward()
        if(self.notification.size < 500)then
            self.notification:SetSize(self.notification.size + 10)
            triggers.Configuration.Notifications.Config.SetPositionAndSize(self.notification)
        end
    end
    self.frame.mask.Event.RightClick = function()
        if(triggers.Configuration.Notifications.Config.visible)then
            triggers.Notifications:ShowOthersAfterConfigerz()
            triggers.Configuration.Notifications.Config:Close()
        else
            triggers.Configuration.Notifications.Config:Show(self)
        end
    end
    self.initialized = true
end

function Notification:Disable(v)
    self.disabled = v
    if(self.disabled)then
        self.frame.mask:SetBackgroundColor(0,0,0,.5)
    else
        self.frame.mask:SetBackgroundColor(1,1,1,.2)
    end
    triggers.Configuration.Notifications.Config.OnDisable(self, v)
end

function Notification:SetTop(top)
    self.position.top = top
    self.frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", self.position.left, self.position.top)
end
function Notification:SetLeft(left)
    self.position.left = left
    self.frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", self.position.left, self.position.top)
end
function Notification:SetSize(size)
    self.size = size
    Notification.startSize = self.size
    self.frame:SetWidth(self.size)
    self.frame:SetHeight(self.size)
    local size = self.size * .8
    if(size > TRIGGER_CONFIG.Settings.MaxStaticTextSize)then
        size = TRIGGER_CONFIG.Settings.MaxStaticTextSize
    end
    self.frame.header:SetFontSize(size)
end

function Notification:InTollerance(point1, point2, tollerance)
    return point1.x > point2.x - tollerance and point1.x < point2.x + tollerance and
        point1.y > point2.y - tollerance and point1.y < point2.y + tollerance
end

function Notification:CheckRegion(theirSize,topLeftCorner_theirs,topRightCorner_theirs,bottomLeftCorner_theirs,bottomRightCorner_theirs)
    local topRightCorner_mine = { x = self.position.left + self.size, y = self.position.top }
    local topLeftCorner_mine = { x = self.position.left, y = self.position.top }
    local bottomRightCorner_mine = { x = self.position.left + self.size, y = self.position.top + self.size }
    local bottomLeftCorner_mine = { x = self.position.left, y = self.position.top + self.size }
    local tollerance = 20

    if(self:InTollerance(topLeftCorner_theirs,topRightCorner_mine,tollerance) and
         self:InTollerance(bottomLeftCorner_theirs,bottomRightCorner_mine,tollerance))then
         return topRightCorner_mine
    end
    if(self:InTollerance(topLeftCorner_theirs,bottomLeftCorner_mine, tollerance) and
         self:InTollerance(topRightCorner_theirs,bottomRightCorner_mine, tollerance)) then
         return bottomLeftCorner_mine
    end
        if(self:InTollerance(topLeftCorner_mine,topRightCorner_theirs,tollerance) and
         self:InTollerance(bottomLeftCorner_mine,bottomRightCorner_theirs,tollerance))then
         return {x = self.position.left - theirSize, y = self.position.top}
    end
    if(self:InTollerance(topLeftCorner_mine,bottomLeftCorner_theirs, tollerance) and
         self:InTollerance(topRightCorner_mine,bottomRightCorner_theirs, tollerance)) then
         return {x = self.position.left, y = self.position.top - theirSize}
    end
    return nil
end

function Notification:Delete()
    self.frame:SetVisible(false)
    self.frame.mask:SetVisible(false)
    self.remove = true
    self.disabled = true
    triggers.Notifications:ShowOthersAfterConfigerz()
end

function Notification:EditMode(enabled)
    if(self.disabled)then
        self.frame.mask:SetBackgroundColor(0,0,0,.5)
    else
        self.frame.mask:SetBackgroundColor(1,1,1,.2)
    end
    self.frame.mask:SetVisible(enabled)
    if(enabled)then
        self:SetViewState({visible=true})
    else
        self:SetViewState({})
    end
end

function Notification:SpecMatch(spec)
    -- exit if the spec is not set, means user has no points in role - error
    if(spec == nil or string.len(spec) <= 0) then return false end
    -- check if the notification roles has a value then we verify spec matches
    if(self.roles ~= nil and string.len(self.roles) > 0)then
        local rs = libTUtility.StringSplit(self.roles,",")
        local roleMatch = false
        for _, v in ipairs(rs) do
            if(v ~= nil and string.len(v) > 0)then
                local magicCharsEscaped = string.gsub(v,'[%-%.%+%[%]%(%)%^%%%?%*]','%%%1')
                local specPattern = ".*" .. magicCharsEscaped .. ".*"
                if string.find(spec, specPattern) then
                    roleMatch = true
                    break
                end
            end
        end
        return roleMatch
    end
    return true
end
