--
-- Class CPUGraph
--
--   Manages a list of CPUGraph
local addonData, triggers = ...
local CPUGraph = {}

CPUGraph.__index = CPUGraph

local gridCount= 50
local scale = 3

function CPUGraph.Create()
	local obj = {}
	setmetatable(obj,CPUGraph)
	obj:Init()
	return obj
end

function CPUGraph:Init()
	self.size = { width = 240, height = 105 }
	self.description = "CPUGraph"
	self.visible = false
	self.lastCheckCPU = 0
	self.CPUList = CPUList.new()
	self.cor = nil
end

function CPUGraph.UISetup(handle,parent)
	local bCPUcontext = UI.CreateContext("TriggersCPUcontext")
	local self = triggers.CPUGraph
	self.parent = parent
	self.frame = UI.CreateFrame("Frame",self.description, bCPUcontext)
	self.frame:SetWidth(self.size.width)
	self.frame:SetHeight(self.size.height)
	self.frame:SetMouseMasking("limited")
	self.frame:SetLayer(101)
	Library.LibSimpleWidgets.SetBorder("rounded", self.frame, 1,1,1,1)
	self.frame:SetBackgroundColor(0.05,0.05,0.05,.95)
	self.frame:SetPoint("TOPLEFT",UIParent,"CENTER", self.size.width / 2 * -1, self.size.height / 2 * -1)
	self.frame:SetVisible(false)

	function self.frame.Event:LeftDown()
		self.grab = true
		self.moved = false
		m = Inspect.Mouse()
		self.grabOffset = { x = m.x - self:GetLeft(), y = m.y - self:GetTop() }
	end
	
	function self.frame.Event:MouseMove(x,y)
		if self.grab then
			self:SetPoint("TOPLEFT", UIParent, "TOPLEFT", (x - self.grabOffset.x) , (y - self.grabOffset.y))
			self.moved = true
		end
	end
	
	function self.frame.Event:LeftUp()
		self.grab = false
	end
	


	self.frame.grid = {}
	local x = UI.CreateFrame("Texture",string.format("%s %s" ,self.description, i) , self.frame)
	x:SetWidth(4)
	x:SetHeight(0)
	x:SetLayer(102)
	x:SetBackgroundColor(1,1,0,1)
	x:SetPoint("BOTTOMLEFT",self.frame,"BOTTOMLEFT",1,-5)
	table.insert(self.frame.grid,x)

	for i = 2, gridCount do
		x = UI.CreateFrame("Texture",string.format("%s %s" ,self.description, i) , self.frame)
		x:SetWidth(4)
		x:SetHeight(2)
		x:SetLayer(102)
		x:SetBackgroundColor(1,1,0,1)
		x:SetPoint("BOTTOMLEFT",self.frame.grid[i-1],"BOTTOMRIGHT",0,0)
		table.insert(self.frame.grid,x)
	end


	self.frame.label3 = UI.CreateFrame("Text",string.format("%s label %s" ,self.description, 3) , self.frame)
	self.frame.label3:SetWidth(10)
	self.frame.label3:SetLayer(102)
	self.frame.label3:SetPoint("TOPRIGHT",self.frame,"TOPRIGHT", -10,0)
	self.frame.label3:SetText("3%")

	self.frame.label2 = UI.CreateFrame("Text",string.format("%s label %s" ,self.description, 2) , self.frame)
	self.frame.label2:SetWidth(10)
	self.frame.label2:SetLayer(102)
	self.frame.label2:SetPoint("TOPRIGHT",self.frame,"TOPRIGHT", -10,35)
	self.frame.label2:SetText("2%")

	self.frame.label1 = UI.CreateFrame("Text",string.format("%s label %s" ,self.description, 1) , self.frame)
	self.frame.label1:SetWidth(10)
	self.frame.label1:SetLayer(102)
	self.frame.label1:SetPoint("TOPRIGHT",self.frame,"TOPRIGHT", -10,70)
	self.frame.label1:SetText("1%")
end

function CPUGraph:ToggleVisibility()
	self.visible = not self.visible
	self.frame:SetVisible(self.visible)
end



function CPUGraph.CPUUpdate()
	local self = triggers.CPUGraph
	local now = Inspect.Time.Frame()
	if(self.visible)then
		if(self.cor == nil or coroutine.status(self.cor) == 'dead')then
			self.cor = coroutine.create( function ()
											local cpu = Inspect.Addon.Cpu()[triggers.addonId]
											if(cpu == nil)then return end
											local tot = 0
											for i,j in pairs(cpu) do
												if (string.match(i,".*CPU.*") == nil) then
													tot = tot + j
												end
											end
											tot=tot*100
											CPUList.AddItem(self.CPUList, tot)
											coroutine.yield()
											for i = 1, gridCount do
												local v = self.CPUList.items[self.CPUList.last - (i - 1)]
												if(v ~= nil)then
													local l = (v / scale) * self.size.height -4
													--self.frame.grid[i]:SetPoint("BOTTOMLEFT",self.frame,"BOTTOMLEFT",i*4,-(l+5))
													self.frame.grid[i]:SetHeight(l)
													if(i % 10 == 0)then
														coroutine.yield()
													end
												end

											end
										end
				)
		end
		coroutine.resume(self.cor)
	end
end

CPUList = {}
function CPUList.new ()
	return {first = 0, last = -1, items = {}}
end

function CPUList.pushleft (list, value)
	local first = list.first - 1
	list.first = first
	list.items[first] = value
end

function CPUList.pushright (list, value)
	local last = list.last + 1
	list.last = last
	list.items[last] = value
end

function CPUList.popleft (list)
	local first = list.first
	if first > list.last then error("list is empty") end
	local value = list.items[first]
	list.items[first] = nil        -- to allow garbage collection
	list.first = first + 1
	return value
end

function CPUList.popright (list)
	local last = list.last
	if list.first > last then error("CPUList is empty") end
	local value = list.items[last]
	list.items[last] = nil         -- to allow garbage collection
	list.last = last - 1
	return value
end

function CPUList.AddItem(list, value)
	if(value > 3)then value = 3 end
	CPUList.pushright(list,value)
	while(list.last - list.first > gridCount)do
		CPUList.popleft(list)
	end
end

Command.Event.Attach(Triggers.Event.ContextCreated , CPUGraph.UISetup, "CPUGraph.UISetup")
Command.Event.Attach(Event.System.Update.Begin, CPUGraph.CPUUpdate, "CPUUpdate")

triggers.CPUGraph = CPUGraph.Create()
