--
--  Class ImageScroller

local addonData, triggers = ...

local ImageScroller = {}
ImageScroller.__index = ImageScroller

function ImageScroller.Create()
	local obj = {}
	setmetatable(obj,ImageScroller)
	obj:Init()
	return obj
end

function ImageScroller:Init()
	self.items = {}
	self.width = 200
	self.height = UIParent:GetHeight()
	self.description = "ImageScroller"
	self.scrollImageCount = table.getn(triggers.defaults.images)
	self.scrollHeight = (self.scrollImageCount * 200) + 100
	self.imageSize = 180
end

function ImageScroller:ClearImages()
	for i, image in ipairs(self.frame.ImageList.images) do
		image:SetVisible(false)
		image.fileName = nil
		image.imageResource = nil
		image.ItemSelected = nil
	end
end

function ImageScroller:SetTriggerImages()
	for i, imageInfo in ipairs(triggers.defaults.images) do
		local image = self.frame.ImageList.images[i]
		image:SetVisible(true)
		image:SetTexture(triggers.addonId,imageInfo)
		image.fileName = imageInfo
		image.imageResource = triggers.addonId
		image.ItemSelected = function (self)
			self.parent:SetVisible(false)
			triggers.ImageScroller:ImageSelected(self.imageResource, self.fileName)
		end
	end
end

function ImageScroller:SetRiftImages()
	local filter = self.frame.header.filter:GetText()
	local _, searchValues = libAbilityCatalog.Catalog:GetCatalogItemStartingWith(filter,libTConstants.CatalogItemType.ALL, false)
	local uniqueImages = {}
	for _, ability in ipairs(searchValues)do
		if(ability.icon ~= nil and uniqueImages[ability.icon] == nil)then
			uniqueImages[ability.icon] = true
		end
	end
	local imageList = {}
	for abilityIcon, _ in pairs(uniqueImages)do
		table.insert(imageList,abilityIcon)
	end
	for i, image in ipairs(self.frame.ImageList.images) do
		if(#imageList >= i)then
			image:SetVisible(true)
			image:SetTexture("Rift",imageList[i])
			image.fileName = imageList[i]
			image.imageResource = "Rift"
			image.ItemSelected = function(self)
				self.parent:SetVisible(false)
				triggers.ImageScroller:ImageSelected(self.imageResource, self.fileName)
			end
		end
	end
end

function ImageScroller.UISetup(handle,parent)
	self = triggers.ImageScroller
	self.frame = UI.CreateFrame("SimpleScrollView",self.description, parent)
	self.frame:SetWidth(self.width)
	self.height = UIParent:GetHeight()
	self.frame:SetHeight(self.height)
	self.frame:SetMouseMasking("limited")
	self.frame:SetLayer(1)
	self.frame:SetVisible(false)
	self.frame:SetPoint("RIGHT", UIParent, "RIGHT")
	self.frame:SetBackgroundColor(0,0,0,.7)
	self.frame:SetShowScrollbar(true)
	self.frame:SetScrollbarWidth(20)
	self.frame:SetScrollInterval(self.imageSize)

	self.frame.ImageList = UI.CreateFrame("Frame",self.description .. "ImageList", self.frame)
	self.frame.ImageList:SetMouseMasking("limited")
	self.frame.ImageList:SetLayer(2)
	self.frame.ImageList:SetVisible(true)
	self.frame:SetContent(self.frame.ImageList)
	self.frame.ImageList.images = {}
	self.frame.ImageList:SetHeight(self.scrollHeight)
	local relPos = nil
	for i, image in ipairs(triggers.defaults.images) do
		img = UI.CreateFrame("Texture", self.description .. "ImageList_image" .. i, self.frame)
		img:SetWidth(self.imageSize)
		img:SetHeight(self.imageSize)
		img:SetMouseMasking("limited")
		img:SetLayer(3)
		img:SetVisible(false)
		img.parent = self.frame
		if (i == 1)then
			img:SetPoint("TOPCENTER", self.frame.ImageList, "TOPCENTER", 0, 90)
		else
			img:SetPoint("TOPCENTER", relPos, "BOTTOMCENTER", 0, 20)
		end
		function img.Event:LeftClick()
			if(self.ItemSelected)then
				self.ItemSelected(self)
			end
		end
		table.insert(self.frame.ImageList.images,img)
		relPos = img
	end

	self.frame.header = UI.CreateFrame("Frame", self.description .. "_Header", self.frame)
	self.frame.header:SetWidth(self.width)
	self.frame.header:SetBackgroundColor(0,0,0,1)
	self.frame.header:SetHeight(90)
	self.frame.header:SetLayer(4)
	self.frame.header:SetVisible(true)
	self.frame.header:SetPoint("TOPLEFT", self.frame, "TOPLEFT")

	self.frame.header.cancel = libTUI.Button(self.frame.header
									,triggers.addonId
									,"_Cancel"
									,{width = 42, height=30}
									,"TOPRIGHT", self.frame.header, "TOPRIGHT"
									,-25
									,5
									,"\\images\\remove.png"
									,"\\images\\removeOver.png"
									,false)

	self.frame.header.none = libTUI.Button(self.frame.header
									,triggers.addonId
									,"_Cancel"
									,{width = 80, height=35}
									,"BOTTOMCENTER", self.frame.header, "BOTTOMCENTER"
									,0
									,-10
									,"\\images\\none.png"
									,"\\images\\noneOver.png"
									,false)

	self.frame.header.none.Event.LeftClick = function()
		triggers.ImageScroller.frame:SetVisible(false)
		triggers.ImageScroller:ImageSelected()
	end

	self.frame.header.cancel.is = self
	function self.frame.header.cancel.Event:LeftClick()
		self.is:Show(false)
	end

	self.frame.header.showRift = libTUI.CheckBox(self.frame.header
												,"_showRift"
												,"Rift Images"
												,100
												,"TOPLEFT", self.frame.header, "TOPLEFT"
												,10
												,5)
	function self.frame.header.showRift.Event:CheckboxChange()
		triggers.ImageScroller:ClearImages()
		if (not self:GetChecked())then
			triggers.ImageScroller:SetTriggerImages()
			triggers.ImageScroller.frame.header.filter:SetVisible(false)
			triggers.ImageScroller.frame.header.filter.label:SetVisible(false)
			triggers.ImageScroller.frame.header.filter.background:SetVisible(false)

		else
			triggers.ImageScroller.frame.header.filter:SetVisible(true)
			triggers.ImageScroller.frame.header.filter.label:SetVisible(true)
			triggers.ImageScroller.frame.header.filter.background:SetVisible(true)
			triggers.ImageScroller:SetRiftImages()
		end
	end

	self.frame.header.filter = libTUI.Input( self.frame.header
											,"_Filter"
											,"filter"
											,self.width
											,"TOPLEFT"
											,self.frame.header.showRift
											,"BOTTOMLEFT"
											,10
											,5
											,self.width - 50
											,""
											,true)

	function self.frame.header.filter.Event:TextfieldChange()
		triggers.ImageScroller:ClearImages()
		triggers.ImageScroller:SetRiftImages()
	end

	self:SetTriggerImages()
	self.frame.header.filter:SetVisible(false)
	self.frame.header.filter.label:SetVisible(false)
	self.frame.header.filter.background:SetVisible(false)
end

function ImageScroller:ImageSelected(resource, image)
end

function ImageScroller:Show(show)
	self.height = UIParent:GetHeight()
	self.frame:SetHeight(self.height)
	self.frame:SetVisible(show)
end

triggers.ImageScroller = ImageScroller.Create()

Command.Event.Attach(Triggers.Event.ContextCreated , ImageScroller.UISetup, "ImageScroller.UISetup")