--
--  Class Trigger

local addonData, triggers = ...

triggers.Trigger = {}
local Trigger = triggers.Trigger
Trigger.__index = Trigger

function Trigger.Create()
    local obj = {}
    setmetatable(obj,Trigger)
    obj:Init()
    obj.id = libTUtility.uuid()
    return obj
end

function Trigger:Init()
    self.name = nil
    self.abilityId = nil
    self.type = libTConstants.CatalogItemType.BUFF
    self.disabled = false
    self.roles = bit.bor(libTConstants.Roles.DPS, libTConstants.Roles.TANK, libTConstants.Roles.HEALER, libTConstants.Roles.SUPPORT)
    self.timer = false
    self.time = nil
    self.stacker = false
    self.stacks = nil
    self.selfCast = false
    self.buffState = libTConstants.BuffState.MISSING
    self.unit = libTConstants.Unit.TARGET
    self.relation = libTConstants.Relation.FRIENDLY
    self.abilityState = libTConstants.AbilityState.READY
    self.encounterId = nil
    self.stackCompare = -1
    self.timerCompare = -1
    self.text = nil
    self.showSpark = true
    self.color = { 1, 1, 1}
    self.filtered = true
    self.abilityNew = nil
    self.opacity = 100
end

function Trigger:toString()
    local disabled = self.disabled and "x" or ""
    local roles = libTConstants.Roles.toShortString(self.roles)
    local type = libTConstants.CatalogItemType.toString(self.type)
    local unit = libTConstants.Unit.toString(self.unit)
    local unitRelation = libTConstants.Relation.toString(self.relation)
    local buffState = libTConstants.BuffState.toString(self.buffState)
    local abilityState = libTConstants.AbilityState.toString(self.abilityState)
    local selfCast = self.selfCast and "self" or "anyone"
    local stacks = ""
    if(self.stacker and (self.stacks or 0) > 0)then
        if self.stackCompare > 0 then
            stacks = string.format("with >= %s stacks",self.stacks)
        else
            stacks = string.format("with <= %s stacks",self.stacks)
        end
    end
    local timer = ""
    if(self.timer)then
        if self.timerCompare > 0 then
            timer = string.format(" with >= %ss remaining",self.time)
        else
            timer = string.format(" with <= %ss remaining",self.time)
        end
    end
    local result = ""

    local whom = ""

    if self.unit == libTConstants.Unit.PLAYER then
        if self.type == libTConstants.CatalogItemType.BUFF then
            if self.buffState == libTConstants.BuffState.MISSING then
                whom = "when you are missing"
            else
                whom = "when you have"
            end
        else
            whom = "when you are"
        end
    else
        if self.type == libTConstants.CatalogItemType.BUFF then
            if self.buffState == libTConstants.BuffState.MISSING then
                whom = string.format("when your %s %s is missing",unitRelation,unit)
            else
                whom = string.format("when your %s %s has",unitRelation,unit)
            end
        else
            whom = string.format("when your %s %s is",unitRelation,unit)
        end
    end

    if(self.type == libTConstants.CatalogItemType.BUFF)then
        result = string.format("%s(%s) %s buff '%s' cast by %s %s %s",disabled,roles,whom,self.name,selfCast,stacks,timer)
    elseif (self.type == libTConstants.CatalogItemType.CASTBAR)then
        result = string.format("%s(%s) %s casting '%s' %s",disabled,roles,whom,self.name,timer)
    elseif (self.type == libTConstants.CatalogItemType.ABILITY)then
        result = string.format("%s(%s) when your ability '%s' is %s %s",disabled,roles,self.name,abilityState,timer)
    elseif (self.type == libTConstants.CatalogItemType.RESOURCE)then

    end
    if(self.encounterId ~= nil)then
        local encounterDetails = libEncounters.EncounterManager:GetEncounter(self.encounterId)
        if(encounterDetails)then
            result = string.format("%s when fighting %s",result,  encounterDetails.name)
        end
    end
    return libTUtility.Trim(result)
end

function Trigger:Serialize()
    local section = {}
    section.name = self.name
    section.abilityId = self.abilityId
    section.type = self.type
    section.disabled = self.disabled
    section.roles = self.roles
    section.timer = self.timer
    section.time = tonumber(self.time)
    section.stacker = self.stacker
    section.stacks = tonumber(self.stacks)
    section.selfCast = self.selfCast
    section.buffState = self.buffState
    section.unit = self.unit
    section.relation = self.relation
    section.abilityState = self.abilityState
    section.encounterId = self.encounterId
    section.stackCompare = self.stackCompare
    section.timerCompare = self.timerCompare
    section.id = self.id
    section.showSpark = self.showSpark
    section.color = { self.color[1], self.color[2], self.color[3] }
    section.filtered = self.filtered
    section.abilityNew = self.abilityNew
    if(self.text ~= nil)then
        self.text = libTUtility.Trim(self.text)
        if(string.len(self.text) <= 0)then
            self.text = nil
        end
    end
    if (self.opacity and self.opacity == 100) then
        self.opacity = nil
    end
    section.opacity = self.opacity
    section.text = self.text
    return section
end

function Trigger.Deserialize(section, t)
    if(t == nil)then
        t = Trigger.Create()
    end
    t.name =section.name
    t.abilityId = section.abilityId
    t.type = section.type
    t.disabled = section.disabled
    t.roles = section.roles
    t.timer = section.timer or false
    t.time = tonumber(section.time)
    t.stacker = section.stacker or false
    t.stacks = tonumber(section.stacks)
    t.selfCast = section.selfCast
    t.buffState = section.buffState
    t.filtered = section.filtered
    t.abilityNew = section.abilityNew
    if(t.filtered == nil)then
        t.filtered = true
    end
    t.unit = section.unit
    if (t.Unit == libTConstants.Unit.PLAYER)then
        t.relation = libTConstants.Relation.FRIENDLY
    else
        t.relation = section.relation
    end

    t.abilityState = section.abilityState
    t.encounterId = section.encounterId
    t.timerCompare = section.timerCompare or -1
    t.stackCompare = section.stackCompare or -1
    if(section.showSpark == nil)then
        t.showSpark = true
    else
        t.showSpark = section.showSpark
    end

    if(section.text ~= nil)then
        section.text = libTUtility.Trim(section.text)
        if(string.len(section.text) <= 0)then
            section.text = nil
        end
    end
    t.text = section.text
    if(not section.id)then
        t.id = libTUtility.uuid()
    else
        t.id = section.id
    end
    if(section.color ~= nil)then
        t.color = { section.color[1], section.color[2], section.color[3] }
    else
        t.color = {1,1,1}
    end
    t.opacity = section.opacity
    return t
end
