--
-- Class VirginVersion
--
--   Manages a list of VirginVersion

local triggers, toc = ...

local VirginVersion = {}
VirginVersion.__index = VirginVersion

function VirginVersion.Create()
	local obj = {}
	setmetatable(obj,VirginVersion)
	obj:Init()
	return obj
end

function VirginVersion:Init()
	self.versionInfo = {}
	self.version = 5
	self.description = "VirginVersion"
	self.welcomeText = "Please take time to configure the default notifications.\n <font color='#88AAFF'>Left-click-drag</font> Moves the notification\n <font color='#88AAFF'>Scroll-wheel</font> Resizes the notification\n <font color='#88AAFF'>Right-click</font> edits the notification\n <font color='#88AAFF'>Double-click</font> disables the notification"
	self.infoText = "You can change these images at any time"
	self.menuText = "Menu Icon:\n <font color='#88AAFF'>Left-click-drag</font> Move\n <font color='#88AAFF'>Left-click</font> Edit Mode\n <font color='#88AAFF'>Right-click</font> Menu"
	self.isVariablesLoaded = false
	self.isPlayerLoaded = false
end

function VirginVersion:SetupNotificationValues(x,y,image)
	local n =	UI.CreateFrame("Texture",self.description .. "_images", self.frame)
	n:SetPoint("CENTER",self.frame,"CENTER",x,y)
	n:SetTexture(triggers.addonId,image)
	n:SetWidth(80)
	n:SetHeight(80)
	n:SetVisible(true)
	return n
end
function VirginVersion:SetupNotificationText(n,x,y,text)
	local t =	UI.CreateFrame("Text",self.description .. "_texts", n)
	t:SetPoint("CENTER",n,"CENTER",x,y)
	t:SetText(text)
	t:SetWidth(80)
	t:SetHeight(60)
	t:SetFontSize(17)
	t:SetVisible(true)

	return t
end

function VirginVersion.OnPlayerLoaded()
	local self = triggers.VirginVersion
	self.isPlayerLoaded = true
	self:FirstTime()
end

function VirginVersion.OnVariablesLoaded()
	local self = triggers.VirginVersion
	TRIGGER_VERSION = TRIGGER_VERSION or {}
	self.versionInfo = TRIGGER_VERSION
	self.isVariablesLoaded = true
	self:FirstTime()
end

function VirginVersion.UISetup()
	local self = triggers.VirginVersion
	self.isContextCreated = true
	self:FirstTime()
end

function VirginVersion:FirstTime()
	if(self.isPlayerLoaded and self.isVariablesLoaded and self.isContextCreated)then
		local mVersion = self.versionInfo.version or 0
		if mVersion == 0 then
			Command.Event.Detach(libAbilityCatalog.Event.Loaded, VirginVersion.FirstTime)
			print("First time Setup")
			self.parent = triggers.BWMain.UIContext
			self.frame = UI.CreateFrame("SimpleWindow","Welcome", self.parent)
			self.frame:SetMouseMasking("limited")
			self.frame:SetLayer(101)
			self.frame.config = self
			self.frame:SetPoint("CENTER",UIParent,"CENTER")
			self.frame:SetVisible(true)
			self.frame:SetCloseButtonVisible(true)
			self.frame.welcomeText = UI.CreateFrame("Text",self.description .. "_text", self.frame)
			self.frame.welcomeText:SetPoint("TOPLEFT",self.frame,"TOPLEFT",20,60)
			self.frame.welcomeText:SetVisible(true)
			self.frame.welcomeText:SetText(self.welcomeText,true)
			self.frame.welcomeText:SetFontSize(15)

			self.frame.infoText = UI.CreateFrame("Text",self.description .. "_text", self.frame)
			self.frame.infoText:SetPoint("BOTTOMLEFT",self.frame,"BOTTOMLEFT",20,-20)
			self.frame.infoText:SetVisible(true)
			self.frame.infoText:SetText(self.infoText,true)
			self.frame.infoText:SetFontSize(15)

			self.frame.Debilitated = self:SetupNotificationValues(140,-70,"\\auras\\Zipper.png")
			self.frame.Fight       = self:SetupNotificationValues(100,20,"\\auras\\fight.png")
			self.frame.Interupt    = self:SetupNotificationValues(00,-20,"\\auras\\yell.png")
			self.frame.Move        = self:SetupNotificationValues(-100,-60,"\\auras\\move.png")
			self.frame.Purge       = self:SetupNotificationValues(-150,30,"\\auras\\purge.png")
			self.frame.Rooted      = self:SetupNotificationValues(150,80,"\\auras\\lock.png")
			self.frame.Shield      = self:SetupNotificationValues(-40,70,"\\auras\\shield.png")
			self.frame.Taunt       = self:SetupNotificationValues(-140,130,"\\auras\\taunt.png")

			self:SetupNotificationText(self.frame.Debilitated,-50,0,"(target)\nDebilitated")
			self:SetupNotificationText(self.frame.Fight,-40,40,"Increased\n Damage")
			self:SetupNotificationText(self.frame.Move,-25,40,"Move")
			self:SetupNotificationText(self.frame.Purge,55,30,"Purge")
			self:SetupNotificationText(self.frame.Interupt,-40,40,"Interupt!")
			self:SetupNotificationText(self.frame.Rooted,10,60,"(target)\nRooted")
			self:SetupNotificationText(self.frame.Shield,60,55,"Shield")
			self:SetupNotificationText(self.frame.Taunt,0,60,"Taunt")


			self.frame.MenuButton =	UI.CreateFrame("Texture",self.description .. "_images", self.frame)
			self.frame.MenuButton:SetPoint("CENTER",self.frame,"CENTER",-60,150)
			self.frame.MenuButton:SetTexture(triggers.addonId,"\\images\\button.png")
			self.frame.MenuButton:SetWidth(40)
			self.frame.MenuButton:SetHeight(40)
			self.frame.MenuButton:SetVisible(true)

			self.frame.menuText = UI.CreateFrame("Text",self.description .. "_menutext", self.frame)
			self.frame.menuText:SetPoint("TOPLEFT",self.frame.MenuButton,"TOPRIGHT",0,0)
			self.frame.menuText:SetVisible(true)
			self.frame.menuText:SetText(self.menuText,true)
			self.frame.menuText:SetFontSize(15)

			function self.frame.Event:Close()
				local self = triggers.VirginVersion
				self.versionInfo.version = self.version
			end

			local sn = triggers.defaults.sharedNotifications
			for _, name in ipairs(sn) do
				local section = triggers.defaults.defaultNotifications[name]
				triggers.Notifications:AddNotificationFromVariable(section)
			end
			triggers.MapIcon:EditMode()
		end
	end
end

triggers.VirginVersion = VirginVersion.Create()

Command.Event.Attach(Event.Addon.SavedVariables.Load.End, VirginVersion.OnVariablesLoaded, "VirginVersion.OnVariablesLoaded")
Command.Event.Attach(libAbilityCatalog.Event.Loaded, VirginVersion.OnPlayerLoaded , "VirginVersion.FirstTime")
Command.Event.Attach(Triggers.Event.ContextCreated , VirginVersion.UISetup, "VirginVersion.UISetup")
